/**
 * @file
 * @brief %Classifier linux implementation
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-24 14:18:49 +0100 (Die, 24 Feb 2004) $
 * $Revision: 647 $
 *
 * @since    Wed Mar 19 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: LinuxClassifier.cpp 647 2004-02-24 13:18:49Z rene $
 */

#include "Classifier.h"
#include <dlfcn.h>

Classifier::Classifier(const string &name, const classifierparams &params)
{
	getClassifier_t getClassifier = NULL;
	ca = NULL;

	string libName = "lib" + name + ".so";

	/* RTLD_NOW means that all external symbols of that library should be
	   resolved now - this allows us to catch missing external libraries
	   immediately */
	if ((dlHandle = dlopen(libName.c_str(), RTLD_NOW)) == NULL)
	{
		// error - unable to load library
		plog("Error loading classification algorithm library '%s': %s\n", libName.c_str(), dlerror());
		throw 1;
	}

	getClassifier = (getClassifier_t) dlsym(dlHandle, "getClassifier");
	// we can't check _only_ for dlsym return value because the symbol may return NULL
	if (dlerror() != NULL || getClassifier == NULL)
	{
		plog("Error locating symbol getAlgorithm in library\n");
		dlclose(dlHandle);
		throw 2;
	}

	ca = getClassifier(params);
	dll_init_log(dlHandle);
}

Classifier::~Classifier()
{
	dlclose(dlHandle);
}
