/**
 * @file
 * @brief Growing neural gas
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Mon Mar 3 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: Unit.h 612 2004-02-12 20:08:39Z harald $
 */

#ifndef _UNIT_H
#define _UNIT_H

#include "../../FeatureContainer/FeatureContainer.h"
#include "../../util/splaytree.h"

class Unit;

#if NO_OPTIMIZATIONS
# undef OPTIMIZATIONS_SPLAYTREE
# undef OPTIMIZATIONS_CACHE_SIMILARITY
# undef OPTIMIZATIONS_SPLIT_MERGE

# define OPTIMIZATIONS_SPLAYTREE 0
# define OPTIMIZATIONS_CACHE_SIMILARITY 0
# define OPTIMIZATIONS_SPLIT_MERGE 0
#endif

/**
 * @todo documentation
 */
template <class comparator>
class unittree : public splaytree<comparator>
{ public: unittree() : splaytree<comparator>(NULL) {} };

/** @todo documentation */
typedef map<Unit*, unsigned long> edge_map;

//@{
/** Unit collection */

#if OPTIMIZATIONS_SPLAYTREE
typedef unittree<Unit*> unit_list;
#else
typedef list<Unit*> unit_list;
#endif
//@}

/**
 * @todo documentation
 */
class Unit
{
	private:
		//@{
		/** @todo documentation */

		// this is ugly, ugly, ugly
		unit_list const*const all_units;

		static list<unsigned long> used_meta_ids;	// remember the meta ids which have already been used (just to speed things up a bit)
		unsigned long meta_id;

		featurevector position;
		edge_map edge;
								// the hash map keeps track of all edges and their age
		double error_short;				// accumulated short-term error
		double error_long;				// accumulated long-term error
		double error_insert;				// the inherited error at the time of insertion of this node
		double insertion_threshold;
		double youth;
		double radius;

		mutable bool traversing;
		mutable double helper_local_similarity;		// this is a cache for performance reasons

		void insertEdge(Unit* u);
		void eraseEdge(Unit* u);

		static unsigned long getNextMetaId();		// serach for the next unused meta id

		void movePositionTowards(const featurevector *sample, bool isWinner);
		void setMetaId(unsigned long id);
		void updateMetaId();
		bool isConnectedHelper(Unit* u) const;		// used by isConnected
		bool isConnected(Unit* u) const;		// this should be used to check if a unit is connected to another one
		void updateRadius();
		//@}

	public:
		/** @todo documentation */
		Unit(unit_list const*const units, const featurevector pos);
		/** @todo documentation */
		Unit(unit_list const*const units, const featurevector pos, unsigned long id);
		/** Destructor */
		virtual ~Unit();

		/** @todo documentation */
		Unit *splitUnit();

		/** @todo documentation */
		double getShortTermError() const;
		/** @todo documentation */
		void setShortTermError(double error);
		/** @todo documentation */
		double getLongTermError() const;
		/** @todo documentation */
		void setLongTermError(double error);
		//void decreaseErrors();				// decreases both the short and the long term errors according to their time constants (called for the winning unit)
		/** @todo documentation */
		void updateErrors(const featurevector *sample);

		/** @todo documentation */
		double getInsertionError() const;
		/** @todo documentation */
		void setInsertionError(double error);

		/** @todo documentation */
		double getInsertionThreshold() const;
		/** @todo documentation */
		void setInsertionThreshold(double threshold);
		/** @todo documentation */
		void updateInsertionThreshold();

		/** @todo documentation */
		double getYouth() const;
		/** @todo documentation */
		void setYouth(double youth);
		/** @todo documentation */
		void decreaseYouth();				// decreases the youth (called for the winning unit)

		/** @todo documentation */
		void decreaseInsertionThreshold();		// decreases the insertion threshold (called for the winning neuron)

		/** @todo documentation */
		const featurevector* getPosition() const;
		/** @todo documentation */
		double getDistance(const featurevector *sample) const; // returns the distance to the given sample, normlized to [0; 1]
		/** @todo documentation */
		void moveTowards(const featurevector *sample);

		/** @todo documentation */
		double getQualityLearning() const;		// compute the quality measure for learning at this unit (B^L)
		/** @todo documentation */
		double getQualityInsertion() const;		// compute the quality measure for insertion (B^I)
		/** @todo documentation */
		double getLearningRate(bool isWinner) const;	// compute the learning rate (called for the winning neuron and its neighbors)

		/** @todo documentation */
		void calculateLocalSimilarity() const;          // compute the local similarity of input weights (the average difference of this unit's position to allits neighbors)
														// this has to be called whenever a node (or one of its neighbors is moved) !

		/** @todo documentation */
		double getLocalSimilarity() const;              // return the pre-computed local similarity of input weights (the average difference of this unit's position to all its neighbors)

		/** @todo documentation */
		static size_t getNumMetaIds();
		/** @todo documentation */
		static unsigned long getMaxMetaId();
		/** @todo documentation */
		static void clearMetaIds();		// clear the list - only used for unserialization
		/** @todo documentation */
		static void reserveMetaId(unsigned long meta_id);	// reserve this meta id
		/** @todo documentation */
		static void releaseMetaId(unsigned long meta_id); // release a meta id so that it can be used again
		/** @todo documentation */
		unsigned long getMetaId() const;

		/** @todo documentation */
		void ageEdge(Unit* u);				// increments the age of a single adge
		/** @todo documentation */
		void ageEdges();				// increments the age of all connected edges by one (and also for the edges to this node, i.e. symmetrically)
		/** @todo documentation */
		void addEdge(Unit* u);
		/** @todo documentation */
		void removeEdge(Unit* u);
		/** @todo documentation */
		void setEdgeAge(Unit* u, unsigned long age);		// sets the edge age (symmetrically), only used for unserialize in GNG
		/** @todo documentation */
		edge_map::size_type getEdgeCount() const;	// returns the number of edges, i.e. the number of neighbors
		/** @todo documentation */
		edge_map* getEdges();			// return the edge map

		/** @todo documentation */
		string serialize() const;
		/** @todo documentation */
		size_t unserialize(const featurevector& features, const string data);

		/** @todo documentation */
		bool operator < (Unit *u);
		/** @todo documentation */
		bool operator > (Unit *u) { return u < this; }
		/** @todo documentation */
		bool operator != (Unit *u) { return (this < u) || (this > u); }
		/** @todo documentation */
		bool operator == (Unit *u) { return !(this != u); }
};

#endif /* _UNIT_H */
