/**
 * @file
 * @brief Persistant file reader
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-03-02 16:55:18 +0100 (Die, 02 Mär 2004) $
 * $Revision: 678 $
 *
 * @since    Mon Aug 11 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: PersistantReader.cpp 678 2004-03-02 15:55:18Z rene $
 */

#include "PersistantReader.h"
#include <scew.h>

PersistantReader::PersistantReader(const char *file)
{
	unsigned cnt;

	scew_parser *parser = scew_parser_create();
	scew_parser_load_file(parser, file);
	
	scew_tree *tree = scew_parser_tree(parser);
	scew_element *root = scew_tree_root(tree);

	cnt = scew_element_count(root);
	for (unsigned j=0; j < cnt; j++) {
		scew_element *node = scew_element_by_index(root, j);
		const XML_Char *feat = scew_attribute_value(scew_attribute_by_name(node, (XML_Char*) "id"));

		if (feat!= NULL) {
			unsigned cnt = scew_element_count(node);
			for (unsigned j=0; j < cnt; j++) {
				scew_element *item = scew_element_by_index(node, j);
				const XML_Char *id = scew_attribute_value(scew_attribute_by_name(item, (XML_Char*) "id"));
				const XML_Char *val = scew_element_contents(item);

				if (id != NULL) {
					if (val == NULL) val = (XML_Char*) "";
					data[string((char*) feat)][string((char*) id)] = string((char*) val);
				}
			}
		}
	}

	scew_tree_free(tree);
	scew_parser_free(parser);
}

map<string, string> *PersistantReader::getPersistantData(string feature)
{
	return &data[feature];
}
