/**
 * @file
 * @brief %Feature container
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Thu Apr 10 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: LinuxFeatureContainer.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "FeatureContainer.h"
#include <string.h>
#include <dlfcn.h>

FeatureProvider* FeatureContainer::loadProvider(const char *name, const providerparams &args)
{
	void* dlHandle;
	getProvider_t getProvider = NULL;

	char* libName = new char[strlen(name) + 6];
	strcpy(libName, "lib");
	strcat(libName, name);
	strcat(libName, ".so");

	/* RTLD_NOW means that all external symbols of that library should be
	   resolved now - this allows us to catch missing external libraries
	   immediately */
	if ((dlHandle = dlopen(libName, RTLD_NOW)) == NULL)
	{
		// error - unable to load library
		plog("Error loading library '%s': %s\n", libName, dlerror());
		return NULL;
	}
	delete libName;

	getProvider = (getProvider_t) dlsym(dlHandle, "getProvider");
	// we can't check _only_ for dlsym return value because the symbol may return NULL
	if (dlerror() != NULL || getProvider == NULL)
	{
		dlclose(dlHandle);
		return NULL;
	}
	
	dll_init_log(dlHandle);

	handle.push_back(dlHandle);

	return getProvider(args);
}

void FeatureContainer::freeProvider(void *provider)
{
	dlclose(provider);
}
