/**
 * @file
 * @brief %Feature container
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Thu Apr 10 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: LoggingFeatureContainer.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "LoggingFeatureContainer.h"
#include "../ConfigReader/ConfigReader.h"
#include "../util/featurevector.h"

LoggingFeatureContainer::LoggingFeatureContainer(ConfigReader *config) : FeatureContainer(config)
{
	map<string, string> globals = (*config->getGlobals());
	out = fopen(globals["features_log"].c_str(), "a");

	if (out != NULL) {
		fputs("[init]", out);
		for (featuremap::iterator iter = features.begin(); iter != features.end(); iter++) {
			fprintf(out, "%s;", (*iter).first.c_str());
		}

		fputc('\n', out);
		fflush(out);
	}
}

LoggingFeatureContainer::~LoggingFeatureContainer()
{
	if (out != NULL) fclose(out);
}

void LoggingFeatureContainer::nextSample()
{
	FeatureContainer::nextSample();

	if (out != NULL) {
		const featurevector *f = FeatureContainer::getSampleVector();

		fputs(serializeFeatureVector(*f).c_str(), out);
		fputc('\n', out);
		fflush(out);
	}
}
