/**
 * @file
 * @brief %Feature container
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Thu Apr 10 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: ReplayFeatureContainer.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "ReplayFeatureContainer.h"
#include "../ConfigReader/ConfigReader.h"
#include "../util/featurevector.h"

ReplayFeatureContainer::ReplayFeatureContainer(ConfigReader *config) : FeatureContainer(config)
{
	map<string, string> globals = (*config->getGlobals());
	in = fopen(globals["features_log"].c_str(), "r");
	assert(in != NULL);
}

ReplayFeatureContainer::~ReplayFeatureContainer()
{
	fclose(in);
}

void ReplayFeatureContainer::nextSample()
{
	char buf[4096];

	/* free old features */
	for (featurevector::iterator iter1 = s.begin(); iter1 != s.end(); iter1++) {
		if (*iter1 != NULL) {
			delete (*iter1);
		}
	}
	s.clear();

	/// @todo care about init lines !

	// call fgets until something is read or EOF is reached
	while (!fgets(buf, sizeof(buf), in) || (buf[0] == '\0') || (strncmp(buf, "[init]", 6) == 0)) {
		if (feof(in)) {
			throw 1;
		}
	}

	featurevector f = FeatureContainer::getFeatureVector();
	// this will clone the objects from f into s if necessary
	unserializeFeatureVector(f, s, buf);
	for (featurevector::iterator iter2 = f.begin(); iter2 != f.end(); iter2++) {
		if ((*iter2) != NULL) {
			delete (*iter2);
		}
	}
}

const featurevector* ReplayFeatureContainer::getSampleVector() const
{
	return &s;
}

void ReplayFeatureContainer::resetFile()
{
	// why doesn't that work ?
	//in.seekg(0, ios::beg);
	//in.clear();
	fseek(in, 0, SEEK_SET);
}

long ReplayFeatureContainer::getFilesize()
{
	long length;
	fseek(in, 0, SEEK_END);
	length = ftell(in);
	fseek(in, 0, SEEK_SET);

	return length;
}

long ReplayFeatureContainer::getCurrentPos()
{
	return ftell(in);
}
