/**
 * @file
 * @brief %Feature container
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Thu Apr 10 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: WindowsFeatureContainer.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "FeatureContainer.h"

FeatureProvider* FeatureContainer::loadProvider(const char *name, const providerparams &args)
{
	HMODULE dll;
	getProvider_t getProvider = NULL;

#ifdef _WIN32_WCE
	size_t len = strlen(name) + 1;
	wchar_t *buffer = new wchar_t[mbstowcs(NULL, name, len)];
	mbstowcs(buffer, name, len);

	if ((dll = LoadLibrary(buffer)) == NULL) {
#else
	if ((dll = LoadLibrary(name)) == NULL) {
#endif

		LPTSTR lpMsgBuf;

		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			GetLastError(),
			0,
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
		);

		MessageBox(NULL, lpMsgBuf, TEXT("Error"), MB_OK|MB_ICONINFORMATION);
		LocalFree(lpMsgBuf);

#ifdef _WIN32_WCE
		delete buffer;
#endif

		return NULL;
	}

#ifdef _WIN32_WCE
	delete buffer;
#endif

	if ((getProvider = (getProvider_t) GetProcAddress(dll, TEXT("getProvider"))) == NULL) {
		FreeLibrary(dll);
		return NULL;
	}
	
	dll_init_log(dll);

	handle.push_back(dll);
	
	return getProvider(args);
}

void FeatureContainer::freeProvider(void *provider)
{
	FreeLibrary((HMODULE) provider);
}