/**
 * @file
 * @brief Active window feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-03-02 13:36:18 +0100 (Die, 02 Mär 2004) $
 * $Revision: 669 $
 *
 * @since    Wed Apr 9 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: ActiveWindow.cpp 669 2004-03-02 12:36:18Z rene-cvs $
 */
 	 
// for sprintf
#include <stdio.h>
#include "ActiveWindow.h"

EXPORT_FEATURE(ActiveWindowFeatureProvider);

#if _DEBUG_FEATURES
string ActiveWindowFeature::toString() const
{
	char buffer[256];
	sprintf(buffer, "ActiveWindowFeature: '%s', has code %s", getName().c_str(), serialize().c_str());
	return buffer;
}
#endif

ActiveWindowFeatureProvider::ActiveWindowFeatureProvider(providerparams &params) : FeatureProvider("ActiveWindow", &features) {
	windowValid = false;

	if (params.count("display") == 0) {
		display = "";
	} else {
		display = params["display"];
	}

	features.push_back("ActiveWindow");

	activeWindowFeature_maxlen = 0;
}

Feature* ActiveWindowFeatureProvider::getFeature(string name) const {
	if (name == "ActiveWindow") {
		return new ActiveWindowFeature(const_cast<stringcode*> (&activeWindowFeature_code), const_cast<long*> (&activeWindowFeature_maxlen));
	} else {
		return NULL;
	}
}

Feature* ActiveWindowFeatureProvider::getSample(string name) const {
	if (name == "ActiveWindow" && windowValid) {
		return new ActiveWindowFeature(const_cast<stringcode*> (&activeWindowFeature_code), const_cast<long*> (&activeWindowFeature_maxlen), window);
	} else {
		return NULL;
	}
}
