/**
 * @file
 * @brief Active window feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Wed Apr 9 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: ActiveWindowLinux.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "ActiveWindow.h"
#include <X11/Xlib.h>

void ActiveWindowFeatureProvider::nextSample(clock_t checkpoint)
{
	Display* disp;
	Window focusWindow;
	int revToReturn;
	char* windowName;

	(void) checkpoint;	// to get rid of unused variable warnings

	windowValid = false;

	// NULL defaults to the value of the DISPLAY environment variable
        if (display == "") {
		disp = XOpenDisplay(NULL);
	}
	else {
		disp = XOpenDisplay(display.c_str());
	}
	if (disp == NULL) {
		plog("ActiveWindowFeatureProvider failed to open display\n");
		return;
	}

	XGetInputFocus(disp, &focusWindow, &revToReturn);

	if (focusWindow == None) {
		// no window has the focus --> no sample
		window = "";
	} else if (focusWindow == PointerRoot) {
		// this is a special case ("focus follows mouse") - currently not handled
		window = "SpecialWindow (PointerRoot): FocusFollowsMouse";
	} else {
		// a real window - get the window title
		XFetchName(disp, focusWindow, &windowName);
		if (windowName != NULL) {
			// yes, the window indeed has been assigned a name that the window manager can display
			// this is a heuristic, valid for KDE's kwin - I am not sure about other window managers
			string windowLabel(windowName);
			size_t appNamePos = windowLabel.rfind(" - ") + 3;
			if (appNamePos < windowLabel.size()) {
				window = windowLabel.substr(appNamePos);
			} else {
				window = windowLabel;
			}
			XFree(windowName);
		}
	}

	XCloseDisplay(disp);

	windowValid = true;
}
