/**
 * @file
 * @brief Active window feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Wed Apr 9 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: ActiveWindowWindows.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "ActiveWindow.h"

#include <Tlhelp32.h>

#ifdef _WIN32_WCE
#define	CloseSnapshot(ph) CloseToolhelp32Snapshot(ph);
#else
/**
 * CloseHandle helper macro
 *
 * The Toolhelp32Snapshot handle has to be closed differently on WIN32 and
 * CE, this macro provides a unified way to close such handles.
 */
#define CloseSnapshot(ph) CloseHandle(ph);
#endif

void ActiveWindowFeatureProvider::nextSample(clock_t checkpoint) {
	DWORD pid;
    HANDLE ph;

	this->windowValid = false;

	HWND fgwnd = GetForegroundWindow();
	GetWindowThreadProcessId(fgwnd, &pid);

	if((ph = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, pid)) != (void *) -1) {
		PROCESSENTRY32 pe;

		pe.dwSize = sizeof(PROCESSENTRY32);

		if (Process32First(ph, &pe)) {
			do {
				if (pe.th32ProcessID == pid) {
#ifdef UNICODE
					char name[sizeof(pe.szExeFile) / 2 + 1];
					wcstombs(name, pe.szExeFile, sizeof(name));
					window = name;
#else
					window = pe.szExeFile;
#endif

					this->windowValid = true;
					CloseSnapshot(ph);

					return;
				}
			} while (Process32Next(ph, &pe));
		}

		CloseSnapshot(ph);
	} else {
		window = "";
	}
}