/**
 * @file
 * @brief Bluetooth feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-03-02 13:36:18 +0100 (Die, 02 Mär 2004) $
 * $Revision: 669 $
 *
 * @since    Tue Mar 18 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: Bluetooth.cpp 669 2004-03-02 12:36:18Z rene-cvs $
 */

// for sprintf
#include <stdio.h>
#include "Bluetooth.h"

#if _DEBUG_FEATURES
string BluetoothPeersFeature::toString() const
{
	string ret = "BluetoothPeers: '";
	for (unsigned int i = 0; i < getListValues().size(); i++) {
		ret += getListValues()[i];
		ret += ", ";
	}
	ret += "', has code ";
	ret += serialize().c_str();

	return ret;
}

string BluetoothNumPeersFeature::toString() const
{
	char buffer[64];
	sprintf(buffer, "BluetoothNumPeersFeature: %ld", getVal());
	return buffer;
}
#endif

BluetoothFeatureProvider::BluetoothFeatureProvider(providerparams &params) : FeatureProvider("Bluetooth", &features)
{
	(void) params;	// to get rid of unused variable warnings

	peerListValid = false;

	features.push_back("Peers");
	features.push_back("NumPeers");

	peersFeature_maxlen = 0;
#ifndef _SYMBIAN
	numPeersFeature_minval = numeric_limits<long>::max();
	numPeersFeature_maxval = numeric_limits<long>::min();
#else
	numPeersFeature_minval = KMaxTInt;
	numPeersFeature_maxval = KMinTInt;
#endif
}

BluetoothFeatureProvider::~BluetoothFeatureProvider()
{
}

Feature* BluetoothFeatureProvider::getSample(string name) const
{
	if (name == "Peers" && peerListValid) {
		return new BluetoothPeersFeature(const_cast<stringcode*> (&peersFeature_code), const_cast<long*> (&peersFeature_maxlen), &peerList);
	} else if (name == "NumPeers" && peerListValid) {
		return new BluetoothNumPeersFeature(const_cast<long*> (&numPeersFeature_minval), const_cast<long*> (&numPeersFeature_maxval), (unsigned) peerList.size());
	}

	return NULL;
}

Feature* BluetoothFeatureProvider::getFeature(string name) const
{
	if (name == "Peers") {
		return new BluetoothPeersFeature(const_cast<stringcode*> (&peersFeature_code), const_cast<long*> (&peersFeature_maxlen));
	} else if (name == "NumPeers") {
		return new BluetoothNumPeersFeature(const_cast<long*> (&numPeersFeature_minval), const_cast<long*> (&numPeersFeature_maxval));
	}

	return NULL;
}
