/**
 * @file
 * @brief Bluetooth feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-08-29 11:13:55 +0200 (Son, 29 Aug 2004) $
 * $Revision: 903 $
 *
 * @since    Tue Mar 18 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: BluetoothLinux.h 903 2004-08-29 09:13:55Z rene $
 */

#ifndef _BLUETOOTHLINUX_H
#define _BLUETOOTHLINUX_H

#include "Bluetooth.h"
#include "../../util/Thread.h"

/**
 * @todo documentation
 */
class BluetoothLinuxFeatureProvider : public BluetoothFeatureProvider, public Thread
{
	private:
		/**
		 * @todo documentation
		 */
		class L2Connection : Thread
		{
			public:
				//@{
				/** magic values from Linux l2ping .... */
				static const int ident = 200;
				static const int size = 20;
				//@}

				/** FeatureProvider */
				BluetoothLinuxFeatureProvider* prov;
				/** Target MAC address */
				const string mac;

				virtual void run() throw();

				/**
				 * Constructor
				 *
				 * @param prov Feature provider
				 * @param mac Hardware address of the target
				 */
				L2Connection(BluetoothLinuxFeatureProvider* prov, string mac);
		};

		//@{
		/** Provider properties */
		
		int device_id;
		unsigned int delay;
		unsigned int inquiryLength;
		bool flushInquiryCache;
		unsigned int maxpingtime;
		unsigned short minlinkquality;
		unsigned int l2_pingdelay;
		bool use_l2_connections;
		mutable Lock lockPeerList;
		stringvector l2connections;
		//@}

	public:
		/**
		 * Constructor
		 *
		 * @param params Map of initialization parameters
		 */
		BluetoothLinuxFeatureProvider(featureparams& params);
		/** Destructor */
		virtual ~BluetoothLinuxFeatureProvider() throw();
		
		virtual void run() throw();
		virtual Feature* getSample(string name) const;
};

#endif // _BLUETOOTHLINUX_H
