/**
 * @file
 * @brief Bluetooth feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Tue Mar 18 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: BluetoothWindows.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "Bluetooth.h"

/**
 * @name Driver Functions
 *
 * API functions of the Digianswer Bluetooth driver.
 */
//@{

/**
 * Initialize the Bluetooth stack
 */
static void (*BtInit) ();

/**
 * Start inquiry
 */
static void (*BtStart) ();

/**
 * Stop inquiry
 */
static void (*BtStop) ();

/**
 * Get hardware addresses in range
 *
 * @return An array of strings containing the hardware addresses
 */
static char** (*BtGetHardwareAddressesInRange) ();

/**
 * Free the array of strings
 *
 * @param addrList The array of strings
 */
static void (*BtFreeHardwareAddressesList) (char** addrList);
//@}

/**
 * Handle of the Bluetooth driver library
 */
static HMODULE dll = NULL;

EXPORT_FEATURE(BluetoothFeatureProvider)

/**
 * Library entry point
 */
BOOL APIENTRY DllMain(HANDLE hModule, DWORD  fdwReason, LPVOID lpReserved)
{
	switch (fdwReason) { 
		case DLL_PROCESS_ATTACH:
#ifdef _WIN32_WCE
			if ((dll = LoadLibrary(TEXT("widcommBluetoothAdapter"))) == NULL) return FALSE;
#else
			if ((dll = LoadLibrary(TEXT("dgaBluetoothManager"))) == NULL) return FALSE;
#endif
			if ((BtInit = (void(__cdecl*)()) GetProcAddress(dll, TEXT("init"))) == NULL) return FALSE;
			if ((BtStart = (void(__cdecl*)()) GetProcAddress(dll, TEXT("start"))) == NULL) return FALSE;
			if ((BtStop = (void(__cdecl*)()) GetProcAddress(dll, TEXT("stop"))) == NULL) return FALSE;
			if ((BtGetHardwareAddressesInRange = (char**(__cdecl*)()) GetProcAddress(dll, TEXT("getHardwareAddressesInRange"))) == NULL) return FALSE;
			if ((BtFreeHardwareAddressesList = (void(__cdecl*)(char**)) GetProcAddress(dll, TEXT("freeHardwareAddressesList"))) == NULL) return FALSE;

			BtInit();
			BtStart();

			library_initialize();
			break;

		case DLL_PROCESS_DETACH:
			library_finalize();

			if (dll != NULL) {
				BtStop();
				FreeLibrary(dll);
			}
			break;
	}

	return TRUE;
}

void BluetoothFeatureProvider::nextSample(clock_t checkpoint)
{
	peerListValid = false;
	peerList.clear();

	if ((BtGetHardwareAddressesInRange != NULL) && (BtFreeHardwareAddressesList != NULL)) {
		char **addrList = BtGetHardwareAddressesInRange();

		for (int i=0; (addrList != NULL) && (addrList[i] != NULL); i++) {
			peerList.push_back(addrList[i]);
		}
		
		BtFreeHardwareAddressesList(addrList);
	}

	peerListValid = true;
}