/**
 * @file
 * @brief GSM feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Mon May 12 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: GSM.h 612 2004-02-12 20:08:39Z harald $
 */

#ifndef _GSM_H
#define _GSM_H

#include "../AbstractString.h"
#include "../../util/Thread.h"

/** Default delay (in seconds) between two GSM cell scans */
#define GSM_SCANDELAY 30

/**
 * @todo documentation
 */
class FEATURE_EXPORT_CLASS GSMCellFeature : public AbstractStringFeature
{
	friend class GSMFeatureProvider;

	protected:
		/**
		 * %Feature constructor
		 *
		 * This constructor initializes the feature as prototypes
		 * value. For creating a specific sample of a feature, the
		 * sample constructor should be used.
		 *
		 * @param code A reference to a static variable (it must be
		 *             same for all created objects of _one_ feature)
		 *             for persistantly storing the code table.
		 * @param maxlen A reference to a static variable (it must be
		 *               same for all created objects of _one_ feature)
		 *               for persistantly storing the maximum string length.
		 */
		GSMCellFeature(stringcode* code, long* maxlen) : AbstractStringFeature(AbstractStringFeature::None, code, maxlen) {}

		/**
		 * Sample constructor
		 *
		 * This constructor initializes the feature with a specific
		 * value and should thus be used for samples.
		 *
		 * @param code A reference to a static variable (it must be
		 *             same for all created objects of _one_ feature)
		 *             for persistantly storing the code table.
		 * @param maxlen A reference to a static variable (it must be
		 *               same for all created objects of _one_ feature)
		 *               for persistantly storing the maximum string length.
		 * @param cellid The string representation of the sample value.
		 */
		GSMCellFeature(stringcode* code, long* maxlen, string cellid) : AbstractStringFeature(AbstractStringFeature::None, code, maxlen, cellid) {}

	public:
		virtual const string getName() const { return "GSM.CellID"; }

#if _DEBUG_FEATURES
		virtual string toString() const;
#endif
};


/**
 * @todo documentation
 */
class FEATURE_EXPORT_CLASS GSMFeatureProvider : public FeatureProvider, private Thread
{
	private:
		//@{
		/** @todo documentation */
		stringvector features;
		int delay;
		stringcode gsmCellFeature_code;
		long gsmCellFeature_maxlen;

		// these have to be implemented for each OS
		void init(featureparams& params);
		virtual bool openSerialPort(string serialport);
		virtual void closeSerialPort();
		virtual bool writeLine(string line);
		virtual string readLine();
		//@}

	protected:
		//@{
		/** @todo documentation */
		string cellId;
		bool cellIdValid;
		mutable Lock lockCellId;

		string serialPort;
		//@}

	public:
		/**
		 * Constructor
		 *
		 * @param params Map of initialization parameters
		 */
		GSMFeatureProvider(providerparams& params);
		/** Destructor */
		virtual ~GSMFeatureProvider();

		virtual void run();

		virtual Feature* getFeature(string name) const;
		virtual Feature* getSample(string name) const;
		virtual void nextSample(clock_t checkpoint);
};

#endif // _GSM_H
