/**
 * @file
 * @brief Network feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-06-08 11:38:41 +0200 (Die, 08 Jun 2004) $
 * $Revision: 809 $
 *
 * @since    Mon Jul 7 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: Network.cpp 809 2004-06-08 09:38:41Z rene $
 */

//@{
/** Default properties */
#define SYSTEM_COMMAND_1 "nmap -sP -T4 "
#define SYSTEM_COMMAND_2 " | grep \"Host .* appears to be up.\" | cut -d')' -f1 | cut -d'(' -f2"
#define DEFAULT_NETWORK "10.0.0.0/24"
//@}

#include "Network.h"

EXPORT_FEATURE(NetworkFeatureProvider);

NetworkFeatureProvider::NetworkFeatureProvider(featureparams &params) :
	SystemCommandStringListFeatureProvider("Network", "HostsUp", "",
		 &features, &clientsFeature_code, &clientsFeature_maxlen, params)

{
	string network;
	if (params.count("network") == 0)
		network = DEFAULT_NETWORK;
	else
		/// @todo check parameter !!
		network = params["network"];
		
#ifdef _DEBUG_FEATURES
	plog("Initializing Network Ping on network %s\n", network.c_str());
#endif

	systemCommand = SYSTEM_COMMAND_1 + network + SYSTEM_COMMAND_2;

	features.push_back("HostsUp");

	// start the system command thread
	this->start();
}

NetworkFeatureProvider::~NetworkFeatureProvider() throw()
{
	this->SystemCommandStringListFeatureProvider::~SystemCommandStringListFeatureProvider();
}
