/**
 * @file
 * @brief PHP feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Mon Sep 8 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: PHPWrapper.c 612 2004-02-12 20:08:39Z harald $
 */

#include "PHPCommon.h"

#ifdef _WINDOWS
# define ZEND_WIN32
# define PHP_WIN32
# define ZTS
#endif // _WINDOWS

/** disable Zend debugging */
#define ZEND_DEBUG 0

#include "php.h"
#include "php_ini.h"
#include "php_main.h"
#include "php_variables.h"

/** thread safe local storage */
static void ***tsrm_ls;

/** Module initializer */
PHP_MINIT_FUNCTION(intelligence)
{
	REGISTER_LONG_CONSTANT("NumericalDiscrete",	NumericalDiscrete,	CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("NumericalContinuous",	NumericalContinuous,	CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("StringNone",	StringNone,	CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("StringLevenshtein",	StringLevenshtein,	CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("StringListNone",	StringListNone,	CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("StringListLevenshtein",	StringListLevenshtein,	CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("Custom",	Custom,	CONST_CS | CONST_PERSISTENT);

	return SUCCESS;
}

/** PHP exported functions */
static function_entry intelligence_functions[] = {
	{ NULL, NULL, NULL }
};

/** PHP intelligence module */
static zend_module_entry php_intelligence_module = {
	STANDARD_MODULE_HEADER,
	"Intelligence",
	intelligence_functions,
	PHP_MINIT(intelligence),
	NULL,
	NULL,
	NULL,
	NULL,
	"0.1.0",
	STANDARD_MODULE_PROPERTIES
};

/** SAPI startup */
static int php_intelligence_startup(sapi_module_struct *sapi_module)
{
	if (php_module_startup(sapi_module, &php_intelligence_module, 1) == FAILURE) {
		return FAILURE;
	}
	return SUCCESS;
}

/** SAPI write */
static int sapi_intelligence_ub_write(const char *str, uint str_length TSRMLS_DC)
{
	return str_length;
}

/** SAPI flush */
static void sapi_intelligence_flush(void *server_context)
{
}

/** SAPI register variables */
static void sapi_intelligence_register_variables(zval *track_vars_array TSRMLS_DC)
{
	php_register_variable ("SERVER_SOFTWARE", "intelligence", track_vars_array TSRMLS_CC);
}

/** SAPI post data read */
static int sapi_intelligence_post_read(char *buf, uint count_bytes TSRMLS_DC)
{
	return 0;
}

/** SAPI cookies read */
static char* sapi_intelligence_read_cookies(TSRMLS_D)
{
	return NULL;
}

/** SAPI send headers */
static int sapi_intelligence_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
{
	return SAPI_HEADER_SENT_SUCCESSFULLY;
}

/** SAPI module */
static sapi_module_struct intelligence_sapi_module = {
	"intelligence",					/* name */
	"Intelligence PHP SAPI",		/* pretty name */

	php_intelligence_startup,		/* startup */
	php_module_shutdown_wrapper,	/* shutdown */

	NULL,							/* activate */
	NULL,							/* deactivate */

	sapi_intelligence_ub_write,		/* unbuffered write */
	sapi_intelligence_flush,		/* flush */
	NULL,							/* get uid */
	NULL,							/* getenv */

	php_error,						/* error handler */

	NULL,							/* header handler */
	sapi_intelligence_send_headers,	/* send headers handler */
	NULL,							/* send header handler */

	sapi_intelligence_post_read,	/* read POST data */
	sapi_intelligence_read_cookies,	/* read Cookies */

	sapi_intelligence_register_variables,	/* register server variables */
	NULL,							/* Log message */

	NULL,							/* Block interruptions */
	NULL,							/* Unblock interruptions */

	STANDARD_SAPI_MODULE_PROPERTIES
};

/** Load php runtime */
void init_php()
{
	intelligence_sapi_module.executable_location = "C:\\"; /// @todo fix that;

#ifdef ZTS
	tsrm_startup(1, 1, 0, NULL);
	tsrm_ls = (void ***) ts_resource(0);
#endif ZTS
	
	sapi_startup(&intelligence_sapi_module);
	sapi_module.startup(&intelligence_sapi_module);

	zend_first_try {
        /* Set some CLI defaults */
		SG(options) |= SAPI_OPTION_NO_CHDIR;
		zend_alter_ini_entry("html_errors", 12, "0", 1, PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
		zend_alter_ini_entry("max_execution_time", 19, "0", 1, PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);

		zend_uv.html_errors = 0; /* tell the engine we're in non-html mode */
		CG(interactive) = 0;
	} zend_catch {
	} zend_end_try();
}

/** Unload PHP runtime */
void shutdown_php()
{
	php_module_shutdown(TSRMLS_C);
	sapi_shutdown();
#ifdef ZTS
	tsrm_shutdown();
#endif
}

static zval call_function(const char *file, const char *func)
{
	zend_file_handle file_handle;
	zval function_name, retval;

	ZVAL_NULL(&retval);

	/* request startup */
	if (php_request_startup(TSRMLS_C) == FAILURE) {
		SG(headers_sent) = 1;
		SG(request_info).no_headers = 1;
		php_request_shutdown((void *) 0);

		return retval;
	}
	
	/* disable headers */
	SG(headers_sent) = 1;
	SG(request_info).no_headers = 1;
		
	file_handle.type = ZEND_HANDLE_FILENAME;
	file_handle.filename = file;
	file_handle.free_filename = 0;
	file_handle.opened_path = NULL;

	php_execute_script(&file_handle TSRMLS_CC);
	
	/* call userland */
	INIT_ZVAL(function_name);
	ZVAL_STRING(&function_name, func, 0);

	call_user_function(CG(function_table), NULL, &function_name, &retval, 0, NULL TSRMLS_CC);

	if (Z_TYPE(retval) == IS_STRING) {
		/**
		 * request shutdown would free the emalloc'ed string and spit out a warning, so
		 * we have to strndup the string first and free the emalloc'ed one to have the
		 * string available even after reqest shutdown.
		 */
		char *tmp = Z_STRVAL(retval);
		Z_STRVAL(retval) = strdup(Z_STRVAL(retval)); /// @todo strlen , Z_STRLEN(retval));
		efree(tmp);
	}
	
	php_request_shutdown((void *) 0);
	
	return retval;
}

/** Initialize feature */
PHPFeatureType php_init_feature(const char *file)
{
	zval ret = call_function(file, "init_feature");

	if (Z_TYPE(ret) == IS_LONG) {
		return (PHPFeatureType) Z_LVAL(ret);
	} else {
		return Error;
	}
}

/** Next sample */
void* php_next_sample(const char *file, PHPFeatureType type)
{
	static zval ret;
	ret = call_function(file, "next_sample");

	switch (type) {
		case NumericalDiscrete:
			convert_to_long(&ret);
			return &Z_LVAL(ret);
			
		case NumericalContinuous:
			convert_to_double(&ret);
			return &Z_DVAL(ret);
			
		case StringNone:
		case StringLevenshtein:
			convert_to_string(&ret);
			return &Z_STRVAL(ret);
			
		case StringListNone:
		case StringListLevenshtein:
			/// @todo foo
			return NULL;
			break;
			
		case Custom: {
			void *tmp = malloc(sizeof(zval));
			memcpy(tmp, &ret, sizeof(zval));
			return tmp;
		}

		default:
			return NULL;
			break;
	}
}

/** Move towards */
void* php_move_towards(const char *file, PHPFeatureType type, void *a, void *b)
{
	zval *param[2];

	ALLOC_ZVAL(param[0]);
	ALLOC_ZVAL(param[1]);

	INIT_PZVAL(param[0]);
	INIT_PZVAL(param[1]);

	switch (type) {
		case NumericalDiscrete:
			ZVAL_LONG(param[0], *((long *) a));
			ZVAL_LONG(param[1], *((long *) b));
			break;
			
		case NumericalContinuous:
			ZVAL_DOUBLE(param[0], *((double *) a));
			ZVAL_DOUBLE(param[1], *((double *) b));
			break;
			
		case StringNone:
		case StringLevenshtein:
			ZVAL_STRING(param[0], a, 1);
			ZVAL_STRING(param[1], b, 1);
			break;
			
		case StringListNone:
		case StringListLevenshtein:
			/// @todo foo
			return NULL;
			break;
			
		case Custom:
			*param[0] = *((zval *) a);
			*param[1] = *((zval *) b);

			/// @todo COPY CTOR
			break;

		default:
			ZVAL_NULL(param[0]);
			ZVAL_NULL(param[1]);
			break;
	}

	return NULL;
}

/** Get distance */
double php_get_distance(const char *file, PHPFeatureType type, void *a, void *b)
{
	zval *param[2];

	ALLOC_ZVAL(param[0]);
	ALLOC_ZVAL(param[1]);

	INIT_PZVAL(param[0]);
	INIT_PZVAL(param[1]);

	return 0.0;
}
