/**
 * @file
 * @brief Power feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-03-02 13:36:18 +0100 (Die, 02 Mär 2004) $
 * $Revision: 669 $
 *
 * @since    Mon Mar 10 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: Power.cpp 669 2004-03-02 12:36:18Z rene-cvs $
 */

#include "Power.h"

EXPORT_FEATURE(PowerFeatureProvider);

// Feature
PowerFeature::PowerFeature()
{
	plugged = (rand_double() > 0.5) ? true : false;
}

PowerFeature::PowerFeature(bool plugged)
{
	this->plugged = plugged;
}

double PowerFeature::getPosition() const
{
	return plugged ? 1.0 : 0.0;
}

double PowerFeature::getDistance(Feature *f) const
{
	PowerFeature *p = (PowerFeature *) f;

	return (this->plugged == p->plugged) ? 0.0 : 1.0;
}

void PowerFeature::moveTowards(Feature *f, double factor)
{
	/// @todo can we do this more deterministic ?
	if (rand_double() < factor) {
		this->plugged = ((PowerFeature*) f)->plugged;
	}
}

string PowerFeature::serialize() const
{
	if (plugged)
		return "1";
	else
		return "0";
}

void PowerFeature::unserialize(string value)
{
	if (value == "1")
		plugged = true;
	else if (value == "0")
		plugged = false;
	else {
		plog("Error on PowerFeature::unserialize - unknown state !\n");
		plugged = false;
	}
}

#if _DEBUG_FEATURES
string PowerFeature::toString() const
{
	string ret = "PowerFeature: ";
	ret += plugged ? "on-line" : "off-line";
	return ret;
}
#endif

Feature* PowerFeature::clone() const
{
	return new PowerFeature(plugged);
}

void PowerFeature::aggregate(aggregatelist )
{
}


PowerFeatureProvider::PowerFeatureProvider(providerparams &params) : FeatureProvider("Power", &features)
{
	(void) params;	// to get rid of unused variable warnings

	features.push_back("Plugged");
	this->pluggedValid = false;
}

Feature* PowerFeatureProvider::getFeature(string name) const
{
	if (name == "Plugged") {
		return new PowerFeature();
	}

	return NULL;
}

Feature* PowerFeatureProvider::getSample(string name) const
{
	if (name == "Plugged" && pluggedValid) {
		return new PowerFeature(plugged);
	}

	return NULL;
}
