/**
 * @file
 * @brief Power feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Mon Mar 10 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: Power.h 612 2004-02-12 20:08:39Z harald $
 */

#ifndef _POWER_H
#define _POWER_H

#include "../Feature.h"

/**
 * @todo documentation
 */
class FEATURE_EXPORT_CLASS PowerFeature : public Feature
{
	friend class PowerFeatureProvider;

	private:
		/** Indicates wheter the device is plugged or running on battery */
		bool plugged;

	protected:
		/**
		 * %Feature constructor
		 *
		 * This constructor initializes the feature with a random value
		 * and should thus only be used for creating e.g. prototypes
		 * of points in a clustering space. For creating a specific
		 * sample of a feature, the second constructor should be used.
		 */
		PowerFeature();

		/** 
		 * Sample constructor
		 *
		 * This constructor initializes the feature with a specific
		 * value and should thus be used for samples.
		 */
		PowerFeature(bool plugged);

	public:
		virtual double getPosition() const;
		virtual double getDistance(Feature *f) const;
		virtual void moveTowards(Feature *f, double factor);

		virtual string serialize() const;
		virtual void unserialize(string value);

		virtual Feature* clone() const;
		virtual void aggregate(aggregatelist samples);

		virtual FeatureType getType() const { return boolean; }
		virtual const string getName() const { return "Power.Plugged"; }
		
#if _DEBUG_FEATURES
		virtual string toString() const;
#endif
};

/**
 * @todo documentation
 */
class FEATURE_EXPORT_CLASS PowerFeatureProvider : public FeatureProvider
{
	private:
		/** List of features */
		stringvector features;

		//@{
		/** @todo documentation */
		bool plugged;
		bool pluggedValid;
		//@}

	public:
		/**
		 * Constructor
		 *
		 * @param params Map of initialization parameters
		 */
		PowerFeatureProvider(providerparams &params);

		virtual Feature* getFeature(string name) const;
		virtual Feature* getSample(string name) const;
		virtual void nextSample(clock_t checkpoint);
};

#endif // _POWER_H
