/**
 * @file
 * @brief Power feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-01-15 15:06:13 +0100 (Don, 15 Jän 2004) $
 * $Revision: 505 $
 *
 * @since    Mon Mar 10 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: PowerLinux.cpp 505 2004-01-15 14:06:13Z harald $
 */

#include <stdio.h>

#include "Power.h"

void PowerFeatureProvider::nextSample(clock_t checkpoint)
{
  FILE* f;

	(void) checkpoint;	// to get rid of unused variable warnings

  pluggedValid = false;

  // first check if APM or ACPI support is available
  if ((f=fopen("/proc/apm", "r")) != NULL) {
    // APM is available
    float ver1, ver2;
    unsigned int a, onPowerI, c, d;
    int percentBattery, remainingTime;

    if (fscanf(f, "%f %f %x %x %x %x %d%% %d",
      &ver1, &ver2, &a, &onPowerI, &c, &d, &percentBattery, &remainingTime) != 8) {
      // what to do here ? exceptions ??
      fclose(f);
      return;
    }

    plugged = (onPowerI == 1);
    pluggedValid = true;
  }
  // kernel 2.4.x ACPI
  else if((f=fopen("/proc/acpi/ac_adapter/0/status", "r")) != NULL ||
  // kernel 2.5.x ACPI
					(f=fopen("/proc/acpi/ac_adapter/ACAD/state", "r")) != NULL) {
    // ACPI is available
    char line[80];

    if (fgets(line, 80, f) == NULL) {
			fclose(f);
			return;
		}

    plugged = (strstr(line, "on-line") != NULL);
    pluggedValid = true;
  }
  else {
    // unable to query
    // exceptions ??
    return;
  }

  fclose(f);
}
