/**
 * @file
 * @brief System command string feature class declaration
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-23 12:26:48 +0100 (Mon, 23 Feb 2004) $
 * $Revision: 638 $
 *
 * @since    Mon Jul 7 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: SystemCommandStringList.h 638 2004-02-23 11:26:48Z rene $
 */

#ifndef _SYSTEMCOMMANDSTRINGLIST_H
#define _SYSTEMCOMMANDSTRINGLIST_H

#include "AbstractString.h"
#include "../util/Thread.h"

/**
 * @todo documentation
 */
class SystemCommandStringListFeature: public AbstractStringListFeature
{
	private:
		/** @todo documentation */
		const string providerName;
		/** @todo documentation */
		const string featureName;

	public:
		/** @todo documentation */
		SystemCommandStringListFeature(stringcode *code, long* maxlen, const stringvector *names, const string providerName, const string featureName) :
			AbstractStringListFeature(code, maxlen, names), providerName(providerName), featureName(featureName) {}
		/** @todo documentation */
		SystemCommandStringListFeature(stringcode *code, long* maxlen, const string providerName, const string featureName) :
			AbstractStringListFeature(code, maxlen), providerName(providerName), featureName(featureName) {}

		virtual const string getName() const;

#if _DEBUG_FEATURES
		virtual string toString() const;
#endif
};

/// This class returns a feature out of any system command that can return a
/// newline-separated list of strings.
class SystemCommandStringListFeatureProvider : public FeatureProvider, public Thread
{
	private:
		//@{
		/** @todo documentation */
		const string providerName;
		const string featureName;
		stringcode* stringCodeList;
		long* maxlen;
		int delay;
		mutable Lock lockStringList;

		string executeSystemCommand();
		//@}

	protected:
		/// Warning: change with care and only when necessary !
		string systemCommand;

	public:
		/** @todo documentation */
		static void initialize(string name);

		/// this->start has to be called in the subclass constructor
		SystemCommandStringListFeatureProvider(
			string featureProviderName,
			string featureName,
			string systemCommand,
			stringvector* featureList,
			stringcode* stringCodeList,
			long* maxlen,
			featureparams& params);
		/** Destructor */
		virtual ~SystemCommandStringListFeatureProvider() throw();

		virtual Feature* getFeature(string name) const;
		virtual Feature* getSample(string name) const;
		virtual void nextSample(clock_t checkpoint);

		virtual void run() throw();

	protected:
		//@{
		/** @todo documentation */
		stringvector stringList;
		bool stringListValid;
		//@}
};

#endif

