/**
 * @file
 * @brief Time feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-03-01 16:47:51 +0100 (Mon, 01 Mär 2004) $
 * $Revision: 666 $
 *
 * @since    Wed Apr 9 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: TimeFeature.h 666 2004-03-01 15:47:51Z rene $
 */

#ifndef _TIMEFEATURE_H
#define _TIMEFEATURE_H

#include "../Feature.h"

/**
 * @todo documentation
 */
class FEATURE_EXPORT_CLASS TimeFeature : public PersistantFeature
{
	friend class TimeFeatureProvider;

	public:
		/** Possible kinds of time features */
		typedef enum {Timestamp, Second, Minute, Hour, Day, Weekday, Yearday, Month, Year} FeatureKind;

	protected:
		/** @todo documentation */
		TimeFeature(FeatureKind kind, time_t* minval, time_t* maxval);
		/** @todo documentation */
		TimeFeature(FeatureKind kind, time_t* minval, time_t* maxval, time_t timestamp);

	public:
		virtual double getPosition() const;
		virtual double getDistance(Feature *f) const;
		virtual void moveTowards(Feature *f, double factor);
		virtual string serialize() const;
		virtual void unserialize(string value);

		virtual featureparams write() const;
		virtual void read(featureparams *param);

		virtual Feature* clone() const;
		virtual void aggregate(aggregatelist samples);

		virtual FeatureType getType() const { return numerical_discrete; }
		virtual const string getName() const;

		/** @return The value of the feature. */
		virtual time_t getVal() const { return timestamp; }

		const time_t getMin() const { return *minval; }
		const time_t getMax() const { return *maxval; }

#if _DEBUG_FEATURES
		virtual string toString() const;
#endif

	private:
		//@{
		/** @todo documentation */

		time_t* minval;
		time_t* maxval;

		time_t timestamp;
		FeatureKind feature;
		//@}
};

/**
 * @todo documentation
 */
class FEATURE_EXPORT_CLASS TimeFeatureProvider : public FeatureProvider
{
	private:
		//@{
		/** @todo documentation */

		time_t timeFeature_maxval;
		time_t timeFeature_minval;
		stringvector features;
		map<string, TimeFeature::FeatureKind> featuremap;

		time_t timestamp;
		//@}

	public:
		/** @todo documentation */
		TimeFeatureProvider(providerparams &params);

		virtual Feature* getFeature(string name) const;
		virtual Feature* getSample(string name) const;
		virtual void nextSample(clock_t checkpoint);
};

#endif // _TIMEFEATURE_H
