/**
 * @file
 * @brief Video feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-06-16 17:22:00 +0200 (Mit, 16 Jun 2004) $
 * $Revision: 825 $
 *
 * @since    Wed Apr 9 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: Video.h 825 2004-06-16 15:22:00Z rene $
 */

#ifndef _VIDEO_H
#define _VIDEO_H

#include "../../util/Thread.h"
#include "../Numerical.h"

class FEATURE_EXPORT_CLASS VideoMotionFeature : public NumericalContinuousFeature
{
	friend class VideoFeatureProvider;
	
	protected:
		VideoMotionFeature(double* minval, double* maxval) : NumericalContinuousFeature(minval, maxval) {}
		VideoMotionFeature(double* minval, double* maxval, double val) : NumericalContinuousFeature(minval, maxval, val) {}

	public:
		virtual const string getName() const { return "Video.Motion"; }

#if _DEBUG_FEATURES
		virtual string toString() const;
#endif
};


/**
 * Provider for the video features
 */
class FEATURE_EXPORT_CLASS VideoFeatureProvider : public FeatureProvider, public Thread
{
	private:
		/**
		 * @todo documentation
		 */
		struct SampleData {
			double average;
			unsigned long average_counter;
			
			/**
			 * Constructor
			 */
			SampleData() {
				average_counter = 0;
				average = 0.0;
			}
		};

		/** List of features */
		stringvector features;

		//@{
		/** Feature properties */
		string deviceName;
		int height, width, frequency;

		double videoMotionFeature_minval;
		double videoMotionFeature_maxval;

		double curMotion;
		mutable Lock lockCurMotion;
		
		// helper methods;
		char* v4l_start();
	
	public:
		VideoFeatureProvider(providerparams &params);
		virtual ~VideoFeatureProvider();

		virtual Feature* getFeature(string name) const;
		virtual Feature* getSample(string name) const;
		virtual void nextSample(clock_t checkpoint);
		
		virtual void run();
};

#endif // _VIDEO_H
