
/* motion.h */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>






/* video4linux stuff */
#define NORM_DEFAULT    0
#define NORM_PAL        0
#define NORM_NTSC       1
#define NORM_SECAM      2
#define NORM_PAL_NC	3
#define IN_DEFAULT      8
#define IN_TV           0
#define IN_COMPOSITE    1
#define IN_COMPOSITE2   2
#define IN_SVIDEO       3


struct video_dev {
	int fd;
	char *device;
	int input;
	int width;
	int height;
	long freq;

	pthread_mutex_t mutex;
	pthread_mutexattr_t attr;
	int owner;
	int frames;
	
	/* Device type specific stuff: */
	
	/* v4l */
	int v4l_fmt;
	int v4l_read_img;
	unsigned char *v4l_buffers[2];
	int v4l_curbuffer;
	int v4l_maxbuffer;
	int v4l_bufsize;
};


/* alg.h */
struct coord {
	int x;
	int y;
};

struct segment {
	struct coord coord;
	int width;
	int height;
	int open;
	int count;
};

struct images {
	unsigned char *new2;
	unsigned char *ref;
	unsigned char *out;
	unsigned char *mask;
	int width;
	int height;
	int type;
	int size;
	int motionsize;
};

/*
	these used to be global variables but now each thread will have its
	own context
 */
struct context {
	struct images imgs;

	int threadnr;
	int daemon;

	int snapshot;
	int makemovie;
	int finish;
	int alarmtime;

	int event_nr;
	int prev_event;

	int shots;
	struct tm *currenttime;
	time_t currenttimep;
	time_t lasttime;
	time_t eventtime;
	time_t lastshot;

	char timestamp[255];
	char datestamp[255];

	int lastrate;
	int filesc;
	int filesmc;
	char **files;
	char **filesm;
	int *filesrate;
	int *filesmrate;
	int moved;
	int switched;
};


/* video functions, video.c */
char *vid_next (struct context *, int dev, char *map, int width, int height);
void vid_autobright (struct config *, int dev, char *image, int height, int width);
char *vid_keepalive(struct context *, struct images *imgs, int dev, int pipe, int mpipe);

struct coord alg_locate_center (struct images *, int width, int height);
void alg_locate (struct coord cent, struct images *, int width, int height);
int alg_diff (struct context *);
int alg_lightswitch (struct context *, int diffs, int dev, int pipe, int mpipe);
int alg_switchfilter (struct context *, int, struct coord *);




/*/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// motion: video.c */
/* for the v4l stuff: */
#include "sys/mman.h"
#include "sys/ioctl.h"

void yuv422to420p(unsigned char *map, unsigned char *cap_map, int width, int height)
{
	unsigned char *src, *dest, *src2, *dest2;
	int i, j;

	/* Create the Y plane */
	src=cap_map;
	dest=map;
	for (i=width*height; i; i--) {
		*dest++=*src;
		src+=2;
	}
	/* Create U and V planes */
	src=cap_map+1;
	src2=cap_map+width*2+1;
	dest=map+width*height;
	dest2=dest+(width*height)/4;
	for (i=height/2; i; i--) {
		for (j=width/2; j; j--) {
			*dest=((int)*src+(int)*src2)/2;
			src+=2;
			src2+=2;
			dest++;
			*dest2=((int)*src+(int)*src2)/2;
			src+=2;
			src2+=2;
			dest2++;
		}
		src+=width*2;
		src2+=width*2;
	}
}


/*******************************************************************************************
	Video4linux capture routines
*/



static char *v4l_next (struct video_dev *viddev, char *map, int width, int height)
{
	int dev=viddev->fd;
	int frame=viddev->v4l_curbuffer;
	struct video_mmap vid_mmap;
	char *cap_map=NULL;

    	sigset_t    set, old;

	if (viddev->v4l_read_img) {
		if (read(dev, map, viddev->v4l_bufsize) != viddev->v4l_bufsize)
			return NULL;
	} else {
		vid_mmap.format=viddev->v4l_fmt;
		vid_mmap.width=width;
		vid_mmap.height=height;
    		/* Block signals during IOCTL */
		sigemptyset (&set);
		sigaddset (&set, SIGCHLD);
		sigaddset (&set, SIGALRM);
		sigaddset (&set, SIGUSR1);
		sigaddset (&set, SIGTERM);
		sigaddset (&set, SIGHUP);
		pthread_sigmask (SIG_BLOCK, &set, &old);

		cap_map=viddev->v4l_buffers[viddev->v4l_curbuffer];
		viddev->v4l_curbuffer++;
		if (viddev->v4l_curbuffer >= viddev->v4l_maxbuffer)
			viddev->v4l_curbuffer=0;
		vid_mmap.frame=viddev->v4l_curbuffer;

		if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
	        	sigprocmask (SIG_UNBLOCK, &old, NULL);
			printf("mcatpure error %d %d %d\n", errno, EAGAIN, getpid());
			perror("mcapture");
			return (NULL);
		}

		vid_mmap.frame=frame;
		if (ioctl(dev, VIDIOCSYNC, &vid_mmap.frame) == -1) {
	        	sigprocmask (SIG_UNBLOCK, &old, NULL);
			printf("sync error %d\n", getpid());
			perror("sync");
		}
		
        	pthread_sigmask (SIG_UNBLOCK, &old, NULL);	/*undo the signal blocking*/
	}
	if (!viddev->v4l_read_img) {
		if (viddev->v4l_fmt==VIDEO_PALETTE_YUV422) {
			yuv422to420p(map, cap_map, width, height);
		} else
			memcpy(map, cap_map, viddev->v4l_bufsize);
	}

	return map;
}

void v4l_set_input (struct video_dev *viddev, char *map, int width, int height, int input, int norm, int skip, long freq)
{
	int dev=viddev->fd;
	int i;
	struct video_channel vid_chnl;
	struct video_tuner vid_tuner;

        if (input != viddev->input || width != viddev->width || height!=viddev->height || freq!=viddev->freq) {
		if (freq) {
			if (ioctl (dev, VIDIOCGTUNER, &vid_tuner)==-1) {
				perror("ioctl (VIDIOCGTUNER)");
			} else {
				if (vid_chnl.channel & VIDEO_TUNER_LOW) {
					freq=freq*16; /* steps of 1/16 KHz */
				} else {
					freq=(freq*10)/625;
				}
				if (ioctl(dev, VIDIOCSFREQ, &freq)==-1) {
					perror("ioctl (VIDIOCSFREQ)");
					return;
				}
			}
		}
                vid_chnl.channel = -1;
                if (ioctl (dev, VIDIOCGCHAN, &vid_chnl) == -1) {
                        perror ("ioctl (VIDIOCGCHAN)");
                } else {
			printf("input: %d\n", input);
                        vid_chnl.channel = input;
			vid_chnl.norm = norm;
                        if (ioctl (dev, VIDIOCSCHAN, &vid_chnl) == -1) {
                                perror ("ioctl (VIDIOCSCHAN)");
                                return;
                        }
                }
		viddev->input=input;
		viddev->width=width;
		viddev->height=height;
		/* skip a few frames if needed */
		for (i=0; i<skip; i++)
			v4l_next (viddev, map, width, height);
        }
}

static void v4l_autobright (int dev, unsigned char *image, int height, int width)
{
	struct video_picture vid_pic;
	int i, j=0, avg=0, offset=0;
	
	for (i=0; i<width*height*3; i+=100) {
		avg+=*image++;
		j++;
	}
	avg=avg/j;

	if (avg > 140 || avg < 64) {
		if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
			perror("ioctl VIDIOCGPICT");
		}
		if (avg > 140) {
			offset=avg-140;
			if (vid_pic.brightness > 100*offset)
				vid_pic.brightness-=100*offset;
		}
		if (avg < 64) {
			offset=64-avg;
			if (vid_pic.brightness < 65535-100*offset)
				vid_pic.brightness+=100*offset;
		}
		printf("auto_brightness: %d\n", vid_pic.brightness);
		if (ioctl(dev, VIDIOCSPICT, &vid_pic)==-1) {
			perror("ioctl VIDIOCSPICT");
		}
	}
}



char *vid_next (struct context *cnt, int dev, char *map, int width, int height)
{
	struct config *conf=&cnt->conf;
	int i;
	char *ret;

	i=-1;
	while (viddevs[++i])
		if (viddevs[i]->fd==dev)
			break;
	if (!viddevs[i])
		return NULL;

	if (viddevs[i]->owner!=cnt->threadnr) {
		pthread_mutex_lock(&viddevs[i]->mutex);
		viddevs[i]->owner=cnt->threadnr;
		viddevs[i]->frames=conf->roundrobing_frames;
		cnt->switched=1;
	}

	v4l_set_input (viddevs[i], map, width, height, conf->input, conf->norm, conf->roundrobing_skip, conf->frequency);
	ret=v4l_next (viddevs[i], map, width, height);

	if (--viddevs[i]->frames <= 0) {
		viddevs[i]->owner=-1;
		pthread_mutex_unlock(&viddevs[i]->mutex);
	}
	
	return ret;
}

char *vid_keepalive(struct context *cnt, struct images *imgs, int dev, int pipe, int mpipe)
{
	struct config *conf=&cnt->conf;

	imgs->new=vid_next(cnt, dev, imgs->new, imgs->width, imgs->height);
	if (imgs->new) {
		memcpy(imgs->ref, imgs->new, imgs->height*imgs->width*3);
		if (conf->vidpipe)
			vid_putpipe(pipe, imgs->new, imgs->size);
		if (conf->motionvidpipe)
			vid_putpipe(mpipe, imgs->out, imgs->size);
	}
	return imgs->new;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////7
// motion: alg.c
/* locate the center of the movement. */
struct coord alg_locate_center (struct images *imgs, int width, int height)
{
	char *out=imgs->out;
	int x, y, centc=0;
	struct coord cent;

	cent.x=0;
	cent.y=0;
	/* Locate movement */
	for (y=0; y<height; y++) {
		for (x=0; x<width; x++) {
			if (imgs->type==VIDEO_PALETTE_RGB24) {
				if (*(out++) + *(out++) + *(out++)) {
					cent.x+=x;
					cent.y+=y;
					centc++;
				}
			} else {
				if (*(out++)) {
					cent.x+=x;
					cent.y+=y;
					centc++;
				}
			}
		}
	}
	if (centc) {
		cent.x=cent.x/centc;
		cent.y=cent.y/centc;
	}
	return (cent);
}

/* draw a box around the movement */
void alg_locate (struct coord cent, struct images *imgs, int width, int height)
{
	char *out=imgs->out, *new=imgs->new;
	int x, y, maxx=0, maxy=0, minx=width, miny=height;
	int centc=0, xdist=0, ydist=0;

	centc=0;
	for (y=0; y<height; y++) {
		for (x=0; x<width; x++) {
			if (imgs->type==VIDEO_PALETTE_RGB24) {
				if (*(out++) + *(out++) + *(out++)) {
					if (x > cent.x) xdist+=x-cent.x;
					if (y > cent.y) ydist+=y-cent.y;
					if (x < cent.x) xdist+=cent.x-x;
					if (y < cent.y) ydist+=cent.y-y;
					centc++;
				}
			} else {
				if (*(out++)) {
					if (x > cent.x) xdist+=x-cent.x;
					if (y > cent.y) ydist+=y-cent.y;
					if (x < cent.x) xdist+=cent.x-x;
					if (y < cent.y) ydist+=cent.y-y;
					centc++;
				}
			}
		}	
	}
	if (centc) {
		minx=cent.x-xdist/centc*2;
		maxx=cent.x+xdist/centc*2;
		/* Make the box a litle bigger in y direction to make sure the
		 * heads fit in */
		miny=cent.y-ydist/centc*3;
		maxy=cent.y+ydist/centc*2;
	}
	if (minx < 0) minx=0;
	if (miny < 0) miny=0;
	if (maxx > width-1) maxx=width-1;
	if (maxy > height-1) maxy=height-1;
	if (maxx < 0) maxx=0;
	if (maxy < 0) maxy=0;
	if (minx > width-1) minx=width-1;
	if (miny > height-1) miny=height-1;
	out=imgs->out;
	
	/* Draw a box around the movement */
	if (imgs->type==VIDEO_PALETTE_RGB24) {
		for (x=minx; x<=maxx; x++) {
			new[(x+width*miny)*3+0]=~new[(x+width*miny)*3+0];
			new[(x+width*miny)*3+1]=~new[(x+width*miny)*3+1];
			new[(x+width*miny)*3+2]=~new[(x+width*miny)*3+2];
			new[(x+width*maxy)*3+0]=~new[(x+width*maxy)*3+0];
			new[(x+width*maxy)*3+1]=~new[(x+width*maxy)*3+1];
			new[(x+width*maxy)*3+2]=~new[(x+width*maxy)*3+2];
			out[(x+width*miny)*3+0]=~out[(x+width*miny)*3+0];
			out[(x+width*miny)*3+1]=~out[(x+width*miny)*3+1];
			out[(x+width*miny)*3+2]=~out[(x+width*miny)*3+2];
			out[(x+width*maxy)*3+0]=~out[(x+width*maxy)*3+0];
			out[(x+width*maxy)*3+1]=~out[(x+width*maxy)*3+1];
			out[(x+width*maxy)*3+2]=~out[(x+width*maxy)*3+2];
		}
		for (y=miny; y<=maxy; y++) {
			new[(minx+y*width)*3+0]=~new[(minx+y*width)*3+0];
			new[(minx+y*width)*3+1]=~new[(minx+y*width)*3+1];
			new[(minx+y*width)*3+2]=~new[(minx+y*width)*3+2];
			new[(maxx+y*width)*3+0]=~new[(maxx+y*width)*3+0];
			new[(maxx+y*width)*3+1]=~new[(maxx+y*width)*3+1];
			new[(maxx+y*width)*3+2]=~new[(maxx+y*width)*3+2];
			out[(minx+y*width)*3+0]=~out[(minx+y*width)*3+0];
			out[(minx+y*width)*3+1]=~out[(minx+y*width)*3+1];
			out[(minx+y*width)*3+2]=~out[(minx+y*width)*3+2];
			out[(maxx+y*width)*3+0]=~out[(maxx+y*width)*3+0];
			out[(maxx+y*width)*3+1]=~out[(maxx+y*width)*3+1];
			out[(maxx+y*width)*3+2]=~out[(maxx+y*width)*3+2];
		}
	} else {
		for (x=minx; x<=maxx; x++) {
			new[x+width*miny]=~new[x+width*miny];
			new[x+width*maxy]=~new[x+width*maxy];
			out[x+width*miny]=~out[x+width*miny];
			out[x+width*maxy]=~out[x+width*maxy];
		}
		for (y=miny; y<=maxy; y++) {
			new[minx+y*width]=~new[minx+y*width];
			new[maxx+y*width]=~new[maxx+y*width];
			out[minx+y*width]=~out[minx+y*width];
			out[maxx+y*width]=~out[maxx+y*width];
		}
	}
}

// NOT UP TO DATE (RGB/YUV)
int alg_diff_box(int x0, int y0, int x1, int y1, char *image, int width, int height)
{
	int diffs=0;
	int x, y;

	for (x=x0; x<=x1; x++)
		for (y=y0; y<=y1; y++)
			if (image[(width*y+x)*3+0] ||
			    image[(width*y+x)*3+1] ||
			    image[(width*y+x)*3+2])
			    	diffs++;
	return diffs;
}

// NOT UP TO DATE (RGB/YUV)
// If no longer experimental move it to draw.c
void draw_box(int x0, int y0, int x1, int y1, char *image, int width, int height)
{
	int x;
	int y;
	
	if (x0<0)
		x0=0;
	if (x1<0)
		x1=0;
	if (y0<0)
		y0=0;
	if (y1<0)
		y1=0;
	if (x0>width-1)
		x0=width-1;
	if (x1>width-1)
		x1=width-1;
	if (y0>height-1)
		y0=height-1;
	if (y1>height-1)
		y1=height-1;

	for (x=x0; x<=x1; x++) {
		image[(x+width*y0)*3+0]=~image[(x+width*y0)*3+0];
		image[(x+width*y0)*3+1]=~image[(x+width*y0)*3+1];
		image[(x+width*y0)*3+2]=~image[(x+width*y0)*3+2];
		image[(x+width*y1)*3+0]=~image[(x+width*y1)*3+0];
		image[(x+width*y1)*3+1]=~image[(x+width*y1)*3+1];
		image[(x+width*y1)*3+2]=~image[(x+width*y1)*3+2];
	}
	for (y=y0; y<=y1; y++) {
		image[(x0+y*width)*3+0]=~image[(x0+y*width)*3+0];
		image[(x0+y*width)*3+1]=~image[(x0+y*width)*3+1];
		image[(x0+y*width)*3+2]=~image[(x0+y*width)*3+2];
		image[(x1+y*width)*3+0]=~image[(x1+y*width)*3+0];
		image[(x1+y*width)*3+1]=~image[(x1+y*width)*3+1];
		image[(x1+y*width)*3+2]=~image[(x1+y*width)*3+2];
	}

}

// NOT UP TO DATE (RGB/YUV)
// Experimental
void alg_locate_new (struct coord cent, char *out, char *new, int width, int height)
{
	int diffline;
	int lastline;
	int i, j, k;
	int diffs=0;
	int object;
	int avgv=1;
	int avgh=1;
	int size=0;
	int *uph=NULL;
	int *upv=NULL;
	int *downh=NULL;
	int *downv=NULL;
	int uphc=0;
	int upvc=0;
	int downhc=0;
	int downvc=0;
	int boxx0;
	int boxx1;
	int boxy0;
	int boxy1;
	
	// get existing diffs instead of getting our own
	for (i=0; i<width*height*3; i++)
		if (out[i]) diffs++;

	// check if diffs!=0
	if (diffs) {
		avgv=diffs/width;
		avgh=diffs/height;
	}

	printf("%d\n", avgv);
	size=0;
	object=0;
	lastline=-80;
	for (i=4; i<width; i++){
		diffline=0;
		for (j=0; j<height; j++) {
			if (out[j*width*3+i*3+0] || out[j*width*3+i*3+1] || out[j*width*3+i+3+2])
				diffline++;
		}
		if (!object && size>avgv*2 && i-80>lastline) {
			object=1;
			upvc++;
			upv=realloc(upv, sizeof(int)*upvc);
			upv[upvc-1]=i;
			for (j=0; j<height; j++) {
				new[j*width*3+(i-4)*3+2]=255;
			}
		}
		size=size+diffline;
		if ((diffline<=avgv)) {
			size=size*3/4;
			if (object && size<avgv) {
				object=0;
				lastline=i;
				if (size>0)
					size=0;
			}
		}
	}
	size=0;
	object=0;
	lastline=width+80;
	for (i=width-4; i>=0; i--){
		diffline=0;
		for (j=0; j<height; j++) {
			if (out[j*width*3+i*3+0] || out[j*width*3+i*3+1] || out[j*width*3+i+3+2])
				diffline++;
		}
		if (!object && size>avgv*2 && i+80<lastline) {
			object=1;
			downvc++;
			downv=realloc(downv, sizeof(int)*downvc);
			downv[downvc-1]=i;
			for (j=0; j<height; j++) {
				new[j*width*3+(i+4)*3+0]=255;
			}
		}
		size=size+diffline;
		if ((diffline<=avgv)) {
			size=size*3/4;
			if (object && size<avgv) {
				object=0;
				lastline=i;
				if (size>0)
					size=0;
			}
		}
	}
	size=0;
	object=0;
	lastline=-80;
	for (i=4; i<height; i++){
		diffline=0;
		for (j=0; j<width; j++) {
			if (out[i*width*3+j*3+0] || out[i*width*3+j*3+1] || out[i*width*3+j+3+2])
				diffline++;
		}
		if (!object && size>avgh*2 && i-80>lastline) {
			object=1;
			uphc++;
			uph=realloc(uph, sizeof(int)*uphc);
			uph[uphc-1]=i;
			for (j=0; j<width; j++) {
				new[(i-4)*width*3+j*3+2]=255;
			}
		}
		size=size+diffline;
		if ((diffline<=avgh)) {
			size=size*3/4;
			if (object && size<avgh) {
				object=0;
				lastline=i;
				if (size>0)
					size=0;
			}
		}
	}
	size=0;
	object=0;
	lastline=height+80;
	for (i=height-4; i>=0; i--){
		diffline=0;
		for (j=0; j<width; j++) {
			if (out[i*width*3+j*3+0] || out[i*width*3+j*3+1] || out[i*width*3+j+3+2])
				diffline++;
		}
		if (!object && size>avgh*2 && i+80<lastline) {
			object=1;
			downhc++;
			downh=realloc(downh, sizeof(int)*downhc);
			downh[downhc-1]=i;
			for (j=0; j<width; j++) {
				new[(i+4)*width*3+j*3+0]=255;
			}
		}
		size=size+diffline;
		if ((diffline<=avgh)) {
			size=size*4/5;
			if (object && size<avgh) {
				object=0;
				lastline=i;
				if (size>0)
					size=0;
			}
		}
	}
	printf("upvc: %d downvc: %d uphc: %d downhc: %d\n", upvc, downvc, uphc, downhc);
	size=0;
	for (i=0; i<upvc; i++) {
		boxx0=upv[i];
		boxx1=width-1;
		for (j=0; j<downvc; j++)
			if (downv[j]>boxx0)
				boxx1=downv[j];
		boxy0=0;
		boxy1=height-1;
		for (j=0; j<uphc; j++) {
			boxy0=uph[j];
			for (k=0; k<downhc; k++)
				if (downh[k]>boxy0)
					boxy1=downh[k];
			size+=(boxx1-boxx0)*(boxy1-boxy0);
		}
	}
	for (i=0; i<upvc; i++) {
		boxx0=upv[i];
		boxx1=width-1;
		for (j=0; j<downvc; j++)
			if (downv[j]>boxx0)
				boxx1=downv[j];
		boxy0=0;
		boxy1=height-1;
		for (j=0; j<uphc; j++) {
			boxy0=uph[j];
			for (k=0; k<downhc; k++)
				if (downh[k]>boxy0)
					boxy1=downh[k];
			if (alg_diff_box(boxx0, boxy0, boxx1, boxy1, out, width, height)>diffs*((boxx1-boxx0)*(boxy1-boxy0)/(diffs*2)))
				draw_box(boxx0-8, boxy0-8, boxx1+8, boxy1+8, new, width, height);
		}
	}
	if (upv) free(upv);
	if (uph) free(uph);
	if (downv) free(downv);
	if (downh) free(downh);

	printf("\n");
}

/*	The heart of the motion detector
 *	Basicly: ref-new and count the pixels that are over a noise threshold
 */
int alg_diff_standard (struct context *cnt)
{
	struct images *imgs=&cnt->imgs;
	int i, diffs=0;
	long int level=0;
	int noise=cnt->conf.noise;
	char *ref=imgs->ref;
	char *new=imgs->new;
	char *out=imgs->out;
	unsigned char *mask=imgs->mask;


	/* If the average level of the picture is to low, compensate by 
	 * lowering the noise threshold
	 */
	if (cnt->conf.nightcomp) {
		i=imgs->motionsize;
		for (i--; i>=0; i--) {
			level+=(unsigned char)new[i];
		}
		level/=imgs->motionsize;
		if (level < noise*2)
			noise*=2;
	}

	i=imgs->motionsize;
	for (; i>0; i--) {
		*out=*ref-*new;
		if (mask)
			*out=((int)((char)*out**mask++)/255);
		if (*out >  noise ||
		    *out < -noise ) {
			if (!cnt->conf.realmotion) *out=*new;
			diffs++;
		} else {
			if (!cnt->conf.realmotion) *out=0;
		}
		out++;
		ref++;
		new++;
	}

	return diffs;
}

/*
	Very fast diff function, does not do nightcompensation or mask
	overlaying.
*/
int alg_diff_fast (struct context *cnt)
{
	struct images *imgs=&cnt->imgs;
	int i, diffs=0;
	int noise=cnt->conf.noise;
	char *out=imgs->out;
	char *ref=imgs->ref;
	char *new=imgs->new;

	memset(out, 0, imgs->size);
	i=imgs->motionsize;
	for (; i>0; i-=40) {
		*out=*ref-*new;
		if (*out >  noise ||
		    *out < -noise ) {
			if (!cnt->conf.realmotion) *out=*new;
			diffs++;
		} else {
			if (!cnt->conf.realmotion) *out=0;
		}
		out+=40;
		ref+=40;
		new+=40;
	}
	diffs=diffs*40;
	return diffs;
}

/*
	diff_hybrid uses diff_fast to quickly decide if there is anything worth
	sending to diff_standard.
*/
int alg_diff_hybrid (struct context *cnt)
{
	int diffs;
	
	diffs=alg_diff_fast(cnt);
	if (diffs > cnt->conf.max_changes/2)
		diffs=alg_diff_standard(cnt);
	return diffs;
}

int alg_diff (struct context *cnt)
{
	return alg_diff_hybrid(cnt);
}


/* Detect a sudden massive change in the picture.
   It is assumed to be the light being switched on or a camera displacement.
   In any way the user doesn't think it is worth capturing.
 */
int alg_lightswitch (struct context *cnt, int diffs, int dev, int pipe, int mpipe)
{
	struct images *imgs=&cnt->imgs;
	int i, j;
	
	/* is 2/3 of the image right?  */
	if (diffs > imgs->width*imgs->height*2) {
		printf("lightswitch detected\n");
		/* wait for the device to settle and update the
		 * reference frame to prevent the next frames
		 * from being detected
		 * if 1/3 of the frame is still moving asume that the camera
		 * has not yet compensated.
		 */
		i=10;
		while(i && (imgs->new=vid_next(cnt, dev, imgs->new, imgs->width, imgs->height))) {
			if (alg_diff(cnt)
			    > imgs->width*imgs->height)
				i=10;
			j=imgs->width*imgs->height;
			if (imgs->type==VIDEO_PALETTE_RGB24)
				j*=3;
			for (j--; j>=0; j--) {
				imgs->ref[j]=imgs->new[j];
			}
			event(EVENT_IMAGE, cnt, imgs->new, &pipe, cnt->currenttime);
			event(EVENT_IMAGEM, cnt, imgs->out, &mpipe, cnt->currenttime);
			i--;
		}
		return 0;
	}
	return diffs;
}

int alg_switchfilter (struct context *cnt, int diffs, struct coord *cent)
{
	int linediff=diffs/cnt->conf.height;
	struct images *imgs=&cnt->imgs;
	char *out=cnt->imgs.out;
	int y, x, line;
	int lines=0, vertlines=0;
	char tmp[80];

	if (imgs->type==VIDEO_PALETTE_RGB24)
		for (y=0; y<cnt->conf.height; y++) {
			line=0;
			for (x=0; x<cnt->conf.width*3; x++) {
				if (*(out++)) {
					line++;
				}
			}
			if (line>cnt->conf.width/6) {
				vertlines++;
			}
			if (line>linediff*2) {
				lines++;
			}
		}
	else
		for (y=0; y<cnt->conf.height; y++) {
			line=0;
			for (x=0; x<cnt->conf.width; x++) {
				if (*(out++)) {
					line++;
				}
			}
			if (line>cnt->conf.width/18) {
				vertlines++;
			}
			if (line>linediff*2) {
				lines++;
			}
		}
//	printf("%d %d %d %d  %d\n", lines, vertlines, linediff, lines-vertlines, diffs);
	if (vertlines>cnt->conf.height/10 && lines<vertlines/3 && 
	    (vertlines>cnt->conf.height/4 || lines-vertlines>lines/2)) {
	    	sprintf(tmp, "%d %d", lines, vertlines);
	    	draw_text(cnt->imgs.new, 10, 10, 288, 352, tmp, cnt->imgs.type);
		return diffs;
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////7
// motion: motion.c

int motion_detected (struct context *cnt, int diffs, int dev, int devpipe, int devmpipe)
{
	struct config *conf=&cnt->conf;
	struct images *imgs=&cnt->imgs;
	struct coord location;

	if (conf->locate || cnt->track.port || conf->switchfilter)
		location=alg_locate_center(imgs, imgs->width, imgs->height);
	if (conf->locate)
		alg_locate(location, imgs, imgs->width, imgs->height);
	if (diffs && conf->switchfilter) {
		diffs=alg_switchfilter(cnt, diffs, &location);
		if (diffs <= cnt->conf.max_changes )
			return 0;
	}
	event(EVENT_MOTION, cnt, NULL, NULL, cnt->currenttime);
	cnt->lasttime=cnt->currenttimep;
	/* Take action if necesarry */
	if (cnt->event_nr!=cnt->prev_event) {
		event(EVENT_FIRSTMOTION, cnt, NULL, NULL, cnt->currenttime);
		cnt->prev_event=cnt->event_nr;
		cnt->eventtime=cnt->currenttimep;
	}

	if (cnt->shots<conf->frame_limit-1 && cnt->currenttimep-cnt->lastshot >= conf->mingap ) {
		cnt->lastshot=cnt->currenttimep;
		/* Output the latest picture 'image_new' or image_out for motion picture.
		 */
		event(EVENT_IMAGE_DETECTED, cnt, NULL, NULL, cnt->currenttime);
	}

	if (cnt->track.port) {
		if (track_move(cnt, dev, devpipe, devmpipe, location, imgs)) {
			cnt->moved=2;
		}
	}
	return diffs;
}


void *motion_loop(void *arg)
{
	struct context *cnt=arg;
	int dev, devpipe=-1, devmpipe=-1, i, j, diffs, detecting_motion;
	time_t lastframe=0;
	FILE *picture;
	int postcap=0;
	char tmp[20];

	int passflag=0;
	long int *rolling_average_data;
	long int rolling_average_limit, required_frame_time, frame_delay;
	int rolling_frame=0;
	struct timeval tv1, tv2;
	unsigned long int rolling_average, elapsedtime, splittime, timenow=0, timebefore=0;

	printf("thread pid: %d\n", getpid());

	if (!cnt->conf.filepath)
		cnt->conf.filepath=".";

	/* Work out expected frame rate based on config setting */
	if (cnt->conf.frame_limit)
		required_frame_time=1000000L/cnt->conf.frame_limit;
	else
		required_frame_time=0;
	frame_delay=required_frame_time;
	/* Reserve enough space for a 10 second timing history buffer */
	rolling_average_limit=10*cnt->conf.frame_limit;
	rolling_average_data=malloc(sizeof(long int)*rolling_average_limit);
	if (!rolling_average_data)
	{
		perror("Unable to allocate memory [rolling_average_data]");
		exit(1);
	}
	/* Preset history buffer with expected frame rate */
	for (j=0; j< rolling_average_limit; j++)
		rolling_average_data[j]=required_frame_time;

	/* set the device settings */
	dev=vid_start (cnt, cnt->conf.width, cnt->conf.height, cnt->conf.input, cnt->conf.norm);
	if (dev==-1) {
		perror("Capture error");
		exit(1);
	}
	cnt->imgs.new=malloc(cnt->imgs.size);
	cnt->imgs.ref=malloc(cnt->imgs.size);
	cnt->imgs.out=malloc(cnt->imgs.size);

	/* Allow videodevice to settle in */
	sleep(1);
	/* Capture first image, or we will get an alarm on start */
	if (!(cnt->imgs.new=vid_next(cnt, dev, cnt->imgs.new, cnt->conf.width, cnt->conf.height))) {
		perror("Capture error");
		exit(1);
	}
	/* create a reference frame */
	memcpy(cnt->imgs.ref, cnt->imgs.new, cnt->imgs.size);

	if (cnt->conf.vidpipe) {
		printf("Opening video loopback device for normal pictures\n");
		devpipe=vid_startpipe(cnt->conf.vidpipe, cnt->conf.width, cnt->conf.height, cnt->imgs.type);
		if (devpipe < 0) {
			printf ("Failed to open video loopback\n");
			exit(1);
		}
	}
	if (cnt->conf.motionvidpipe) {
		printf("Opening video loopback device for motion pictures\n");
		devmpipe=vid_startpipe(cnt->conf.motionvidpipe, cnt->conf.width, cnt->conf.height, cnt->imgs.type);
		if (devmpipe < 0) {
			printf ("Failed to open video loopback\n");
			exit(1);
		}
	}

#ifdef HAVE_MYSQL
	if(cnt->conf.mysql_db) {
		cnt->database=(MYSQL *) malloc(sizeof(MYSQL));
		mysql_init(cnt->database);
		if (!mysql_real_connect(cnt->database, cnt->conf.mysql_host, cnt->conf.mysql_user,
		    cnt->conf.mysql_password, cnt->conf.mysql_db, 0, NULL, 0)) {
			printf("Cannot connect to database\n");
			exit(1);
		}
	}
#endif	
#ifdef HAVE_PGSQL
	if (cnt->conf.pgsql_db) {
		char connstring[255];
	
		/* create the connection string. 
		   Quote the values so we can have null values (blank)*/
		snprintf(connstring, 255,
		    "dbname='%s' host='%s' user='%s' password='%s' port='%d'",
		    cnt->conf.pgsql_db, /* dbname */
		    (cnt->conf.pgsql_host ? cnt->conf.pgsql_host : ""), /* host (may be blank) */
		    (cnt->conf.pgsql_user ? cnt->conf.pgsql_user : ""), /* user (may be blank) */
		    (cnt->conf.pgsql_password ? cnt->conf.pgsql_password : ""), /* password (may be blank) */
		    cnt->conf.pgsql_port
		);

		cnt->database_pg = PQconnectdb(connstring);
		if (PQstatus(cnt->database_pg) == CONNECTION_BAD)
		{
			fprintf(stderr, "Connection to database '%s' failed.\n", cnt->conf.pgsql_db);
			fprintf(stderr, "%s", PQerrorMessage(cnt->database_pg));
			exit(1);
		}
	}
#endif

	if (cnt->conf.mask_file) {
		if ((picture=fopen(cnt->conf.mask_file, "r"))) {
			cnt->imgs.mask=get_pgm(picture, cnt->conf.width, cnt->conf.height, cnt->imgs.type==VIDEO_PALETTE_RGB24 ? 1 : 0);
			fclose(picture);
		} else {
			fprintf(stderr, "Error opening mask file\n");
		}
		if (!cnt->imgs.mask) {
			perror ("failed to read mask image");
			exit(1);
		}
	} else
		cnt->imgs.mask=NULL;

	/* Set signal handlers */
	signal(SIGALRM, sig_handler);
	signal(SIGUSR1, sig_handler);
	signal(SIGHUP, sig_handler);
	signal(SIGTERM, sig_handler);


	if (cnt->conf.webcam_port)
		webcam_init(cnt);

	/* Prevent first few frames from triggering motion... */
	cnt->moved=8;
	if (cnt->alarmtime) 
		alarm(cnt->alarmtime);
	/* Should go on forever... unless you bought vaporware :) */
	while (cnt->finish ? cnt->makemovie : (int)(cnt->imgs.new=vid_next(cnt, dev, cnt->imgs.new, cnt->conf.width, cnt->conf.height))) {
		/* Get current time and preserver last time for frame interval calc. */
		timebefore=timenow;
		gettimeofday(&tv1, NULL);
		timenow=tv1.tv_usec+1000000L*tv1.tv_sec;

		/* Make a differences picture in image_out */
		if (cnt->conf.max_changes)
			diffs=alg_diff(cnt);
		else
			diffs=0;
		if (cnt->conf.autobright)
			vid_autobright(&cnt->conf, dev, cnt->imgs.new, cnt->conf.width, cnt->conf.height);
		if (cnt->moved) {
			cnt->moved--;
			diffs=0;
		}
		cnt->currenttimep=time(NULL);
		cnt->currenttime=localtime(&cnt->currenttimep);
		if (lastframe!=cnt->currenttimep) {
			cnt->lastrate=cnt->shots;
			cnt->shots=-1;
			lastframe=cnt->currenttimep;
		}
		/* lights turned on ? */
		if (cnt->conf.lightswitch) {
			diffs=alg_lightswitch (cnt, diffs, dev, devpipe, devmpipe);
		}
		/* Is the mpeg movie to long? */
		if (cnt->conf.maxmpegtime && (
#ifdef HAVE_FFMPEG
		    (cnt->ffmpeg_new || cnt->ffmpeg_motion) ||
#endif /* HAVE_FFMPEG */
		    (cnt->files || cnt->filesm)))
			if (cnt->currenttimep-cnt->eventtime>= cnt->conf.maxmpegtime)
				cnt->makemovie=1;
		if ((cnt->currenttimep-cnt->lasttime>=cnt->conf.gap) || cnt->makemovie) {
			if (cnt->event_nr==cnt->prev_event || cnt->makemovie) {
				event(EVENT_ENDMOTION, cnt, NULL, NULL, cnt->currenttime);
				if (cnt->track.port) {
					printf("%d %d %d\n", cnt->prev_event, cnt->event_nr, cnt->makemovie);
					track_center(cnt);
					cnt->moved=3;
				}
				cnt->makemovie=0;
				cnt->event_nr++;
    				if (cnt->conf.mpeg) {
					if (cnt->filesc)
						create_mpeg(cnt, cnt->prev_event,
							    cnt->filesc,
							    cnt->files, 
							    cnt->filesrate
							    );
					if (cnt->filesmc)
						create_mpeg(cnt, cnt->prev_event,
							    cnt->filesmc, 
							    cnt->filesm, 
							    cnt->filesmrate
							    );
				}
				for (i=0; i<cnt->filesc; i++) free(cnt->files[i]);
				if (cnt->files)
					free(cnt->files);
				for (i=0; i<cnt->filesmc; i++) free(cnt->filesm[i]);
				if (cnt->filesm)
					free(cnt->filesm);
				if (cnt->filesrate)
					free(cnt->filesrate);
				if (cnt->filesmrate)
					free(cnt->filesmrate);
				cnt->filesc=0;
				cnt->files=NULL;
				cnt->filesmc=0;
				cnt->filesm=NULL;
				cnt->filesrate=NULL;
				cnt->filesmrate=NULL;
			}
		}

		/* Old image slowly decays, this will make it even harder on
		 * a slow moving object to stay undetected
		 */
		for (i=cnt->imgs.size-1; i>=0; i--) {
			cnt->imgs.ref[i]=(cnt->imgs.ref[i]+cnt->imgs.new[i])/2;
		}

		if (cnt->conf.drawtext_user)
			draw_text(cnt->imgs.new, 10, cnt->conf.height-10, cnt->conf.height, cnt->conf.width, cnt->conf.drawtext_user, cnt->imgs.type);

		cnt->shots++;
		sprintf(cnt->datestamp, "%04d-%02d-%02d",
		    cnt->currenttime->tm_year + 1900, cnt->currenttime->tm_mon + 1,
		    cnt->currenttime->tm_mday);
		
		if (cnt->conf.drawtext_shots) {
			sprintf(cnt->timestamp, "%02d:%02d:%02d-%02d",
			cnt->currenttime->tm_hour, cnt->currenttime->tm_min,
			cnt->currenttime->tm_sec, cnt->shots);
		} else {
			sprintf(cnt->timestamp, "%02d:%02d:%02d",
			cnt->currenttime->tm_hour, cnt->currenttime->tm_min,
			cnt->currenttime->tm_sec);
		}
		draw_text(cnt->imgs.new, cnt->conf.width-10, cnt->conf.height-20, cnt->conf.height, cnt->conf.width, cnt->datestamp, cnt->imgs.type);
		draw_text(cnt->imgs.new, cnt->conf.width-10, cnt->conf.height-10, cnt->conf.height, cnt->conf.width, cnt->timestamp, cnt->imgs.type);
		sprintf(tmp, "%d", diffs);
		if (cnt->conf.drawtext_changes)
			draw_text(cnt->imgs.new, cnt->conf.width-10, cnt->conf.height-30, cnt->conf.height, cnt->conf.width, tmp, cnt->imgs.type);
		
		/* The cat just walked in :) */
		detecting_motion=1;
		if (diffs > cnt->conf.max_changes) {
			if (motion_detected(cnt, diffs, dev, devpipe, devmpipe))
				postcap=cnt->conf.post_capture;
		} else if (postcap) {
			motion_detected(cnt, 0, dev, devpipe, devmpipe);
			postcap--;
		} else
			detecting_motion=0;

		if (cnt->conf.always_diff)
			printf("changes: %d\n", diffs);
		if (cnt->track.port && cnt->moved) {
			for (i=0; i<cnt->imgs.size; i++)
				cnt->imgs.ref[i]=cnt->imgs.new[i];
		}

		if (cnt->snapshot) {
			event(EVENT_IMAGE_SNAPSHOT, cnt, NULL, NULL, cnt->currenttime);
			cnt->snapshot=0;
		}
		if (cnt->conf.timelaps && cnt->currenttime->tm_sec==0 && cnt->shots==0) {
			if (cnt->currenttime->tm_hour==0 && cnt->currenttime->tm_min==0)
				event(EVENT_TIMELAPSEND, cnt, NULL, NULL, cnt->currenttime);
			event(EVENT_TIMELAPS, cnt, NULL, NULL, cnt->currenttime);
		}
		event(EVENT_IMAGE, cnt, cnt->imgs.new, &devpipe, cnt->currenttime);
		event(EVENT_IMAGEM, cnt, cnt->imgs.out, &devmpipe, cnt->currenttime);

		/* Get latest time to calculate time taken to process video data */
		gettimeofday(&tv2, NULL);
		splittime=tv2.tv_usec+1000000L*tv2.tv_sec;
		elapsedtime=splittime-timenow;
		/* Update history buffer but ignore first pass as timebofre will
		   variable will be inaccurate
		 */
		if (passflag)
			rolling_average_data[rolling_frame]=timenow-timebefore;
		rolling_frame++;
		passflag=1;
		if (rolling_frame>=rolling_average_limit)
			rolling_frame=0;
		rolling_average=0L;
		/* Calculate 10 second average and use deviation in delay calculation */
		for (j=0; j<rolling_average_limit; j++)
			rolling_average+=rolling_average_data[j];
		rolling_average/=rolling_average_limit;
		frame_delay=required_frame_time-elapsedtime-(rolling_average-required_frame_time);
		if (frame_delay>0) {
			/* Apply delay to meet frame time */
			if (frame_delay>required_frame_time)
				frame_delay=required_frame_time;
			if (DEBUG && !cnt->conf.quiet)
				printf("Split Time is %7lu. Applying %6lu delay. Avg Frame Time %7lu Frame Error %ld\n",
				    elapsedtime, frame_delay, rolling_average, rolling_average-required_frame_time);
			usleep(frame_delay);
		}

		/* This will limit the framerate to 1 frame while not detecting
		   motion. Using a different motion flag to allow for multiple frames per second
		 */
		if (cnt->conf.low_cpu && !detecting_motion) {
			/* Recalculate remaining time to delay for a total of 1s */
			if (required_frame_time+elapsedtime<1000000) {
				frame_delay=1000000-required_frame_time-elapsedtime;
				usleep(frame_delay);
			}
			/* Correct frame times to ensure required_frame_time is maintained */
			gettimeofday(&tv1, NULL);
			timenow=(tv1.tv_usec+1000000L*tv1.tv_sec)-required_frame_time;
		}
		fflush(0);
	}
	if (!cnt->finish)
		perror("Somebody stole the video device, lets hope we got his picture");
	event(EVENT_STOP, cnt, NULL, NULL, NULL);
	threads_running--;
	return NULL;
}

