/**
 * @file
 * @brief Wlan feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-23 12:26:48 +0100 (Mon, 23 Feb 2004) $
 * $Revision: 638 $
 *
 * @since    Wed Mar 19 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: WlanLinux.h 638 2004-02-23 11:26:48Z rene $
 */

#ifndef _WLANLINUX_H
#define _WLANLINUX_H

#include "Wlan.h"
#include "../../util/Thread.h"


/**
 * @todo documentation
 */
class WlanLinuxFeatureProvider : public WlanFeatureProvider, public Thread
{
	private:
		//@{
		/** @todo documentation */
		string ifname;
		int wlsocket;
		int delay;
		mutable Lock lockApList;
		//@}

	public:
		// Linux interface name (e.g. wlan0, eth1) and delay between scans in
		// milliseconds
		/** @todo documentation */
		WlanLinuxFeatureProvider(featureparams& params);
		/** Destructor */
		virtual ~WlanLinuxFeatureProvider() throw() ;

		// the scanning thread, which continously scans for peers (only for the
		// Peers feature, the other feature values can be read from the kernel with
		// no delay and are thus handled in nextSample)
		virtual void run() throw();

		// we have to overwrite this because of the needed multi-thread locking
		virtual Feature* getSample(string name) const;
		virtual void nextSample(clock_t checkpoint);
	
	private:
		//@{
		/** @todo documentation */
		bool initSocket(string interfaceName);
		void closeSocket();
		bool getEssid(string& essid);
		bool getApAddress(string& apAddress);
		bool getApList(vector<WlanPeerInfo>& apList, bool changeEssid);
		bool getMode(WlanActiveModeFeature::FeatureKind& mode);
		bool getApSignalLevel(double& level);
		//@}
};

#endif // _WLANLINUX_H
