/**
 * @file
 * @brief Wlan feature
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Wed May 14 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: WlanSymbolAPI.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include <windows.h>
#include <s24sdk32.h>
#include <s24api.h>

#include <stdio.h>

#include "Wlan.h"

EXPORT_FEATURE(WlanFeatureProvider);

/** Driver handle */
HINSTANCE hLib = NULL;

/** Library entry point */
BOOL APIENTRY DllMain(HANDLE hModule, DWORD  fdwReason, LPVOID lpReserved)
{
	switch (fdwReason) { 
		case DLL_PROCESS_ATTACH:
			hLib = S24_LoadS24Driver();
			if (hLib == NULL) 
			{
#ifdef _DEBUG_CONSOLE
				cout << "Unable to load S24Driver\n" << flush;
#endif
				return FALSE;
			}
			if (! S24_InitS24Driver(hLib)) 
			{
#ifdef _DEBUG_CONSOLE
				cout << "Unable to initialize S24Driver\n" << flush;
#endif
				return FALSE;
			}
			if (! S24_IsConnected())
			{
#ifdef _DEBUG_CONSOLE
				cout << "S24Driver is not connected\n" << flush;
#endif
				return FALSE;
			}
				
			library_initialize();
			break;

		case DLL_PROCESS_DETACH:
			library_finalize();

			if (hLib != NULL)
				S24_FreeS24Driver(hLib);
			break;
	}

	return TRUE;
}

void WlanFeatureProvider::nextSample(clock_t checkpoint)
{
	unsigned char lpszESSID[32];
	unsigned char accessPointMacAddress[6];
	unsigned short nPercentBeaconsMissed, nPercentTxRetries;
	char buffer[18];
	S24_AP_TABLE4 apTable[64];
	int numAPs=-1;
	WlanPeerInfo peer;

	this->activePeerValid = this->peerListValid = false;

	if (S24_GetESSID(lpszESSID) == RC_SUCCESS && 
		S24_GetAPMacAddress(accessPointMacAddress) == RC_SUCCESS &&
		S24_GetChannelQualityValues(&nPercentBeaconsMissed, &nPercentTxRetries) == RC_SUCCESS)
	{
		this->activePeer.essid = string((char*) lpszESSID);
		sprintf(buffer, "%02x:%02x:%02x:%02x:%02x:%02x", *(accessPointMacAddress), 
			*(accessPointMacAddress+1), *(accessPointMacAddress+2), 
			*(accessPointMacAddress+3), *(accessPointMacAddress+4), 
			*(accessPointMacAddress+5)); 
		this->activePeer.mac = buffer;
		this->activePeer.signalLevel = (float) nPercentBeaconsMissed / 100;
		this->activePeerValid = true;
	} 
#ifdef _DEBUG_CONSOLE
	else if (S24_GetESSID(lpszESSID) != RC_SUCCESS) 
		cout << "S24_GetESSID failed\n" << flush;
	else if (S24_GetAPMacAddress(accessPointMacAddress) != RC_SUCCESS)
		cout << "S24_GetAPMacAddress failed\n" << flush;
	else
		cout << "S24_GetChannelQualityValues failed\n" << flush;
#endif

	if (S24_GetAPTable (apTable, &numAPs) == RC_SUCCESS)
	{
		UCHAR *pApMac=apTable[0].AP_BSS_ID+2;

		peerList.clear();
		for (int count=0; count<numAPs; count+=2)
		{
			sprintf(buffer, "%02x:%02x:%02x:%02x:%02x:%02x", 
				*(pApMac), *(pApMac+1), *(pApMac+2), 
				*(pApMac+3), *(pApMac+4), *(pApMac+5)); 
			peer.mac = string(buffer);
			this->peerList.push_back(peer);
			pApMac+=2*(sizeof(S24_AP_TABLE4)-3);
		}
		this->peerListValid = true;
	} 
#ifdef _DEBUG_CONSOLE
	else 
		cout << "Buffer of AP table too small !\n" << flush;
#endif
}
