/**
 * @file
 * @brief Active lezi predictor
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Tue Dec 16 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: ActiveLezi.h 612 2004-02-12 20:08:39Z harald $
 */

#ifndef _ACTIVELEZI_H
#define _ACTIVELEZI_H

#include "../Predictor.h"
#include "../../util/trie.h"

/**
 * @todo documentation
 */
class ActiveLezi : public PredictorAlgorithm {
	private:
		//@{
		/**
		 * Private property
		 * @todo documentation
		 */
		Trie<unsigned long> dictionary;
		list<unsigned long> phrase;
		list<unsigned long> window;
		unsigned int max_lz_length;
		unsigned int maxWindowModelDepth;

		// this is the context which is used for prediction in getNextContext
		list<unsigned long> curContext;

		// this is only a list of all context IDs seen so far
		list<unsigned long> seen_context_ids;
		//@}

		// a helper used for single- and multi-step prediction
		/** @todo documentation */
		unsigned long getNextContext(const list<unsigned long>* context) const;

	public:
		// the parameter maxWindowModelDepth does not restrict the deepest path in the trie, but only the depth of the context window which is used
		// setting it to a value <1 means to deactivate the context window completely and reducing the algorithm to a normal Lempel-Ziv parsing with prediction
		/** @todo documentation */
		ActiveLezi(predictorparams &params);
		/** Destructor */
		virtual ~ActiveLezi();

		virtual void addContexts(const membershiplist* contexts, time_t time);
		virtual void addContext(unsigned long contextId, time_t time);

		virtual unsigned long getNextContext() const;
		virtual membershiplist getNextContexts() const;

		virtual contexttrajectory getContextTrajectory(unsigned int start, unsigned int end) const;

		virtual unsigned long getContextAt(time_t time) const;
		virtual membershiplist getContextsAt(time_t time) const;

		virtual string serialize() const;
		virtual void unserialize(string data);

		// helper functions for evaluation
		/** @todo documentation */
		unsigned int getMaxDepth() const;

#if _DEBUG_PREDICTORS
		virtual string toString() const;
#endif
};

#endif
