/**
 * @file
 * @brief Hidden Markov Model (HMM) predictor
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-08-19 15:43:08 +0200 (Don, 19 Aug 2004) $
 * $Revision: 900 $
 *
 * @since    Tue Dec 16 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: HMM.h 900 2004-08-19 13:43:08Z rene $
 */

#ifndef _HMM_H
#define _HMM_H

#include "../Predictor.h"
#define HAVE_NAMESPACES
#include <ghmm++/GHMM.h>
#include <ghmm++/GHMM_DiscreteModel.h>

/**
 * @todo documentation
 */
class HMM : public PredictorAlgorithm {
	private:
		//@{
		/**
		 * Private property
		 * @todo documentation
		 */
		//unsigned int windowLength;
		 
		unsigned int numHiddenStates;
		unsigned int numAlphabetSymbols;
		GHMM_DiscreteModel* hmm;

		//list<unsigned long> curContextWindow;
		list<unsigned long> seenContexts;

	public:
		HMM(predictorparams &params);
		// test contructor !
		HMM(unsigned int numHiddenStates, unsigned int numAlphabetSymbols);
		/** Destructor */
		virtual ~HMM();

		virtual void addContexts(const membershiplist* contexts, time_t time);
		virtual void addContext(unsigned long contextId, time_t time);

		virtual unsigned long getNextContext() const;
		virtual membershiplist getNextContexts() const;

		virtual contexttrajectory getContextTrajectory(unsigned int start, unsigned int end) const;

		virtual unsigned long getContextAt(time_t time) const;
		virtual membershiplist getContextsAt(time_t time) const;

		virtual string serialize() const;
		virtual void unserialize(string data);
		
		/// helper function until I find out how to do a iterative model update....
		// have to specify sliding windows size ! 
		// this trains the hmm with more than one sequences of length slidingWindowSize with an offset of slidingWindowHops
		void addContextTrajectory(const vector<unsigned int>& traj, unsigned int from, unsigned int to, unsigned int slidingWindowSize, unsigned int slidingWindowHops);
		unsigned long getNextContext(const vector<unsigned int>& window) const;
		vector<unsigned long> getNextContextsMultiStep(const vector<unsigned int>& window, unsigned int numSteps) const;
		virtual membershiplist getNextContexts(const vector<unsigned int>& window) const;

#if _DEBUG_PREDICTORS
		virtual string toString() const;
#endif
};

#endif
