/**
 * @file
 * @brief A simple %Predictor which tries to predict the duration of staying in the current state.
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-08-18 16:40:15 +0200 (Mit, 18 Aug 2004) $
 * $Revision: 896 $
 *
 * @since    Wed Mar 19 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: SingleStepDuration.h 896 2004-08-18 14:40:15Z rene $
 */

#ifndef _SINGLESTEPDURATION_H
#define _SINGLESTEPDURATION_H

#include "../Predictor.h"

/**
 * @todo documentation
 */
typedef struct {
	//@{
	/** @todo documentation */
	unsigned long duration; // the time until the transition to context will occur
	double probability;
	//@}
} next_context_description;

/** @todo documentation */
typedef map<unsigned long, next_context_description> membershiplist_duration;

/**
 * @todo documentation
 */
class SingleStepDurationPredictor : public PredictorAlgorithm
{
	private:
		//@{
		/** @todo documentation */
		typedef vector<unsigned long> durationfrequencies;
		typedef map<unsigned long, durationfrequencies> nextcontexts;
		// for smoothening (which is equal to a convolution with a mask), we need to use floating point...
		typedef map<unsigned long, vector<double> > nextcontexts_smoothened;
		
		static const unsigned startfrequencieslength = 20;
		static const unsigned incrementfrequencieslength = 10;
		//@}

		/**
		 * @todo documentation
		 */
		struct contextinfo {
			//@{
			/** @todo documentation */
			nextcontexts transitions;
			// the number of times that this state was entered at all
			unsigned long entries;
			contextinfo() : entries(0) {}
			//@}
		};

		//@{
		/** @todo documentation */
		map<unsigned long, contextinfo> frequencies;
		unsigned long curContext;
		unsigned long curDuration;
		bool firstEntry;

		// these are configurable
		unsigned int smoothingWindowWidth;
		double smoothingFactor;
		double peakDifference;
		//@}

	public:
		/** @todo documentation */
		SingleStepDurationPredictor(predictorparams &params);

		/** @todo documentation */
		unsigned long getCurrentContextDuration() const;
		/** @todo documentation */
		membershiplist_duration getNextContextsWithDuration(unsigned int offset=0) const;
		// this method is more flexible, the previous one is just a wrapper around it
		// this one can be used to specify the context and duration to start at (which is used in conjunction with the ALZ predictor, which supplies the new context -- see main-prediction.cpp)
		membershiplist_duration getNextContextsWithDuration(unsigned long startContext, unsigned long startDuration, unsigned int offset=0) const;

		virtual void addContexts(const membershiplist* contexts, time_t time);
		virtual void addContext(unsigned long contextId, time_t time);

		virtual unsigned long getNextContext() const;
		virtual membershiplist getNextContexts() const;

		virtual contexttrajectory getContextTrajectory(unsigned int start, unsigned int end) const;

		virtual unsigned long getContextAt(time_t time) const;
		virtual membershiplist getContextsAt(time_t time) const;

		virtual string serialize() const;
		virtual void unserialize(string data);

#if _DEBUG_PREDICTORS
		virtual string toString() const;
#endif
};

#endif
