/**
 * @file
 * @brief %Predictor windows implementation
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-01-29 11:03:05 +0100 (Don, 29 Jän 2004) $
 * $Revision: 567 $
 *
 * @since    Wed Mar 19 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: WindowsPredictor.cpp 567 2004-01-29 10:03:05Z harald $
 */

#include "Predictor.h"

Predictor::Predictor(const string &name, const predictorparams &params)
{
	getPredictor_t getPredictor = NULL;

	pa = NULL;

#ifdef _WIN32_WCE
	size_t len = name.size() + 1;
	wchar_t *buffer = new wchar_t[mbstowcs(NULL, name.c_str(), len)];
	mbstowcs(buffer, name, len);

	if ((dll = LoadLibrary(buffer)) == NULL) {
#else
	if ((dll = LoadLibrary(name.c_str())) == NULL) {
#endif

		LPTSTR lpMsgBuf;

		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			GetLastError(),
			0,
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
		);

		MessageBox(NULL, lpMsgBuf, TEXT("Error"), MB_OK|MB_ICONINFORMATION);
		LocalFree(lpMsgBuf);
	} else if ((getPredictor = (getPredictor_t) GetProcAddress(dll, TEXT("getPredictor"))) == NULL) {
		FreeLibrary(dll);
	} else {
		pa = getPredictor(params);
		dll_init_log(dll);
	}

#ifdef _WIN32_WCE
	delete buffer;
#endif
}

Predictor::~Predictor()
{
	FreeLibrary(dll);
}