/**
 * @file
 * @brief Bluetooth feature JNI wrapper
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Tue Mar 18 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: JNI_Bluetooth.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "../../../Features/Bluetooth/BluetoothLinux.h"

// the following methods are for the P2PC Framework
#include <jni.h>

/** Prototype */
FeatureProvider* getProvider(const providerparams &params);

/** Feature provider singleton */
static BluetoothLinuxFeatureProvider* JNI_provider = NULL;

extern "C"
{

/** @todo documentation */
JNIEXPORT void JNICALL Java_at_jku_bluetooth_LinuxBluetoothAdapter_nativeInit
  (JNIEnv *, jobject)
{
	featureparams params;

//        printf("nativeInit called\n");
	if (JNI_provider == NULL) {
	    JNI_provider = (BluetoothLinuxFeatureProvider*) getProvider(params);
	}
}

/** @todo documentation */
JNIEXPORT void JNICALL Java_at_jku_bluetooth_LinuxBluetoothAdapter_nativeStart
  (JNIEnv *, jobject)
{
	// do nothing - the thread is started in the FeatureProvider constructor
}

/** @todo documentation */
JNIEXPORT void JNICALL Java_at_jku_bluetooth_LinuxBluetoothAdapter_nativeStop
  (JNIEnv *, jobject)
{
//    printf("nativeStop called\n");
	if (JNI_provider == NULL) {
        return;
	}
    JNI_provider->cancel();
}

#ifdef _DEBUG_FEATURES
/** @todo documentation */
JNIEXPORT jboolean JNICALL Java_at_jku_bluetooth_LinuxBluetoothAdapter_nativeIsAddressValid
  (JNIEnv *jenv, jobject, jstring pAdr)
{
//    printf("nativeIsAddressInRange called, JNI_provider is now %d\n", JNI_provider);
	if (JNI_provider == NULL) {
        return false;
	}

    // first check for null strings
	if (pAdr == NULL) {
		return false;
	}

	jboolean iscopy;
	const char *adr = jenv->GetStringUTFChars(pAdr, &iscopy);
	string mac = adr;
	bool found = false;

//    printf("nativeIsAddressInRange called with '%s'\n", adr);

    JNI_provider->nextSample(0);
	BluetoothPeersFeature* peerList = (BluetoothPeersFeature*) JNI_provider->getSample("Peers");

	if (peerList != NULL) {
	    const stringvector& macs = peerList->getListValue();
	    for (unsigned int i=0; !found && i<macs.size(); i++)
	    {
//                printf("found '%s'\n", macs[i].c_str());
			if (macs[i] == mac) {
				found = true;
			}
	    }

	    delete peerList;
	}

	return found;
}

/** @todo documentation */
JNIEXPORT jobjectArray JNICALL Java_at_jku_bluetooth_LinuxBluetoothAdapter_nativeGetAddressesInRange
  (JNIEnv *jenv, jobject)
{
	if (JNI_provider == NULL) {
        return false;
	}

    JNI_provider->nextSample(0);
	BluetoothPeersFeature* peerList = (BluetoothPeersFeature*) JNI_provider->getSample("Peers");

     jobjectArray ret = NULL;

	if (peerList != NULL) {
	    const stringvector& macs = peerList->getListValue();
	    jenv->NewObjectArray(macs.size(),
		jenv->FindClass("java/lang/String"), NULL);
	    for (unsigned int i=0;i<macs.size(); i++)
	    {
		jenv->SetObjectArrayElement(ret, i, jenv->NewStringUTF(macs[i].c_str()));
	    }
	 }

	return ret;
}
#endif

/** @todo documentation */
JNIEXPORT jstring JNICALL Java_at_jku_bluetooth_LinuxBluetoothAdapter_nativeGetLastMessage
  (JNIEnv *, jobject)
{
	return NULL;
}

/** @todo documentation */
JNIEXPORT jstring JNICALL Java_at_jku_bluetooth_LinuxBluetoothAdapter_nativeGetHardwareAddress
  (JNIEnv *jenv, jobject)
{
	int ctl;
	struct hci_dev_list_req *dl;
	struct hci_dev_req *dr;
	struct hci_dev_info di;
	int i;
	bdaddr_t bdaddr;

	/* Open HCI socket  */
	if ((ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
		perror("Can't open HCI socket.");
		return NULL;
	}

	if (!(dl = (hci_dev_list_req*) malloc(HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t))) ) {
		perror("Can't allocate memory");
		return NULL;
	}
	dl->dev_num = HCI_MAX_DEV;
	dr = dl->dev_req;

	if (ioctl(ctl, HCIGETDEVLIST, (void*)dl) ) {
		perror("Can't get device list");
                return NULL;
	}
	for (i=0; i< dl->dev_num; i++) {
		di.dev_id = (dr+i)->dev_id;
		if( ioctl(ctl, HCIGETDEVINFO, (void*)&di) )
			continue;

		baswap(&bdaddr, &(di.bdaddr));

		// TBD: there should be an array returned !!!!
		delete(dl);
		return jenv->NewStringUTF(batostr(&bdaddr));
	}
	delete(dl);

	return NULL;
}

}
