/**
 * @file
 * @brief JNI wrapper
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-08-29 12:14:03 +0200 (Son, 29 Aug 2004) $
 * $Revision: 905 $
 *
 * @since    Thu May 8 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: JNI_Context.cpp 905 2004-08-29 10:14:03Z rene $
 */

#include "../Classifiers/Classifier.h"
#include "../ConfigReader/ConfigReader.h"

#include <jni.h>

#ifdef _WINDOWS
#define PATH "\\"
#endif

#ifdef _LINUX
#define PATH "./"
#endif

#ifdef _WIN32_WCE
#define PATH "\\"
#endif

// OK, I don't know why we need this, but taking it out makes the lib crash upon being loaded from Java !!!
#include <iostream>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * FeatureContainer instance
 *
 * @remarks also used by JNI_Samples.cpp
 */
FeatureContainer* fc;
/** ClassifierAlgorithm instance */
static ClassifierAlgorithm* ca;

void INIT_EXPORT library_initialize()
{
	fc = NULL;
	ca = NULL;
	//cout << "1\n" << flush;
	open_log(PATH"debug_logfile.txt");
	plog("P2P Intelligence library loaded\n");
}

void FINI_EXPORT library_finalize()
{
	if (fc != NULL)
		delete fc;
	if (ca != NULL)
		delete ca;
	close_log();
}

/** @todo documentation */
JNIEXPORT jboolean JNICALL Java_at_jku_intelligence_context_Context_nativeInit
  (JNIEnv *jenv, jclass, jstring configFilename, jstring persistFilename, jstring gngPersistFilename)
{
	if (fc == NULL && ca == NULL)
	{
		const char *config = jenv->GetStringUTFChars(configFilename, 0);
		const char *persist = jenv->GetStringUTFChars(persistFilename, 0);
		const char* gngPersist = jenv->GetStringUTFChars(gngPersistFilename, 0);
		plog("P2P Intelligence library nativeInit called with '%s', '%s' and '%s'\n", config, persist, gngPersist);

		ConfigReader* r = new ConfigReader(config);
		plog("P2P Intelligence library loaded config\n");

		fc = new FeatureContainer(r);
		plog("P2P Intelligence library loaded FeatureContainer\n");

		const pair<string, classifierparams> *foo = r->getClassifier();
		assert(foo != NULL);
		plog("alg: '%s'\n", (*foo).first.c_str());
		ca = new Classifier((*foo).first, (*foo).second);
		assert(ca != NULL);
		plog("P2P Intelligence library loaded clustering algorithm '%s'\n", (*foo).first.c_str());

		delete r;

		ca->init(fc);

		jenv->ReleaseStringUTFChars(configFilename, config);
		jenv->ReleaseStringUTFChars(persistFilename, persist);
		jenv->ReleaseStringUTFChars(persistFilename, gngPersist);
		plog("P2P Intelligence library nativeInit complete\n");
		return true;
	}
	else
		return false;
}

/** @todo documentation */
JNIEXPORT jlong JNICALL Java_at_jku_intelligence_context_Context_nativeNextSample
  (JNIEnv *, jclass)
{
	if (ca != NULL)
		return ca->nextSample();
	else
		return -1;
}


/** @todo documentation */
JNIEXPORT jint JNICALL Java_at_jku_intelligence_context_Context_nativeGetNumClusters
  (JNIEnv *, jclass)
{
	if (ca != NULL) {
		return (jint) ca->getClusterMembership(fc->getSampleVector()).size();
	} else {
		return -1;
	}
}

/** @todo documentation */
JNIEXPORT void JNICALL Java_at_jku_intelligence_context_Context_nativeGetClusterMembership
  (JNIEnv *jenv, jclass, jlongArray classes, jdoubleArray memberships)
{
	if (ca != NULL) {
		membershiplist ml = ca->getClusterMembership(fc->getSampleVector());

		int i=0;
		jsize len = jenv->GetArrayLength(classes);
		assert(len == jenv->GetArrayLength(memberships));
		assert((unsigned) len == ml.size());

		jlong *classesBody = jenv->GetLongArrayElements(classes, 0);
		jdouble *membershipsBody = jenv->GetDoubleArrayElements(memberships, 0);
		for (membershiplist::const_iterator iter = ml.begin(); iter != ml.end(); iter++) {
			classesBody[i] = (*iter).first;
			membershipsBody[i] = (*iter).second;
			i++;
		}
		jenv->ReleaseLongArrayElements(classes, classesBody, 0);
		jenv->ReleaseDoubleArrayElements(memberships, membershipsBody, 0);
	}
}


#ifdef __cplusplus
}
#endif
