/**
 * @file
 * @brief JNI wrapper
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-08-30 23:27:49 +0200 (Mon, 30 Aug 2004) $
 * $Revision: 915 $
 *
 * @since    Thu May 8 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: JNI_Samples.cpp 915 2004-08-30 21:27:49Z rene $
 */

#include "../ConfigReader/ConfigReader.h"
#include "../Features/AbstractString.h"
#include "../Features/Numerical.h"
#include "../FeatureContainer/FeatureContainer.h"

#include <jni.h>

#ifdef _WINDOWS
#define PATH "\\"
#endif

#ifdef _LINUX
#define PATH "./"
#endif

#ifdef _WIN32_WCE
#define PATH "\\"
#endif

// OK, I don't know why we need this, but taking it out makes the lib crash upon being loaded from Java !!!
#include <iostream>

#ifdef __cplusplus
extern "C" {
#endif

/** FeatureContainer instance */
extern FeatureContainer* fc;

/** Cast a Java object reference to an object */
#define cast_to(type, ref) *((type **) &ref)

/** @todo documentation */
JNIEXPORT jdouble JNICALL Java_at_jku_intelligence_samples_Sample_nativeGetPosition
  (JNIEnv *, jobject, jint objRef)
{
	Feature* f = cast_to(Feature, objRef);
	assert(f != NULL);

	return (jdouble) f->getPosition();
}

/** @todo documentation */
JNIEXPORT jint JNICALL Java_at_jku_intelligence_samples_Sample_nativeGetType
  (JNIEnv *, jobject, jint objRef)
{
	Feature* f = cast_to(Feature, objRef);
	assert(f != NULL);

	return (jint) f->getType();
}

/** @todo documentation */
JNIEXPORT jstring JNICALL Java_at_jku_intelligence_samples_Sample_nativeGetName
  (JNIEnv *jenv, jobject, jint objRef)
{
	Feature* f = cast_to(Feature, objRef);
	assert(f != NULL);

	return jenv->NewStringUTF(f->getName().c_str());
}

/** @todo documentation */
JNIEXPORT void JNICALL Java_at_jku_intelligence_samples_Sample_nativeFreeObject
  (JNIEnv *, jobject, jint /*objRef*/)
{
	// we are no longer allowed to free those objects - getSampleVector takes care of that !
	/*Feature* f = (Feature*) objRef;
	assert(f != NULL);

	delete f;*/
}

#if _DEBUG_FEATURES
JNIEXPORT jstring JNICALL Java_at_jku_intelligence_samples_Sample_nativeToString
  (JNIEnv *jenv, jobject, jint objRef)
{
	Feature* f = (Feature*) objRef;
	assert(f != NULL);

	return jenv->NewStringUTF(f->toString().c_str());
}
#endif

/** @todo documentation */
JNIEXPORT jint JNICALL Java_at_jku_intelligence_samples_NumericalDiscreteSample_nativeGetVal
  (JNIEnv *, jobject, jint objRef)
{
	NumericalDiscreteFeature* f = cast_to(NumericalDiscreteFeature, objRef);
	assert(f != NULL);

	return (jint) f->getVal();
}

/** @todo documentation */
JNIEXPORT jfloat JNICALL Java_at_jku_intelligence_samples_NumericalContinuousSample_nativeGetVal
  (JNIEnv *, jobject, jint objRef)
{
	NumericalContinuousFeature* f = cast_to(NumericalContinuousFeature, objRef);
	assert(f != NULL);

	return (jfloat) f->getVal();
}

/** @todo documentation */
JNIEXPORT jstring JNICALL Java_at_jku_intelligence_samples_StringSample_nativeGetVal
  (JNIEnv *jenv, jobject, jint objRef)
{
	AbstractStringFeature* f = cast_to(AbstractStringFeature, objRef);
	assert(f != NULL);

	return jenv->NewStringUTF(f->getVal().c_str());
}

/** @todo documentation */
JNIEXPORT jobjectArray JNICALL Java_at_jku_intelligence_samples_StringListSample_nativeGetValues
  (JNIEnv *jenv, jobject, jint objRef)
{
	AbstractStringListFeature* f = cast_to(AbstractStringListFeature, objRef);
	assert(f != NULL);

//	plog("P2P Intelligence library StringListSample.nativeGetValues called for object %d\n", objRef);
	const stringvector& names = f->getListValues();

	jobjectArray ret = jenv->NewObjectArray((jsize) names.size(), jenv->FindClass("java/lang/String"), NULL);
	for (unsigned int i=0; i<names.size(); i++) {
		jenv->SetObjectArrayElement(ret, i, jenv->NewStringUTF(names[i].c_str()));
//		plog("++++ %s\n", names[i].c_str());
	}

	return ret;
}

/** @todo documentation */
JNIEXPORT jboolean JNICALL Java_at_jku_intelligence_samples_SampleContainer_nativeInit
  (JNIEnv *jenv, jclass, jstring configFilename, jstring persistFilename)
{
	if (fc == NULL)
	{
		const char *config = jenv->GetStringUTFChars(configFilename, 0);
		const char *persist = jenv->GetStringUTFChars(persistFilename, 0);
		plog("P2P Intelligence library nativeInit called with '%s' and '%s'\n", config, persist);
		ConfigReader* r = new ConfigReader(config);
		plog("P2P Intelligence library loaded config\n");
		fc = new FeatureContainer(r);
		plog("P2P Intelligence library loaded FeatureContainer\n");
		delete r;
		jenv->ReleaseStringUTFChars(configFilename, config);
		jenv->ReleaseStringUTFChars(persistFilename, persist);
		plog("P2P Intelligence library nativeInit complete\n");
		return true;
	}
	else
		return false;
}

/** @todo documentation */
JNIEXPORT void JNICALL Java_at_jku_intelligence_samples_SampleContainer_nativeNextSamples
  (JNIEnv *, jclass)
{
	if (fc != NULL)
		fc->nextSample();
//	plog("P2P Intelligence library nextSample called\n");
}

/** @todo documentation */
JNIEXPORT jobjectArray JNICALL Java_at_jku_intelligence_samples_SampleContainer_nativeGetSampleVector
  (JNIEnv *jenv, jclass)
{
	if (fc != NULL)
	{
//		plog("P2P Intelligence library getSample called\n");

		const featurevector* samples = fc->getSampleVector();
		jclass sampleClass = jenv->FindClass("at/jku/intelligence/samples/Sample");
		jmethodID sampleConstructor = jenv->GetMethodID(sampleClass, "<init>", "(I)V");

		jobjectArray ret = jenv->NewObjectArray((jsize) samples->size(), sampleClass, NULL);
		for (unsigned int i=0; i<samples->size(); i++) {
			jobject jSample = NULL;
			if ((*samples)[i] != NULL) {
				jSample = jenv->NewObject(sampleClass, sampleConstructor, (*samples)[i]);
			}
//			plog("---- %d\n", (*samples)[i]);
			jenv->SetObjectArrayElement(ret, i, jSample);
		}

		return ret;
	}
	else
		return NULL;
}


#ifdef __cplusplus
}
#endif
