/**
 * @file
 * @brief main
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-08-30 23:27:27 +0200 (Mon, 30 Aug 2004) $
 * $Revision: 914 $
 *
 * @since    Mon Mar 10 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: main-features.cpp 914 2004-08-30 21:27:27Z rene $
 */

//#define DO_SOAP
#define DEFAULT_SOAP_PORT 8000
#define DO_CLASSIFICATION
 
 
 
#include "util/defines.h"

#ifdef _LINUX
#include <signal.h>
#include <unistd.h>
#endif

#ifdef _SYMBIAN
#undef _FEATURE_EXPORT
#include "Features/Time/TimeFeature.h"
#include "Features/Wlan/Wlan.h"
#include "Features/Bluetooth/Bluetooth.h"
#include "Features/GSM/GSM.h"
#include "Features/Power/Power.h"
#include "Features/Audio/Audio.h"
#include "Features/ActiveWindow/ActiveWindow.h"
#endif

#include "ConfigReader/ConfigReader.h"
#include "FeatureContainer/LoggingFeatureContainer.h"

#include "Classifiers/Classifier.h"
//#include "Classifiers/gng/GNG.h"

#include "util/Thread.h"

// the function ns1__getActiveId needs to be defined even if we don't really start a SOAP server --> therefore include the headers unconditinally here
#include "trayicon/soapH.h"
#include "trayicon/ContextApplicationsBinding.nsmap"

//@{
/** @todo documentation */
static bool keepRunning = true;
static bool terminated = false;
#ifdef DO_CLASSIFICATION
static long curContextId = -1;
#endif
//@}

#if defined _WINDOWS && ! defined _WIN32_WCE
BOOL CtrlHandler(DWORD fdwCtrlType)
{
    switch (fdwCtrlType) {
        // Handle the CTRL+C signal.
        case CTRL_C_EVENT:
            keepRunning = false;
            return TRUE;

        // CTRL+CLOSE: confirm that the user wants to exit.
        case CTRL_CLOSE_EVENT:
            return TRUE;

        // Pass other signals to the next handler.
        case CTRL_BREAK_EVENT:
        case CTRL_LOGOFF_EVENT:
        case CTRL_SHUTDOWN_EVENT:

        default:
            return FALSE;
    }
}

#define PATH "\\"
#endif

#ifdef _LINUX
void signalhandler(int sig)
{
      switch(sig)     {
              /*case SIGFPE:
                      cerr << "-------------------------------------------------\n" << flush;
                      cerr << "Oops, floating point exception caught !\n" << flush;
                      cerr << "-------------------------------------------------\n" << flush;
		      exit(1);
                      break;*/
              /*case SIGSEGV:
                      cerr << "-------------------------------------------------\n" << flush;
                      cerr << "Oops, segmentation violation caught !\n" << flush;
                      cerr << "-------------------------------------------------\n" << flush;
		      exit(1);
                      break;*/
              case SIGHUP:
              case SIGINT:
              case SIGPIPE:
              case SIGALRM:
              case SIGUSR1:
              case SIGUSR2:
                      plog("-------------------------------------------------\n");
                      plog("Process termination requested, doing clean shutdown\n");
                      plog("-------------------------------------------------\n");
		      keepRunning = false;
                      break;
	}
}


#define PATH "/etc/proca/"
#endif

#ifdef _WIN32_WCE
#define PATH "\\"
#endif

#ifdef _SYMBIAN
#define PATH "d:\\"
#endif

#define DO_CLASSIFICATION

#ifndef _SYMBIAN
/**
 * @todo documentation
 */
class Scanner : Thread
{
private:
	const string path;
public:
	Scanner(string path) : path(path)
	{
		plog("Scanner starting now\n");

		this->start();
	}

	virtual void run()
	{
		//plog("Scanner: Creating FeatureContainer\n");

		FeatureContainer *f;
		ConfigReader *r = new ConfigReader((path + "config.xml").c_str());
		map<string, string> globals = (*r->getGlobals());

		assert(! globals["persist"].empty());
		assert(! globals["sample_delay"].empty());

		if (globals.count("features_log") > 0) {
			f = new LoggingFeatureContainer(r);
		} else {
			f = new FeatureContainer(r);
		}

		int delay = atoi(globals["sample_delay"].c_str());

		plog("Scanner: FeatureContainer created\n");
		plog("Scanner: getting feature vector\n");

		featurevector features = f->getFeatureVector();

#ifdef DO_CLASSIFICATION
		plog("Scanner: Creating Classifier\n");
		const pair<string, classifierparams> *foo = r->getClassifier();
		Classifier *ca = new Classifier((*foo).first, (*foo).second);
		plog("Scanner: loaded '%s'\n", (*foo).first.c_str());
		ca->init(f);
		plog("Scanner: initialized '%s'\n", (*foo).first.c_str());

		FILE* member_file = NULL;
		if (! globals["classes_log"].empty()) {
			member_file = fopen(globals["classes_log"].c_str(), "a");
			assert(member_file != NULL);
		}
#endif

		delete r;

		plog("Scanner: starting main loop\n");


		while (keepRunning) {
                        plog("Scanner: Calling nextSample\n");
			f->nextSample();
                        plog("Scanner: Getting sample vector\n");
			const featurevector* sample = f->getSampleVector();
#if _DEBUG_FEATURES
			for (unsigned int i=0; i<sample->size(); i++) {
				if ((*sample)[i] != NULL) {
					plog("%s\n", (*sample)[i]->toString().c_str());
				} else {
					plog("NULL\n");
				}
			}
#endif

#ifdef DO_CLASSIFICATION
//			plog("Scanner: Performing classification\n");
			curContextId = ca->nextSample(sample);
			plog("Set curContextId to %d\n", curContextId);

#ifdef _GNG_H
			GNG* gng = ((GNG *) ((Classifier *) ca)->getCa());
			meta_cluster_list meta_clusters = gng->getMetaClusters();
//			plog("%i meta clusters / %d clusters total\n", meta_clusters.size(), gng->getNumberNodes());
#endif

			if (member_file != NULL) {
//				plog("Scanner: Writing cluster memberships\n");
				membershiplist ml = ca->getClusterMembership(sample);
				for (membershiplist::iterator iter = ml.begin(); iter != ml.end(); iter++) {
					fprintf(member_file, "%.17f;", (*iter).second);
				}
				fprintf(member_file, "\n");
			}
#endif

//			plog("Waiting\n");
			for (int j=0; j<delay*10 && keepRunning; j++) {
				Thread::sleep(100);
			}
		}

		plog("Scanner: exiting\n");

#ifdef DO_CLASSIFICATION
		if (member_file != NULL) {
			fclose(member_file);
		}
#endif

		plog("Scanner: Destroying FeatureContainer\n");

		for (unsigned int j=0; j<features.size(); j++) {
			delete features[j];
		}
		delete f;

		plog("Scanner: FeatureContainer destroyed\n");

		terminated = true;
	}
};
#endif

#ifdef _SYMBIAN
int getConsoleChar();

featurevector curSample;

#include <es_sock.h>
#include <in_sock.h>

class TcpThread : public Thread
{
private:

public:
	TcpThread() {
		this->start();
	}

	virtual void run() {
		RSocketServ socksvr;
		RSocket sock, conn;
		TRequestStatus iStatus;

		TInt err = socksvr.Connect();
		User::LeaveIfError(err);

		err = sock.Open(socksvr, KAfInet, KSockStream, KProtocolInetTcp);
		User::LeaveIfError(err);

		TInetAddr addr(KInetAddrAny, 4000);
		err = sock.Bind(addr);
		User::LeaveIfError(err);

		err = sock.Listen(10);
		User::LeaveIfError(err);

		while (!terminated) {
			err = conn.Open(socksvr);
			User::LeaveIfError(err);

			sock.Accept(conn, iStatus);
			// really block here until we get an request
			User::WaitForRequest(iStatus); 

			if (iStatus==KErrNone) {
				char* buf = "Test\n";
				TPtrC8 b((const TText8*) buf, strlen(buf)+1);
				conn.Send(b, 0, iStatus);
				// also block here until the send is complete
				User::WaitForRequest(iStatus); 
				conn.Close();
			}
		}
	}
};

/*
class TcpServer : public CActive
{
	public: 
		TcpServer() : CActive(CActive::EPriorityStandard) {}
		~TcpServer()
		{
			CActive::Cancel();
		}

		void ConstructL()
		{
			CActiveScheduler::Add(this);
		}

		void Start()
		{
			StartConnection();
			SetActive();
		}

		void RunL()
		{
	if (iStatus==KErrNone)
		{
		switch (iChannelState)
			{
		case EChannelListening:
			{
			// Connection was made successfully
			// Send some data to client
			char* buf = "Test\n";
			TPtrC8 b((const TText8*) buf, strlen(buf));
			iChannelState=EChannelSending;
			conn.Send(b, 0, iStatus);
			SetActive();
			}
			break;
		case EChannelSending:
			{
			// Data was sent successfully
			iChannelState=EChannelShutingDown;
			conn.Close();

			conn.Close();

			StartConnection();
			SetActive();	// start again
			}
			break;
		default:
			break;
			}
		}
		}
		void DoCancel()
		{
	switch (iChannelState)
		{
		case EChannelSending:
		case EChannelListening:
			{
			sock.CancelAll();
			sock.Close();
			conn.CancelAll();
			conn.Close();
			iChannelState=EChannelIdle;
			}
			break;
		default:
			break;
		}
		}

	private:
		void StartConnection() 
		{
			TInt err = socksvr.Connect();
			User::LeaveIfError(err);

			err = sock.Open(socksvr, KAfInet, KSockStream, KProtocolInetTcp);
			User::LeaveIfError(err);
			err = conn.Open(socksvr);
			User::LeaveIfError(err);

			TInetAddr addr(KInetAddrAny, 4000);
			err = sock.Bind(addr);
			User::LeaveIfError(err);

			err = sock.Listen(1);
			User::LeaveIfError(err);

			sock.Accept(conn, iStatus);
			iChannelState=EChannelListening;
		}

	private:
		RSocketServ socksvr;
		RSocket sock, conn;
	enum TState
		{
		EChannelIdle,EChannelListening,EChannelConnected,
		EChannelShutingDown,EChannelSending,EChannelReceiving
		};
		TState iChannelState;
		TRequestStatus iStatus;
};*/
#endif

/** Main */
int Main(const string path)
{
	open_log((path + "debug_logfile.txt").c_str());
	plog("Start\n");

#ifndef _SYMBIAN
	Scanner scanner(path);
	plog("Main thread: scanner created\n");
#else
	/*TcpServer serv;
	serv.ConstructL();
	serv.Start();*/
	TcpThread server;

	// hard-coded setup for Symbian
	featureparams param;
	vector<FeatureProvider*> providers;
	providers.push_back(new TimeFeatureProvider(param));

	featurevector features1, features2;
	features1.push_back(providers[0]->getFeature("Timestamp"));
	features2.push_back(providers[0]->getFeature("Timestamp"));
#endif

#ifdef DO_SOAP
	ConfigReader *r = new ConfigReader(PATH"config.xml");
	map<string, string> globals = (*r->getGlobals());
	unsigned int soap_port = DEFAULT_SOAP_PORT;
	if (globals.count("soap_port") > 0) {
		soap_port = atoi(globals["soap_port"].c_str());
	}
	// start the embedded HTTP server
	struct soap soap; 
	int m=-1, s; // master and slave sockets 
	soap_init(&soap); 
	m = soap_bind(&soap, "localhost", soap_port, 100); 
	if (m < 0) 
		soap_print_fault(&soap, stderr); 
/*	else
		plog("Socket connection successful: master socket = %d\n", m); */
#endif

	while (!terminated) {
#ifdef DO_SOAP
		// handle incoming SOAP requests via the embedded HTTP server
		if (m >= 0) {
			// beware: this blocks !
			s = soap_accept(&soap); 
			if (s < 0)  { 
				soap_print_fault(&soap, stderr); 
				break;
			} 
/*			plog("accepted connection from IP=%d.%d.%d.%d socket=%d", 
				(soap.ip >> 24)&0xFF, (soap.ip >> 16)&0xFF, (soap.ip >> 8)&0xFF, soap.ip&0xFF, s); */
			soap_serve(&soap); // process RPC request 
//			plog("request served\n"); 
			soap_destroy(&soap); // clean up class instances 
			soap_end(&soap); // clean up everything and close socket 
		} 
#endif

#ifdef _WIN32_WCE
		MSG msg;
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
			if (msg.message == 31 && keepRunning) {
				keepRunning = false;
			}
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
#endif
#ifdef _SYMBIAN
		for (vector<FeatureProvider*>::iterator iter=providers.begin(); iter!=providers.end(); iter++) 
			(*iter)->nextSample(0);

		// hard-coded for now
		for (featurevector::iterator iter = curSample.begin(); iter != curSample.end(); iter++)
			if (*iter != NULL)
				delete (*iter);
		curSample.clear();
		curSample.push_back(providers[0]->getSample("Timestamp"));

		for (featurevector::iterator iter = curSample.begin(); iter != curSample.end(); iter++)
			if (*iter != NULL)
				plog("-- %s\n", (*iter)->toString().c_str());

		int key = getConsoleChar();
		if (key == 63555) {
			plog("Signalling scanner thread to exit\n");
/*			keepRunning = false;
			while (!terminated) {
				plog("Waiting for scanner thread...\n");
				Thread::sleep(500);
			}*/
			terminated = true;
		}
		else
			plog("Press right menu key to exit\n");

		RSocketServ socksvr;
		RSocket conn;
		TRequestStatus iStatus;

		TInt err = socksvr.Connect();
		User::LeaveIfError(err);

		err = conn.Open(socksvr, KAfInet, KSockStream, KProtocolInetTcp);
		User::LeaveIfError(err);

		TInetAddr addr(KInetAddrLoop, 4000);
		conn.Connect(addr, iStatus);
		User::WaitForRequest(iStatus);

		char buf[100];
		TPtr8 b((TText8*) buf, sizeof(buf));
		conn.Recv(b, 0, iStatus);
		// also block here until the send is complete
		User::WaitForRequest(iStatus); 
		plog("Received from TCP: '%s'\n", buf);
		conn.Close();
#endif
	
		Thread::sleep(100);
	}

#ifdef DO_SOAP
	soap_done(&soap); // close master socket 
#endif

#ifdef _SYMBIAN
	for (featurevector::iterator iter = curSample.begin(); iter != curSample.end(); iter++) 
		if (*iter != NULL)
			delete (*iter);

	for (vector<FeatureProvider*>::iterator iter=providers.begin(); iter!=providers.end(); iter++) 
		delete (*iter);
#endif

	plog("Terminating main program now\n");
	close_log();

//	DUMP_LEAKS();

	return 0;
}

/** Implementation of the SOAP method. */
SOAP_FMAC5 int SOAP_FMAC6 ns1__getActiveId(struct soap* soapP, struct ns1__getActiveIdResponse &param) 
{
//	plog("SOAP method getActiveId called: ");
	if (curContextId >= 0) {
		param.contextId = curContextId;
//		plog("returning %d\n", param.contextId);
		return SOAP_OK;
	}
	else {
//		plog("curContextId not set\n");
		return soap_sender_fault(soapP, "Current context not available", "I can only return a context when I have one"); 
	}
}

#ifdef _WIN32_WCE
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	return Main();
}
#endif

#if (defined(_WINDOWS) || defined(_LINUX))
int main (int argc, char* argv[])
{
#if defined _WINDOWS && ! defined _WIN32_WCE
	//_CrtSetBreakAlloc(107);
	SetConsoleCtrlHandler((PHANDLER_ROUTINE) CtrlHandler, TRUE);
#endif
#ifdef _LINUX
	signal(SIGFPE, signalhandler);
	signal(SIGSEGV, signalhandler);
	signal(SIGHUP, signalhandler);
	signal(SIGINT, signalhandler);
	signal(SIGPIPE, signalhandler);
	signal(SIGALRM, signalhandler);
	signal(SIGUSR1, signalhandler);
	signal(SIGUSR2, signalhandler);
#endif

	string path;
	if (argc >= 2)
		path = argv[1];
	else
		path = PATH;

	return Main(path);
}
#endif

#ifdef _SYMBIAN
_LIT(KTxtAppName, "Proca");

GLDEF_C TInt E32Main() // main function called by E32
{
	__UHEAP_MARK;
	CTrapCleanup* cleanup = CTrapCleanup::New(); // get a clean-up stack
	TRAPD(error, Main());
	__ASSERT_ALWAYS(!error, User::Panic(KTxtAppName, error));
	delete cleanup; // destroy the clean-up stack
	CloseSTDLIB();
	__UHEAP_MARKEND;
	return 0;
}
#endif
