/**
 * @file
 * @brief main
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-08-30 22:03:46 +0200 (Mon, 30 Aug 2004) $
 * $Revision: 911 $
 *
 * @since    Wed Dec 17 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: main-prediction.cpp 911 2004-08-30 20:03:46Z rene $
 */

#include "Predictors/Statistics.h"
#include "Predictors/ActiveLezi/ActiveLezi.h"
#include "Predictors/Simple/AveragePredictor.h"
#include "Predictors/Simple/SingleStepDuration.h"
#include "Predictors/HMM/HMM.h"
#include <assert.h>

#ifdef _WINDOWS
#define PATH "\\"
#endif

#ifdef _LINUX
#define PATH "./"
#endif

#ifdef _WIN32_WCE
#define PATH "\\"
#endif

/** @todo documentation */
class PredWrapper {
	public:
		PredictorAlgorithm* pred;
		
	private:
		//@{
		/** @todo documentation */
		unsigned int correctPreds, totalPreds;
		//@}

	public:
		/** @todo documentation */
		PredWrapper(PredictorAlgorithm* pred) : pred(pred) {correctPreds = totalPreds = 0;}
		/** Destructor */
		~PredWrapper() { delete pred; }

		// the current context is first predicted and then fed into the predictor for update
		/** @todo documentation */
		void predictAndUpdate(unsigned long curContext)
		{
			totalPreds++;
			if (pred->getNextContext() == curContext)
				correctPreds++;
			pred->addContext(curContext, 0);
		}

		/** @todo documentation */
		void predictAndUpdateMultiStep(vector<unsigned int>* values, unsigned int curPos, unsigned int multiStepOrder) {
			assert(values->size() > curPos + multiStepOrder);

			contexttrajectory traj = pred->getContextTrajectory(0, multiStepOrder);
			assert(multiStepOrder == traj.size());
			/*for (unsigned int i=0; i<traj.size(); i++) {
				//printf("%d/%d ", (*values)[curPos+i], traj[i]);
				totalPreds++;
				if (traj[i] == (*values)[curPos+i])
					correctPreds++;
			}*/
			// only use the nth step, but not all in between
			totalPreds++;
			if (traj[multiStepOrder-1] == (*values)[curPos+multiStepOrder-1])
				correctPreds++;
//			printf("\n");
			pred->addContext((*values)[curPos], 0);
		}

		/** @todo documentation */
		double getLoss() { return 1 - ((double) correctPreds) / totalPreds; }

		/** @todo documentation */
		const PredictorAlgorithm* getPred() { return pred; }
};

#ifdef _WIN32_WCE
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
#else
/** Main */
int main(int argc, char* argv[])
{
#endif

	open_log(PATH"debug_logfile.txt");

	if (argc < 2) {
		printf("Usage: prediction <clusters filename>\n");
		exit(1);
	}

	predictorparams emptyparams, windowparams2, windowparams3, windowparams5, windowparams7;
	windowparams2["window"] = "2";
	windowparams3["window"] = "3";
	windowparams5["window"] = "5";
	windowparams7["window"] = "7";

#if 0	
	/////////////////////// test ActiveLezi ////////////////////
	printf("-------------------------------------------------------------------\n");
	printf("Test 1: Active LeZi, Sequence from paper\n");
	char testString[] = "aaababbbbbaabccddcbaaaa";
	ActiveLezi pred1(emptyparams);
//	AveragePredictor simple1_2(2), simple1_3(3), simple1_5(5), simple1_7(7);
	PredWrapper simple1_2(new AveragePredictor(windowparams2)),
		simple1_3(new AveragePredictor(windowparams3)),
		simple1_5(new AveragePredictor(windowparams5)),
		simple1_7(new AveragePredictor(windowparams7));

	printf("Now doing online prediction:\n");
	unsigned int correctPredictions=0, incorrectPredictions=0;
	for (unsigned int i=0; i<strlen(testString); i++) {
		pred1.addContext(testString[i], 0);
		simple1_2.predictAndUpdate(testString[i]);
		simple1_3.predictAndUpdate(testString[i]);
		simple1_5.predictAndUpdate(testString[i]);
		simple1_7.predictAndUpdate(testString[i]);
		printf("Current symbol: '%c'\n", testString[i]);
		if (i+1<strlen(testString)) {
			bool correct = (pred1.getNextContext() == (unsigned int) testString[i+1]);
			printf("Prediction: '%c', real next symbol: '%c' --> %s\n", (unsigned char) pred1.getNextContext(), testString[i+1],
				(correct ? "OK" : "NOT OK"));
			if (correct)
				correctPredictions++;
			else
				incorrectPredictions++;
		}
	}
	printf("%s\n", pred1.toString().c_str());
	membershiplist lastpred = pred1.getNextContexts();
	for (membershiplist::const_iterator p = lastpred.begin(); p != lastpred.end(); p++)
		printf("Prediction: '%c' %f\n", (unsigned char) (*p).first, (*p).second);
	printf("Loss of ALZ predictor: %f\n", ((double) incorrectPredictions) / (correctPredictions + incorrectPredictions));
	printf("Loss of averaging (2) predictor: %f\n", simple1_2.getLoss());
	printf("Loss of averaging (3) predictor: %f\n", simple1_3.getLoss());
	printf("Loss of averaging (5) predictor: %f\n", simple1_5.getLoss());
	printf("Loss of averaging (7) predictor: %f\n", simple1_7.getLoss());
#endif

	/*for (unsigned int j=0; j<20; j++) {
		printf("\nNow doing another run:\n");
		correctPredictions=0; incorrectPredictions=0;
		for (unsigned int i=0; i<strlen(testString); i++) {
			pred.addContext(testString[i], 0);
			printf("Current symbol: '%c'\n", testString[i]);
			if (i+1<strlen(testString)) {
				bool correct = (pred.getNextContext() == testString[i+1]);
				printf("Prediction: '%c', real next symbol: '%c' --> %s\n", pred.getNextContext(), testString[i+1],
					(correct ? "OK" : "NOT OK"));
				if (correct)
					correctPredictions++;
				else
					incorrectPredictions++;
			}
		}
		printf("Confidence in this predictor: %f\n", ((double) correctPredictions) / (correctPredictions + incorrectPredictions));
	}*/
	/////////////////////// test ActiveLezi ////////////////////

	printf("\n");

	////////////////// Test incremental autocorrellation function ///////////////
	printf("-------------------------------------------------------------------\n");
	printf("Test 2: ACF, Single-Step from %s\n", argv[1]);
	FILE* memberships = fopen(argv[1], "r");
	assert(memberships != NULL);
	char line[512];

/*	unsigned int maxOrder=15000;
	Statistics stat1(maxOrder), stat2(maxOrder), stat3(maxOrder);*/

#if 0	
	PredWrapper pred2(new ActiveLezi(emptyparams));
	PredWrapper simple2_2(new AveragePredictor(windowparams2)),
		simple2_3(new AveragePredictor(windowparams3)),
		simple2_5(new AveragePredictor(windowparams5)),
		simple2_7(new AveragePredictor(windowparams7));
	PredWrapper dura2(new SingleStepDurationPredictor(emptyparams));
#endif
	// this will only work for the artificial test data !! beware when using with larger files !
//	double* cortest = new double[131040];
	vector<unsigned int> clusterTrajectory;

	unsigned long maxClusterID = -numeric_limits<unsigned int>::max();
	unsigned long minClusterID = numeric_limits<unsigned int>::max();
	while (fgets(line, 512, memberships) != NULL) {
		vector<double> values;
		char* time = strtok(line, ";"), *valstr;
		unsigned long timeval = atoi(time);
		// just to get one warning away
		(void) timeval;
		while ((valstr = strtok(NULL, ";")) != NULL)
			values.push_back(atof(valstr));
		// last value is the best mathing cluster
		values.pop_back();
		unsigned int curCluster = (unsigned int) values.back();
		values.pop_back();
		if (curCluster > maxClusterID)
			maxClusterID = curCluster;
		if (curCluster < minClusterID)
			minClusterID = curCluster;

/*		stat1.nextSample(values[0]);
		stat2.nextSample(values[1]);
		stat3.nextSample(values[2]);
*/
//		cortest[stat1.getNum()-1] = values[0];

		clusterTrajectory.push_back(curCluster);
#if 0
		pred2.predictAndUpdate(curCluster);
		simple2_2.predictAndUpdate(curCluster);
		simple2_3.predictAndUpdate(curCluster);
		simple2_5.predictAndUpdate(curCluster);
		simple2_7.predictAndUpdate(curCluster);
		dura2.predictAndUpdate(curCluster);

/*		if (stat1.getNum() % 1000 == 0) {
			printf("%ld(%d/%f) ", stat1.getNum(), ((ActiveLezi*) pred2.getPred())->getMaxDepth(), pred2.getAccuracy());
			fflush(stdout);
		}*/
#endif	
	}
#if 0
	printf("%d samples, min=%ld, max=%ld\n", clusterTrajectory.size(), minClusterID, maxClusterID);
	fclose(memberships);
/*	printf("meta-cluster 1: %ld values, average=%f, variance=%f\n", stat1.getNum(), stat1.getMean(), stat1.getVariance());
	printf("                isIidAcf=%d, isIidTurningPoint=%d, isIidDifferenceSign=%d, MA order=%d\n",
		stat1.isIidAcf(), stat1.isIidTurningPoint(), stat1.isIidDifferenceSign(), stat1.getMaModelOrder());
	printf("meta-cluster 2: %ld values, average=%f, variance=%f\n", stat2.getNum(), stat2.getMean(), stat2.getVariance());
	printf("                isIidAcf=%d, isIidTurningPoint=%d, isIidDifferenceSign=%d, MA order=%d\n",
		stat1.isIidAcf(), stat2.isIidTurningPoint(), stat2.isIidDifferenceSign(), stat2.getMaModelOrder());
	printf("meta-cluster 3: %ld values, average=%f, variance=%f\n", stat3.getNum(), stat3.getMean(), stat3.getVariance());
	printf("                isIidAcf=%d, isIidTurningPoint=%d, isIidDifferenceSign=%d, MA order=%d\n",
		stat1.isIidAcf(), stat3.isIidTurningPoint(), stat3.isIidDifferenceSign(), stat3.getMaModelOrder());*/
/*	printf("Covariances of meta-cluster 2:\n");
	for (unsigned int i=0; i<maxOrder; i++)
		printf("%05d: %f\n", i, stat2.getAcf(i));*/
	printf("Loss of ALZ predictor: %f\n", pred2.getLoss());
	printf("Loss of averaging (2) predictor: %f\n", simple2_2.getLoss());
	printf("Loss of averaging (3) predictor: %f\n", simple2_3.getLoss());
	printf("Loss of averaging (5) predictor: %f\n", simple2_5.getLoss());
	printf("Loss of averaging (7) predictor: %f\n", simple2_7.getLoss());
	printf("Loss of single step duration predictor: %f\n", dura2.getLoss());

/*	printf("\nMulti-Step transitions:\n");
	printf("%s", ((SingleStepDurationPredictor*) dura2.getPred())->toString().c_str());*/

	printf("\n");
#endif
	////////////////// Test incremental autocorrellation function ///////////////
	//Statistics::testIncrementalAcf(maxOrder, cortest, 131040, 0.001);
	
#if 0
	// this is the straight-forward approach for using HMMs for discrete time series description, as also described in
	// "Improving the Performance of Action Prediction through Identification of Abstract Tasks", Sira Panduranga Rao & Diana J. Cook
// HACK HACK HACK HACK because of extreme memory leak in the HMM lib !!!!	
	printf("-------------------------------------------------------------------\n");
	printf("Test 2.5: HMM, Batch Training, Single-Step with %s\n", argv[1]);
	unsigned int train_size = clusterTrajectory.size() * 4 / 5;
	printf("Training set has size %d, test set is %d\n", train_size, clusterTrajectory.size() - train_size);
	HMM* hmm;
#ifdef THIS_WOULD_BE_CORRECT	
	for (unsigned int numHidden=1; numHidden<20; numHidden++) {
#else
	if (argc > 2) {
		unsigned int numHidden = atoi(argv[2]);
#endif
		printf("Trying HMM with %d hidden units\n", numHidden);
		hmm = new HMM(numHidden, maxClusterID+1);
		hmm->addContextTrajectory(clusterTrajectory, 0, train_size);
		printf("Training done, now doing single-step prediction\n");
		unsigned int numCorrect=0, numIncorrect=0;
		vector<unsigned int> window;
		for (unsigned int i=train_size; i<clusterTrajectory.size(); i++) {
			window.clear();
			for (unsigned int j=i-30; j<i; j++)
				window.push_back(clusterTrajectory[j]);
			unsigned long predCluster = hmm->getNextContext(window);
			if (clusterTrajectory[i] == predCluster)
				numCorrect++;
			else
				numIncorrect++;
/*			printf("Predicting context %d, correct one is %d\n", predCluster, clusterTrajectory[i]);*/
	
			if ((i-train_size) % 1000 == 0) {
				printf("%d(%f) ", i-train_size, (float) numIncorrect / (numCorrect+numIncorrect));
				fflush(stdout);
			}
		}
		printf("Loss of HMM predictor (%d hidden units): %f\n", numHidden, (float) numIncorrect / (numCorrect+numIncorrect));
		delete hmm;
	}

	printf("\n");
#endif
	
// HACK HACK HACK HACK
if (argc == 2) {		
#if 0	
	printf("-------------------------------------------------------------------\n");
	printf("Test 3: Single-Step from %s (%d samples), discarding duration information\n", argv[1], clusterTrajectory.size());
	PredWrapper pred3(new ActiveLezi(emptyparams));
	PredWrapper simple3_2(new AveragePredictor(windowparams2)),
		simple3_3(new AveragePredictor(windowparams3)),
		simple3_5(new AveragePredictor(windowparams5)),
		simple3_7(new AveragePredictor(windowparams7));

	unsigned int lastCluster = numeric_limits<int>::max(), numChanges3=0;
	for (unsigned int i=0; i<clusterTrajectory.size(); i++) {
		unsigned int curCluster = clusterTrajectory[i];
		if (curCluster != lastCluster) {
			numChanges3++;
			lastCluster = curCluster;
			pred3.predictAndUpdate(curCluster);
			simple3_2.predictAndUpdate(curCluster);
			simple3_3.predictAndUpdate(curCluster);
			simple3_5.predictAndUpdate(curCluster);
			simple3_7.predictAndUpdate(curCluster);
		}
		
/*		if (i % 1000 == 0) {
			printf("%d(%d/%f) ", i, ((ActiveLezi*) pred3.getPred())->getMaxDepth(), pred3.getLoss());
			fflush(stdout);
		}*/
	}
//	printf("\n");

	printf("Compressed time series has %d samples\n", numChanges3);
	printf("Loss of ALZ predictor: %f\n", pred3.getLoss());
	printf("Loss of averaging (2) predictor: %f\n", simple3_2.getLoss());
	printf("Loss of averaging (3) predictor: %f\n", simple3_3.getLoss());
	printf("Loss of averaging (5) predictor: %f\n", simple3_5.getLoss());
	printf("Loss of averaging (7) predictor: %f\n", simple3_7.getLoss());

	printf("\n");

#endif
// HACK HACK HACK
}	
	printf("-------------------------------------------------------------------\n");
	printf("Test 4: Multi-Step: 50 steps, starting at 150, resolution reduced by factor 100\n");
	const unsigned int multiStepOrder = 50;
#if 0
	PredWrapper pred4(new ActiveLezi(emptyparams));
	PredWrapper pred4_dura(new ActiveLezi(emptyparams));
	PredWrapper simple4_2(new AveragePredictor(windowparams2)),
		simple4_3(new AveragePredictor(windowparams3)),
		simple4_5(new AveragePredictor(windowparams5)),
		simple4_7(new AveragePredictor(windowparams7));
	PredWrapper dura4(new SingleStepDurationPredictor(emptyparams));
#endif	
	// use 18 hidden units
	unsigned int numHiddenHMM4 = 18;
	// and a window size of 30
	unsigned int windowSize4 = 30;	
	HMM hmm4(numHiddenHMM4, maxClusterID+1);
if (argc > 2) numHiddenHMM4 = atoi(argv[2]);	
if (argc > 3) windowSize4 = atoi(argv[3]);	
	HMM hmm4_fulltraining(numHiddenHMM4, maxClusterID+1);

	unsigned int lastCluster = numeric_limits<int>::max();
	unsigned int trainingSteps4=0, numChanges4=0;
	vector<unsigned int> clusterTrajectory4;
	printf("Compressed (training) trajectory is: ");
	for (unsigned int i=0; i<15000; i+=100) {
		printf("%d ", clusterTrajectory[i]);
		clusterTrajectory4.push_back(clusterTrajectory[i]);
#if 0	
		pred4.pred->addContext(clusterTrajectory[i], 0);
		simple4_2.pred->addContext(clusterTrajectory[i], 0);
		simple4_3.pred->addContext(clusterTrajectory[i], 0);
		simple4_5.pred->addContext(clusterTrajectory[i], 0);
		simple4_7.pred->addContext(clusterTrajectory[i], 0);
		dura4.pred->addContext(clusterTrajectory[i], 0);
		trainingSteps4++;
	
		unsigned int curCluster = clusterTrajectory[i];
		if (curCluster != lastCluster) {
			lastCluster = curCluster;
			pred4_dura.pred->addContext(clusterTrajectory[i], 0);
			numChanges4++;
		}
#endif	
	}
#if 0	
	printf("\n%d changes in %d steps\n", numChanges4, trainingSteps4);
	printf("Training HMM with %d hidden units\n", numHiddenHMM4);
	hmm4.addContextTrajectory(clusterTrajectory4, 0, 150);
	printf("Training done\n");
#endif	
	printf("Further trajectory is: ");
	for (unsigned int i=15000; i<clusterTrajectory.size(); i+=100) {
		printf("%d ", clusterTrajectory[i]);
		clusterTrajectory4.push_back(clusterTrajectory[i]);
	}
	printf("\n");
	printf("Training reference HMM with %d hidden units on the complete training set (%d samples)\n", numHiddenHMM4, clusterTrajectory4.size());
	// now trying a sliding window training....
	// use the same length of sequences for training as we will then use for prediction (windowSize4) !
	// this is also a heuristic: use an overlap of 10 in the hops
	unsigned int trainingWindowHops4=windowSize4/10;
	if (trainingWindowHops4 < 1) trainingWindowHops4=1;
	hmm4_fulltraining.addContextTrajectory(clusterTrajectory4, 0, clusterTrajectory4.size(), windowSize4, trainingWindowHops4);
	printf("Training done\n");
#if 0	
	contexttrajectory traj_alz4 = pred4.pred->getContextTrajectory(0, multiStepOrder);
	unsigned int totalPreds_alz4=0, correctPreds_alz4=0;
	for (unsigned int i=0; i<traj_alz4.size(); i++) {
		totalPreds_alz4++;
		if (traj_alz4[i] == clusterTrajectory[15000+(i+1)*100]) {
			correctPreds_alz4++;
		}
	}
	contexttrajectory traj_simple4_2 = simple4_2.pred->getContextTrajectory(0, multiStepOrder);
	unsigned int totalPreds_simple4_2=0, correctPreds_simple4_2=0;
	for (unsigned int i=0; i<traj_simple4_2.size(); i++) {
		totalPreds_simple4_2++;
		if (traj_simple4_2[i] == clusterTrajectory[15000+(i+1)*100]) {
			correctPreds_simple4_2++;
		}
	}
	contexttrajectory traj_simple4_3 = simple4_3.pred->getContextTrajectory(0, multiStepOrder);
	unsigned int totalPreds_simple4_3=0, correctPreds_simple4_3=0;
	for (unsigned int i=0; i<traj_simple4_3.size(); i++) {
		totalPreds_simple4_3++;
		if (traj_simple4_3[i] == clusterTrajectory[15000+(i+1)*100]) {
			correctPreds_simple4_3++;
		}
	}
	contexttrajectory traj_simple4_5 = simple4_5.pred->getContextTrajectory(0, multiStepOrder);
	unsigned int totalPreds_simple4_5=0, correctPreds_simple4_5=0;
	for (unsigned int i=0; i<traj_simple4_5.size(); i++) {
		totalPreds_simple4_5++;
		if (traj_simple4_5[i] == clusterTrajectory[15000+(i+1)*100]) {
			correctPreds_simple4_5++;
		}
	}
	contexttrajectory traj_simple4_7 = simple4_7.pred->getContextTrajectory(0, multiStepOrder);
	unsigned int totalPreds_simple4_7=0, correctPreds_simple4_7=0;
	for (unsigned int i=0; i<traj_simple4_7.size(); i++) {
		totalPreds_simple4_7++;
		if (traj_simple4_7[i] == clusterTrajectory[15000+(i+1)*100]) {
			correctPreds_simple4_7++;
		}
	}
	contexttrajectory traj_dura4 = dura4.pred->getContextTrajectory(0, multiStepOrder);
	unsigned int totalPreds_dura4=0, correctPreds_dura4=0;
	for (unsigned int i=0; i<traj_dura4.size(); i++) {
		totalPreds_dura4++;
		if (traj_dura4[i] == clusterTrajectory[15000+(i+1)*100]) {
			correctPreds_dura4++;
		}
	}
	// spezial case: ALZ + dura
	contexttrajectory traj_alz_dura4 = pred4_dura.pred->getContextTrajectory(0, multiStepOrder);
	unsigned int totalPreds_alz_dura4=0, correctPreds_alz_dura4=0, i_uncomp=0, numChangesInPred4=0;
	unsigned long curContext=clusterTrajectory[15000], nextContext;
	// now train the duration predictor with the full test set, to maybe get better results
	for (unsigned int i=15000; i<clusterTrajectory.size(); i+=100) {
		dura4.pred->addContext(clusterTrajectory[i], 0);
	}
	for (unsigned int i=0; i<traj_alz_dura4.size() && i_uncomp<multiStepOrder; i++) {
		// as the next context, take the one that ALZ predicted
		nextContext = traj_alz_dura4[i];
		printf("%d->%d: ", curContext, nextContext);
		// for each of the ALZ predictions, get the duration from the duration predictor
		membershiplist_duration dura = ((SingleStepDurationPredictor*) dura4.pred)->getNextContextsWithDuration(curContext, 0);
		// and check the duration for the context ALZ predicted
		printf("%d; ", dura[nextContext].duration);
		for (unsigned int j=0; j<dura[nextContext].duration && i_uncomp<multiStepOrder; j++) {
			// during this interval, we assume the curContext to be predicted (because the transition predicted by ALZ will only happen _after_ the duration
			// or better not ??
			totalPreds_alz_dura4++;
			if (nextContext == clusterTrajectory[15000+(i_uncomp+1)*100]) {
				correctPreds_alz_dura4++;
			}
			i_uncomp++;
		}
		// now the nextContext becomes the current
		if (curContext == nextContext)
			printf("Warning: ALZ thinks that context %d is repeated - this should'd happen when discarding duration information for this time series !\n", curContext);
		curContext = nextContext;
		numChangesInPred4++;
	}
	printf("\n");
#endif	
	// special case: HMM
	unsigned int totalPreds_hmm4=0, correctPreds_hmm4=0;
	unsigned int totalPreds_hmm4_full=0, correctPreds_hmm4_full=0;
	vector<unsigned int> window;
#if 0	
	// fill the first 30 entries of the window
//	printf("HMM prediction, window is now: ");
	for (unsigned int j=150-windowSize4; j<150; j++) {
//		printf("%d ", clusterTrajectory4[j]);
		window.push_back(clusterTrajectory4[j]);
	}
//	printf("\n");
	assert(window.size() == windowSize4);
	for (unsigned int i=0; i<multiStepOrder; i++) {
		totalPreds_hmm4++;

		unsigned long predCluster = hmm4.getNextContext(window);
		printf("next cluster predicted by HMM: %d --> ", predCluster);
		if (predCluster == clusterTrajectory[15000+(i+1)*100]) {
			printf("Yes\n");
			correctPreds_hmm4++;
		}
		else
			printf("No\n");
		
		// iterative....
		window.erase(window.begin());
		window.push_back(predCluster);
		assert(window.size() == windowSize4);
/*		printf("window is now: ");
		for (unsigned int j=0; j<30; j++) {
			printf("%d ", window[j]);
		}
		printf("\n");*/
	}
#endif	
	// fill the first 30 entries of the window
	window.clear();
	printf("HMM prediction with window size %d\n", windowSize4);
//	printf("HMM prediction, window is now: ");
	for (unsigned int j=150-windowSize4; j<150; j++) {
//		printf("%d ", clusterTrajectory4[j]);
		window.push_back(clusterTrajectory4[j]);
	}
//	printf("\n");
	assert(window.size() == windowSize4);
	unsigned int multiStepHops = 1;
if (argc > 4) multiStepHops = atoi(argv[4]);
	printf("Using %d steps in one HMM prediction\n", multiStepHops);
	for (unsigned int i=0; i<multiStepOrder; i+=multiStepHops) {
		totalPreds_hmm4_full+=multiStepHops;
		// HACK HACK HACK
		if (totalPreds_hmm4_full > multiStepOrder)
			totalPreds_hmm4_full = multiStepOrder;

		if (multiStepHops == 1) {
			// use explicitly the old behavior, to be sure with reproducibility
			unsigned long predCluster = hmm4_fulltraining.getNextContext(window);
			//printf("next cluster predicted by HMM: %ld --> ", predCluster);
			if (predCluster == clusterTrajectory[15000+(i+1)*100]) {
				//printf("Yes\n");
				correctPreds_hmm4_full++;
			}
			/*else
				printf("No\n");*/
			// iterative....
			window.erase(window.begin());
			window.push_back(predCluster);
			assert(window.size() == windowSize4);
		}
		else {
			vector<unsigned long> predClusters = hmm4_fulltraining.getNextContextsMultiStep(window, multiStepHops);
			for (unsigned int j=0; j<multiStepHops; j++) {
				if (i+j<multiStepOrder && predClusters[j] == clusterTrajectory[15000+(i+1+j)*100])
					correctPreds_hmm4_full++;
			}
		}
	}
#if 0			
	assert(totalPreds_alz4 == multiStepOrder);
	assert(totalPreds_simple4_2 == multiStepOrder);
	assert(totalPreds_simple4_3 == multiStepOrder);
	assert(totalPreds_simple4_5 == multiStepOrder);
	assert(totalPreds_simple4_7 == multiStepOrder);
	assert(totalPreds_dura4 == multiStepOrder);
	printf("Loss of ALZ predictor: %f\n", 1 - ((double) correctPreds_alz4) / totalPreds_alz4);
	printf("Loss of averaging (2) predictor: %f\n", 1 - ((double) correctPreds_simple4_2) / totalPreds_simple4_2);
	printf("Loss of averaging (3) predictor: %f\n", 1 - ((double) correctPreds_simple4_3) / totalPreds_simple4_3);
	printf("Loss of averaging (5) predictor: %f\n", 1 - ((double) correctPreds_simple4_5) / totalPreds_simple4_5);
	printf("Loss of averaging (7) predictor: %f\n", 1 - ((double) correctPreds_simple4_7) / totalPreds_simple4_7);
	printf("Loss of single step duration predictor: %f\n", 1 - ((double) correctPreds_dura4) / totalPreds_dura4);
	printf("Loss of ALZ/duration predictor combination (%d steps with %d changes): %f\n", totalPreds_alz_dura4, numChangesInPred4, 1 - ((double) correctPreds_alz_dura4) / totalPreds_alz_dura4);
	printf("Loss of HMM predictor: %f\n", 1 - ((double) correctPreds_hmm4) / totalPreds_hmm4);
#endif	
	printf("Loss of HMM predictor with full training: %f\n", 1 - ((double) correctPreds_hmm4_full) / totalPreds_hmm4_full);

	printf("\n");

#if 0
	////////////////// Test multi-step prediction ///////////////
	// this is going to be slow like hell....
	for (unsigned int multiStepOrder = 100; multiStepOrder<=5000; multiStepOrder+=100) {
		printf("-------------------------------------------------------------------\n");
		printf("Test 5: Full Multi-Step: %d\n", multiStepOrder);
	//	const unsigned int multiStepOrder = 30;

		PredWrapper pred4(new ActiveLezi(emptyparams));
		PredWrapper simple4_2(new AveragePredictor(windowparams2)),
			simple5_3(new AveragePredictor(windowparams3)),
			simple5_5(new AveragePredictor(windowparams5)),
			simple5_7(new AveragePredictor(windowparams7));
		PredWrapper dura4(new SingleStepDurationPredictor(emptyparams));

		for (unsigned int i=0; i<clusterTrajectory.size() - multiStepOrder; i++) {
			pred5.predictAndUpdateMultiStep(&clusterTrajectory, i, multiStepOrder);
			simple5_2.predictAndUpdateMultiStep(&clusterTrajectory, i, multiStepOrder);
			simple5_3.predictAndUpdateMultiStep(&clusterTrajectory, i, multiStepOrder);
			simple5_5.predictAndUpdateMultiStep(&clusterTrajectory, i, multiStepOrder);
			simple5_7.predictAndUpdateMultiStep(&clusterTrajectory, i, multiStepOrder);
			dura5.predictAndUpdateMultiStep(&clusterTrajectory, i, multiStepOrder);

			if (i % 1000 == 0) {
				printf("%d(%d/%f) ", i, ((ActiveLezi*) pred5.getPred())->getMaxDepth(), pred5.getAccuracy());
				fflush(stdout);
			}
		}
		printf("\n");

		printf("Loss of ALZ predictor: %f\n", pred5.getLoss());
		printf("Loss of averaging (2) predictor: %f\n", simple5_2.getLoss());
		printf("Loss of averaging (3) predictor: %f\n", simple5_3.getLoss());
		printf("Loss of averaging (5) predictor: %f\n", simple5_5.getLoss());
		printf("Loss of averaging (7) predictor: %f\n", simple5_7.getLoss());
		printf("Loss of single step duration predictor: %f\n", dura5.getLoss());

		printf("\n");
	}
#endif
}
