#include "intelligencefactory.h"
#include "intelligence_h.h"
#include "intelligence.h"

CIntelligence *CIntelligenceFactory::intelligence = NULL;

CIntelligenceFactory::CIntelligenceFactory()
{
	refCount = 0;
	lockCount = 0;

	AddRef();
}

CIntelligenceFactory::~CIntelligenceFactory(void)
{
	delete intelligence;
}

STDMETHODIMP CIntelligenceFactory::Register()
{
	return reg();
}

STDMETHODIMP CIntelligenceFactory::Unregister()
{
	return unreg();
}

STDMETHODIMP_(BOOL) CIntelligenceFactory::StartServer()
{
	CIntelligenceFactory *factory = new CIntelligenceFactory();

	if (FAILED(::CoRegisterClassObject(	IID_IIntelligence, 
										factory, 
										CLSCTX_SERVER,
										REGCLS_MULTIPLEUSE,
										&regToken))) {
		return FALSE;
	} else {
		factory->Release();

		return TRUE;
	}
}

STDMETHODIMP_(VOID) CIntelligenceFactory::StopServer()
{
	if (regToken) {
		::CoRevokeClassObject(regToken);
		Release();
	}
}

STDMETHODIMP CIntelligenceFactory::QueryInterface(REFIID riid, LPVOID *ppv)
{
	if (IsBadWritePtr(ppv, sizeof(LPVOID))) {
        return E_INVALIDARG;
	}

    *ppv = NULL;

	if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory)) {
		*ppv = (LPVOID)this;
		AddRef( ) ;
		return NOERROR;
	} else {
		*ppv = NULL ;
		return ResultFromScode(E_NOINTERFACE);
	}
}

STDMETHODIMP_(ULONG) CIntelligenceFactory::AddRef(void)
{
	InterlockedIncrement((LPLONG) &refCount);

	return refCount;
}

STDMETHODIMP_(ULONG) CIntelligenceFactory::Release(void)
{
	long count = InterlockedDecrement((LPLONG) &refCount);

	if (count == 0 && lockCount == 0) {
		delete this;
	}

	return count;
}

STDMETHODIMP CIntelligenceFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, LPVOID* ppvObj)
{
	if (pUnkOuter != NULL) {
		*ppvObj = NULL;
		return ResultFromScode (CLASS_E_NOAGGREGATION);
	}

   	if (intelligence->QueryInterface (riid, ppvObj) == NOERROR) {
		return NOERROR;
	} else {
		return E_NOINTERFACE;
	}
}

STDMETHODIMP CIntelligenceFactory::LockServer(BOOL bLock)
{
	if (bLock == TRUE) {
		InterlockedIncrement((LPLONG) &lockCount);
	} else {
		long count = InterlockedDecrement((LPLONG) &lockCount);

		if (count == 0 && refCount == 0) {
			delete this;
		}
	}

	return NOERROR;
}
