/**
 * @file
 * @brief Windows NT service
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-21 23:58:58 +0100 (Sam, 21 Feb 2004) $
 * $Revision: 628 $
 *
 * @since    Tue Aug 8 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: servicehost.cpp 628 2004-02-21 22:58:58Z harald $
 */

#include "intelligencefactory.h"
#include "servicehost.h"

bool ServiceHost::keepRunning;

Service* ServiceHost::getService()
{
	if (This == NULL) {
		This = new ServiceHost("intelligence", "Proactive Intelligence Service");
	}

	return This;
}

bool ServiceHost::onInstall()
{
	return !FAILED(CIntelligenceFactory::reg());
}

bool ServiceHost::onUninstall()
{
	return !FAILED(CIntelligenceFactory::unreg());
}

bool ServiceHost::onInit()
{
	this->keepRunning = true;
	return true;
}

void ServiceHost::onStop()
{
	this->keepRunning = false;
}

void ServiceHost::run()
{
	CoInitializeEx(0, COINIT_MULTITHREADED);

#ifdef _DEBUG
	DebugBreak();
#endif

	CIntelligenceFactory::intelligence = new CIntelligence();
	CIntelligenceFactory *factory = new CIntelligenceFactory();

	factory->StartServer();

	while (CIntelligenceFactory::intelligence->wait(keepRunning)) {
		CIntelligenceFactory::intelligence->nextSample();
	}

	factory->StopServer();

	CoUninitialize();
}
