#include "Features/Time/TimeFeature.h"
#include "Features/Numerical.h"
#include "Features/AbstractString.h"
#include "Features/Wlan/Wlan.h"

#include "ConfigReader/ConfigReader.h"
#include "FeatureContainer/ReplayFeatureContainer.h"

#ifdef _WINDOWS
#define PATH "\\"
#endif

#ifdef _LINUX
#define PATH "./"
#endif

#ifdef _WIN32_WCE
#define PATH "\\"
#endif

void convert_to_binary_dimensions(ReplayFeatureContainer* fc) {
	fc->resetFile();
	bool hasSamples = true, headerPrinted = false;
	int num=0;

	FILE* out = fopen("samples.txt", "w");
	assert(out != NULL);

	while (hasSamples) {
		try {
			num++;
			if (num % 1000 == 0) {
				plog(".");
			}
			if (num % 10000 == 0) {
				plog(" %i\n", num);
			}
			// circumvent disk serialization
			fc->nextSample();
			const featurevector *sample = fc->getSampleVector();
			for (featurevector::const_iterator s = sample->begin(); s != sample->end(); s++) {
				if (*s == NULL) {
					if (!headerPrinted)
						printf("Unkown;");
					fprintf(out, "0;");
				}
				else {
					if (!headerPrinted)
						printf("%s;", (*s)->getName().c_str());

					if ((*s)->getName() == "AbstractStringFeature" ||
						(*s)->getName() == "ActiveWindow.ActiveWindow" ||
						(*s)->getName() == "GSM.CellID" ||
						(*s)->getName() == "Wlan.ActiveEssid" ||
						(*s)->getName() == "Wlan.ActiveMacAddress") {
						// handle as a string
						AbstractStringFeature* t = (AbstractStringFeature*) *s;
						for (unsigned int i=0; i<t->getCode()->size(); i++) {
							if (t->getCodeVal() == i)
								fprintf(out, "1;");
							else
								fprintf(out, "0;");
						}
					}
					else if ((*s)->getName() == "AbstractStringListFeature" ||
						(*s)->getName() == "Bluetooth.Peers" ||
						(*s)->getName() == "Wlan.Peers") {
						AbstractStringListFeature* l = (AbstractStringListFeature*) *s;
						// handle as string list
						for (unsigned int i=0; i<l->getCode()->size(); i++) {
							if ((l->getListVal())[i])
								fprintf(out, "1;");
							else
								fprintf(out, "0;");
						}
					}
					else if ((*s)->getName() == "Wlan.ActiveMode") {
						// special handling
						WlanActiveModeFeature* m = (WlanActiveModeFeature*) *s;
						for (int i=0; i<WlanActiveModeFeature::Last; i++) {
							if (m->getVal() == i)
								fprintf(out, "1;");
							else
								fprintf(out, "0;");
						}
					}
					else if ((*s)->getName() == "NumericalDiscreteFeature" ||
						(*s)->getName() == "Audio.Peaks" ||
						(*s)->getName() == "Wlan.NumPeers") {
						// print normalized
						NumericalDiscreteFeature* n = (NumericalDiscreteFeature*) *s;
						long range = (n->getMax() != n->getMin()) ? (n->getMax() - n->getMin()) : 1;
						fprintf(out, "%f;", ((double) n->getVal() - n->getMin()) / range);
					}
					else if ((*s)->getName() == "NumericalContinuousFeature" ||
						(*s)->getName() == "Audio.Mean" ||
						(*s)->getName() == "Wlan.ActiveSignalLevel" ||
						(*s)->getName().find("Audio.Band") == 0) {
						// print normalized
						NumericalContinuousFeature* n = (NumericalContinuousFeature*) *s;
						double range = (n->getMax() != n->getMin()) ? (n->getMax() - n->getMin()) : 1;
						fprintf(out, "%f;", (n->getVal() - n->getMin()) / range);
					}
					else if ((*s)->getName().find("Time") == 0) {
						// print normalized
						TimeFeature* t = (TimeFeature*) *s;
						time_t range = (t->getMax() != t->getMin()) ? (t->getMax() - t->getMin()) : 1;
						fprintf(out, "%f;", ((double) t->getVal() - t->getMin()) / range);
					}
					else {
						// just print
						fprintf(out, "%s;", (*s)->serialize().c_str());
					}
				}
			}
			fprintf(out, "\n");
			if (!headerPrinted) {
				printf("\n");
				headerPrinted = true;
			}
		} catch(...) {
			hasSamples = false;
		}
	}
	plog("Processed %i samples\n", num);
	fclose(out);
}


int main(int argc, char* argv[])
{
	(void) argc;
	(void) argv;

	open_log(PATH"debug_logfile.txt");

	plog("Reading Configuration\n");

	ConfigReader *r = new ConfigReader(PATH"config.xml");
	map<string, string> globals = (*r->getGlobals());

	assert(globals["persist"] != "");
	assert(globals["features_log"] != "");

	plog("Creating ReplayFeatureContainer\n");
	ReplayFeatureContainer* fc = new ReplayFeatureContainer(r);
	plog("Features log file has a size of %i bytes\n", fc->getFilesize());

	delete r;

	convert_to_binary_dimensions(fc);

	delete fc;

	plog("Terminating main program now\n");
	close_log();

	DUMP_LEAKS();

	return 0;
}
