/**
 * @file
 * @brief Reverse Tunnel Client Service
 *
 * @version $Revision: 629 $
 * @date $Date: 2004-02-22 00:01:47 +0100 (Son, 22 Feb 2004) $
 * @author Harald Radi <h.radi@nme.at>
 */

/* $Id: Service.h 629 2004-02-21 23:01:47Z harald $ */

#pragma once

#define WINDOWS_LEAN_AND_MEAN
#include <windows.h>
#include <string>

#include "messages.h"

#define SERVICE_CONTROL_USER 180

using namespace std;

class Service
{
public:
    virtual ~Service();

	bool parseStandardArgs(int argc, char* argv[]);
    bool startService();
	int exitService();

	string ToString();

protected:
	Service(LPTSTR name, LPTSTR desc);

	void setStatus(DWORD dwState);
    void logEvent(WORD wType, DWORD dwID,
                  const char* pszS1 = NULL,
                  const char* pszS2 = NULL,
                  const char* pszS3 = NULL);

    bool isInstalled();
    bool install();
    bool uninstall();
    bool initialize();

	virtual void run();
	virtual bool onInstall();
	virtual bool onUninstall();
	virtual bool onInit();
    virtual void onStop();
    virtual void onInterrogate();
    virtual void onPause();
    virtual void onContinue();
    virtual void onShutdown();
    virtual bool onUserControl(DWORD dwOpcode);

	virtual void printVersion() {};
	virtual void execConsole(int l) { run(); }
    
	static void DebugMsg(const char* pszFormat, ...);
    
    bool isRunning;

	// static data
    static Service *This;

private:
	void printHelp();

    // data members
    LPTSTR ServiceName;
    LPTSTR ServiceDesc;
    SERVICE_STATUS_HANDLE ServiceStatus;
    SERVICE_STATUS Status;

    HANDLE EventSource;

    // static member functions
    static void WINAPI ServiceMain(DWORD dwArgc, LPTSTR* lpszArgv);
	static BOOL WINAPI CtrlHandler(DWORD fdwCtrlType);
    static void WINAPI Handler(DWORD dwOpcode);
};
