#include "registry.h"

#ifdef _TYPELIB_DLL_
extern OLECHAR *szDLLModule;
#define TLIB_PATH szDllModule
#else
#define TLIB_PATH szModule
#endif

OLECHAR szService[] = L"intelligence";

BOOL setKeyAndValue(LPCTSTR pszPath, LPCTSTR szSubkey, LPCTSTR szValue);
BOOL SubkeyExists(LPCTSTR pszPath, LPCTSTR szSubkey);
LONG recursiveDeleteKey(HKEY hKeyParent, LPCTSTR szKeyChild);

HRESULT RegisterServer(HMODULE hModule, const GUID& clsid, const GUID& tlbid, LPCTSTR szFriendlyName, LPCTSTR szVerIndProgID, LPCTSTR szProgID)
{
	ITypeLib *pTL;
	TLIBATTR *pTLA;
	LPTSTR szGUID;
	LPTSTR szLIBID;
	OLECHAR szModule[512];
	OLECHAR szClsId[256];
	OLECHAR szAppId[256];
	
	DWORD dwResult = ::GetModuleFileName(hModule, szModule, sizeof(szModule)/sizeof(OLECHAR));

	StringFromCLSID(clsid, &szGUID);

	wcscpy(szClsId, L"CLSID\\");
	wcscat(szClsId, szGUID);

	wcscpy(szAppId, L"APPID\\");
	wcscat(szAppId, szGUID);

	if(LoadTypeLib(TLIB_PATH, &pTL) == S_OK)
	{
		RegisterTypeLib(pTL, TLIB_PATH, NULL);
		pTL->GetLibAttr(&pTLA);
		StringFromCLSID(pTLA->guid, &szLIBID);
		setKeyAndValue(szClsId, L"TypeLib", szLIBID);
		pTL->Release();
	}

#ifdef _OUTPROC_SERVICE_
	setKeyAndValue(szAppId, L"LocalService", szService);
#endif

#if defined(_OUTPROC_SERVER_) || defined(_OUTPROC_SERVICE_)
	setKeyAndValue(szClsId, L"LocalServer32", szModule);
#else
	setKeyAndValue(szClsId, L"InprocServer32", szModule);
#endif

#ifdef _SCRIPTING_SAFE_
	setKeyAndValue(szClsId, L"Implemented Categories\\{7DD95801-9882-11CF-9FA9-00AA006C42C4}", L"");
	setKeyAndValue(szClsId, L"Implemented Categories\\{7DD95802-9882-11CF-9FA9-00AA006C42C4}", L"");
#endif

	setKeyAndValue(szClsId, NULL, szFriendlyName);
	setKeyAndValue(szClsId, L"ProgID", szProgID);
	setKeyAndValue(szClsId, L"VersionIndependentProgID", szVerIndProgID);

	setKeyAndValue(szVerIndProgID, NULL, szFriendlyName);
	setKeyAndValue(szVerIndProgID, L"CLSID", szGUID);
	setKeyAndValue(szVerIndProgID, L"CurVer", szProgID);

	setKeyAndValue(szProgID, NULL, szFriendlyName) ;
	setKeyAndValue(szProgID, L"CLSID", szGUID) ;

	return S_OK ;
}

LONG UnregisterServer(HMODULE hModule, const GUID& clsid, LPCTSTR szVerIndProgID, LPCTSTR szProgID)
{
	ITypeLib *pTL;
	TLIBATTR *pTLA;
	LONG lResult;
	LPTSTR szCLSID;
	OLECHAR szModule[512];
	OLECHAR szClsId[256];
	OLECHAR szAppId[256];

	DWORD dwResult = ::GetModuleFileName(hModule, szModule, sizeof(szModule)/sizeof(OLECHAR));
	LoadTypeLib(szModule, &pTL);
	pTL->GetLibAttr(&pTLA);
	UnRegisterTypeLib(pTLA->guid, pTLA->wMajorVerNum, pTLA->wMinorVerNum, pTLA->lcid, pTLA->syskind);

	StringFromCLSID(clsid, &szCLSID);

	wcscpy(szClsId, L"CLSID\\");
	wcscat(szClsId, szCLSID);

	wcscpy(szAppId, L"APPID\\");
	wcscat(szAppId, szCLSID);

	lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szAppId);
	lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szClsId);
	lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szVerIndProgID);
	lResult = recursiveDeleteKey(HKEY_CLASSES_ROOT, szProgID);

	return S_OK ;
}

LONG recursiveDeleteKey(HKEY hKeyParent, LPCTSTR lpszKeyChild)
{
	HKEY hKeyChild;
	LONG lRes = RegOpenKeyEx(hKeyParent, lpszKeyChild, 0, KEY_ALL_ACCESS, &hKeyChild);
	if (lRes != ERROR_SUCCESS)
	{
		return lRes ;
	}

	FILETIME time ;
	OLECHAR szBuffer[256] ;
	DWORD dwSize = 256 ;

	while (RegEnumKeyEx(hKeyChild, 0, szBuffer, &dwSize, NULL, NULL, NULL, &time) == S_OK)
	{
		lRes = recursiveDeleteKey(hKeyChild, szBuffer);
		if (lRes != ERROR_SUCCESS)
		{
			RegCloseKey(hKeyChild);

			return lRes;
		}
		dwSize = 256;
	}

	RegCloseKey(hKeyChild);

	return RegDeleteKey(hKeyParent, lpszKeyChild);
}

BOOL SubkeyExists(LPCTSTR pszPath, LPCTSTR szSubkey)
{
	HKEY hKey;
	OLECHAR szKeyBuf[80];

	wcscpy(szKeyBuf, pszPath);

	if (szSubkey != NULL)
	{
		wcscat(szKeyBuf, L"\\");
		wcscat(szKeyBuf, szSubkey );
	}

	LONG lResult = ::RegOpenKeyEx(HKEY_CLASSES_ROOT, szKeyBuf, 0, KEY_ALL_ACCESS, &hKey);
	if (lResult == ERROR_SUCCESS)
	{
		RegCloseKey(hKey);
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

BOOL setKeyAndValue(LPCTSTR szKey, LPCTSTR szSubkey, LPCTSTR szValue)
{
	HKEY hKey;
	OLECHAR szKeyBuf[1024];

	wcscpy(szKeyBuf, szKey);

	if (szSubkey != NULL)
	{
		wcscat(szKeyBuf, L"\\");
		wcscat(szKeyBuf, szSubkey );
	}

	if (RegCreateKeyEx(HKEY_CLASSES_ROOT, szKeyBuf, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL) != ERROR_SUCCESS)
	{
		MessageBox (NULL, L"HKEY_CLASSES_ROOT failed", L"", 0);
		return FALSE;
	}

	if (szValue != NULL)
	{
		if(RegSetValue(hKey, NULL, REG_SZ, szValue, wcslen(szValue)+1) != ERROR_SUCCESS)
		{
			return FALSE;
		}
	}

	RegCloseKey(hKey);

	return TRUE;
}