//======================================================================
// record - record to memory (then playback) for Windows CE 
// based upon the portaudio test program patest_record.c
// CE port by Gordon Gidluck
//======================================================================
/*
 * patest_record.c
 * Record input into an array.
 * Save array to a file.
 * Playback recorded data.
 *
 * Author: Phil Burk  http://www.softsynth.com
 *
 * This program uses the PortAudio Portable Audio Library.
 * For more information see: http://www.portaudio.com
 * Copyright (c) 1999-2000 Ross Bencina and Phil Burk
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * Any person wishing to distribute modifications to the Software is
 * requested to send the modifications to the original developer so that
 * they can be incorporated into the canonical version.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <windows.h>                 // For all that Windows stuff
#include <commctrl.h>                // command bar includes
#include <stdio.h>
#include <stdlib.h>
/* #include <math.h> */
 
#include "wince-record.h"                  // Program-specific stuff
#include "portaudio.h"               // portaudio

//----------------------------------------------------------------------
// defines for portAudio
//
/* #define SAMPLE_RATE  (17932) /* Test failure to open with this value. */
#define SAMPLE_RATE  (44100)
#define NUM_SECONDS     (5)

/* Select sample format. */

//#define PA_SAMPLE_TYPE  paFloat32
//typedef float SAMPLE;

#define PA_SAMPLE_TYPE  paInt16
typedef short SAMPLE;

//#define PA_SAMPLE_TYPE  paUInt8
//typedef unsigned char SAMPLE;


//----------------------------------------------------------------------
// struct for recording
//
typedef struct
{
	int          frameIndex;  /* Index into sample array. */
	int          maxFrameIndex;
	int          samplesPerFrame;
	SAMPLE      *recordedSamples;
} paTestData;

/* Prototypes */ 
// if declared they would go here

//----------------------------------------------------------------------
// Global data
//
const TCHAR szAppName[] = TEXT("record");
HINSTANCE hInst;                     // Program instance handle

// Message dispatch table for MainWindowProc
const struct decodeUINT MainMessages[] = {
    WM_CREATE, DoCreateMain,
    WM_PAINT, DoPaintMain,
    WM_HIBERNATE, DoHibernateMain,
    WM_ACTIVATE, DoActivateMain,
    WM_DESTROY, DoDestroyMain,
};

    FILE *pa_stdout;
    char pa_buf[256];
	int pa_len;
	int pa_nbw;

/* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().
*/
static int recordCallback(	void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
				        PaTimestamp outTime, void *userData )
{
	paTestData *data = (paTestData*)userData;
	SAMPLE *rptr = (SAMPLE*)inputBuffer;
	SAMPLE *wptr = &data->recordedSamples[data->frameIndex * data->samplesPerFrame];
	long framesToCalc;
	long i;
	int finished;
	unsigned long framesLeft = data->maxFrameIndex - data->frameIndex;
		
	(void) outputBuffer; /* Prevent unused variable warnings. */
	(void) outTime;
	
	if( framesLeft < framesPerBuffer )
	{
		framesToCalc = framesLeft;
		finished = 1;
	}
	else
	{
		framesToCalc = framesPerBuffer;
		finished = 0;
	}
	if( inputBuffer == NULL )
	{
	   	for( i=0; i<framesToCalc; i++ )
		{
			*wptr++ = 0;		/* left */
			*wptr++ = 0;		/* right */
		}
	}
	else
	{
		for( i=0; i<framesToCalc; i++ )
		{
			*wptr++ = *rptr++;		/* left */
			*wptr++ = *rptr++;		/* right */
		}
	}
	data->frameIndex += framesToCalc;
	return finished;
}

/* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().
*/
static int playCallback(	void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
				        PaTimestamp outTime, void *userData )
{
	paTestData *data = (paTestData*)userData;
	SAMPLE *rptr = &data->recordedSamples[data->frameIndex * data->samplesPerFrame];
	SAMPLE *wptr = (SAMPLE*)outputBuffer;
	unsigned int i;
	int finished;
	unsigned int framesLeft = data->maxFrameIndex - data->frameIndex;
	(void) inputBuffer; /* Prevent unused variable warnings. */
	(void) outTime;
	
	if( framesLeft < framesPerBuffer )
	{
		/* final buffer... */     
		for( i=0; i<framesLeft; i++ )
		{
			*wptr++ = *rptr++;		/* left */
			*wptr++ = *rptr++;		/* right */
		}
		for( ; i<framesPerBuffer; i++ )
		{
			*wptr++ = 0;		/* left */
			*wptr++ = 0;		/* right */
		}
		data->frameIndex += framesLeft;
		finished = 1;
	}
	else
	{
		for( i=0; i<framesPerBuffer; i++ )
		{
			*wptr++ = *rptr++;		/* left */
			*wptr++ = *rptr++;		/* right */
		}
		data->frameIndex += framesPerBuffer;
		finished = 0;
	}
	return finished;
}

//======================================================================
// Program entry point
//
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPWSTR lpCmdLine, int nCmdShow) {
    MSG msg;
    int rc = 0;
    HWND hwndMain;

    // Initialize application.
    rc = InitApp (hInstance);
    if (rc) return rc;

    // Initialize this instance.
    hwndMain = InitInstance (hInstance, lpCmdLine, nCmdShow);
    if (hwndMain == 0) return 0x10;

    // Application message loop
    while (GetMessage (&msg, NULL, 0, 0)) {
        TranslateMessage (&msg);
        DispatchMessage (&msg);
    }
    // Instance cleanup
    return TermInstance (hInstance, msg.wParam);
 }
//---------------------------------------------------------------
// InitApp - Application initialization
//
int InitApp (HINSTANCE hInstance) {
    WNDCLASS wc;

#if defined(WIN32_PLATFORM_PSPC)
    // If Pocket PC, only allow one instance of the application
    HWND hWnd = FindWindow (szAppName, NULL);
    if (hWnd) {
        SetForegroundWindow ((HWND)(((DWORD)hWnd) | 0x01));    
        return -1;
    }
#endif    
    // Register application main window class.
    wc.style = 0;                             // Window style
    wc.lpfnWndProc = MainWndProc;             // Callback function
    wc.cbClsExtra = 0;                        // Extra class data
    wc.cbWndExtra = 0;                        // Extra window data
    wc.hInstance = hInstance;                 // Owner handle
    wc.hIcon = NULL,                          // Application icon
    wc.hCursor = LoadCursor (NULL, IDC_ARROW); // Default cursor
    wc.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);
    wc.lpszMenuName =  NULL;                  // Menu name
    wc.lpszClassName = szAppName;             // Window class name

    if (RegisterClass (&wc) == 0) return 1;
    return 0;
}
//------------------------------------------------------------------
// InitInstance - Instance initialization
//
HWND InitInstance (HINSTANCE hInstance, LPWSTR lpCmdLine, int nCmdShow) {
    HWND hWnd;
    
	HANDLE hThread1;   // for RecordAudioThread
	DWORD dwThread1ID; // for RecordAudioThread

    // Save program instance handle in global variable.
    hInst = hInstance;

    // Create main window.
    hWnd = CreateWindow (szAppName,           // Window class
                         TEXT("Live2496"),       // Window title
                         WS_VISIBLE,          // Style flags
                         CW_USEDEFAULT,       // x position
                         CW_USEDEFAULT,       // y position
                         CW_USEDEFAULT,       // Initial width
                         CW_USEDEFAULT,       // Initial height
                         NULL,                // Parent
                         NULL,                // Menu, must be null
                         hInstance,           // Application instance
                         NULL);               // Pointer to create
                                              // parameters

   if( (pa_stdout = fopen( "\\2496_log.txt", "w+t" )) != NULL )
   {
     pa_len=sprintf(pa_buf,"begin portaudio log...\n");
	 pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout);

   }
   else
   { return 0; }

    // Create the sine wave thread    
	hThread1 = CreateThread (NULL, 0, RecordAudioThread, hWnd, 0, &dwThread1ID);
	if (hThread1)
      CloseHandle (hThread1);
	else { 
		DestroyWindow (hWnd);
		return 0;
	}

	// setting thread priority to highest makes the audio not play???
//	if (hThread1) SetThreadPriority(hThread1,THREAD_PRIORITY_HIGHEST);

    // End of create sine wave thread	
	
	if (!IsWindow (hWnd)) return 0; // Fail if not created

    // Standard show and update calls
    ShowWindow (hWnd, nCmdShow);
    UpdateWindow (hWnd);
    return hWnd;
}
//----------------------------------------------------------------------
// TermInstance - Program cleanup
//
int TermInstance (HINSTANCE hInstance, int nDefRC) {
    fclose(pa_stdout);  // close the log file
    return nDefRC;
}
//======================================================================
// Message handling procedures for main window
//
//----------------------------------------------------------------------
// MainWndProc - Callback function for application window
//
LRESULT CALLBACK MainWndProc (HWND hWnd, UINT wMsg, WPARAM wParam, 
                              LPARAM lParam) {
    INT i;
    //
    // Search message list to see if we need to handle this
    // message.  If in list, call procedure.
    //
    for (i = 0; i < dim(MainMessages); i++) {
        if (wMsg == MainMessages[i].Code)
            return (*MainMessages[i].Fxn)(hWnd, wMsg, wParam, lParam);
    }
    return DefWindowProc (hWnd, wMsg, wParam, lParam);
}
//----------------------------------------------------------------------
// DoCreateMain - Process WM_CREATE message for window.
//
LRESULT DoCreateMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                      LPARAM lParam) {
    HWND hwndCB;

    // Create a command bar.
    hwndCB = CommandBar_Create (hInst, hWnd, IDC_CMDBAR);
     // Add exit button to command bar. 
    CommandBar_AddAdornments (hwndCB, 0, 0);
    return 0;
}
//----------------------------------------------------------------------
// DoPaintMain - Process WM_PAINT message for window.
//
LRESULT DoPaintMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                     LPARAM lParam) {
    PAINTSTRUCT ps;
    RECT rect;
    HDC hdc;

    // Adjust the size of the client rectangle to take into account
    // the command bar height.
    GetClientRect (hWnd, &rect);
    rect.top += CommandBar_Height (GetDlgItem (hWnd, IDC_CMDBAR));
    hdc = BeginPaint (hWnd, &ps); 
    DrawText (hdc, TEXT ("Record 5 seconds of audio.\n"), -1, &rect, DT_WORDBREAK);
    EndPaint (hWnd, &ps);
    return 0;
}
//----------------------------------------------------------------------
// DoHibernateMain - Process WM_HIBERNATE message for window.
//
LRESULT DoHibernateMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                         LPARAM lParam) {

    // If not the active window, nuke the command bar to save memory.
    if (GetActiveWindow() != hWnd)
        CommandBar_Destroy (GetDlgItem (hWnd, IDC_CMDBAR));
    return 0;
}
//----------------------------------------------------------------------
// DoActivateMain - Process WM_ACTIVATE message for window.
//
LRESULT DoActivateMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                        LPARAM lParam) {
    HWND hwndCB;

// If activating and no command bar, create it.
    if ((LOWORD (wParam) != WA_INACTIVE) &&
        (GetDlgItem (hWnd, IDC_CMDBAR) == 0)) {

        // Create a command bar.
        hwndCB = CommandBar_Create (hInst, hWnd, IDC_CMDBAR);

        // Add exit button to command bar.
        CommandBar_AddAdornments (hwndCB, 0, 0);
    }
    return 0;
}
//----------------------------------------------------------------------
// DoDestroyMain - Process WM_DESTROY message for window.
//
LRESULT DoDestroyMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                       LPARAM lParam) {
    PostQuitMessage (0);
    return 0;
}

//----------------------------------------------------------------------
// RecordAudioThread - Process which records/plays audio
//
int RecordAudioThread (PVOID pArg) {

    HWND hWnd;

// insert portaudio stuff here
	PortAudioStream *stream;
	PaError    err;
	paTestData data;
	int        i;
	int        totalFrames;
	int        numSamples;
	int        numBytes;
	SAMPLE     max, average, val;

	hWnd = (HWND)pArg;

    pa_len=sprintf(pa_buf,"patest_record.c\n");
    pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
	
	data.maxFrameIndex = totalFrames = NUM_SECONDS * SAMPLE_RATE; /* Record for a few seconds. */
	data.frameIndex = 0;
	data.samplesPerFrame = 2;
	numSamples = totalFrames * data.samplesPerFrame;

	numBytes = numSamples * sizeof(SAMPLE);
	data.recordedSamples = (SAMPLE *) malloc( numBytes );
	if( data.recordedSamples == NULL )
	{
        pa_len=sprintf(pa_buf,"Could not allocate record array.\n");
        pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
		exit(1);
	}
	for( i=0; i<numSamples; i++ ) data.recordedSamples[i] = 0;

	err = Pa_Initialize();
	if( err != paNoError ) goto error;       
	 
/* Record some audio. -------------------------------------------- */
	err = Pa_OpenStream(
				&stream,
				1,        //Pa_GetDefaultInputDeviceID(),
				data.samplesPerFrame,               /* stereo input */
				PA_SAMPLE_TYPE,
				NULL,
				paNoDevice,
				0,
				PA_SAMPLE_TYPE,
				NULL,
				SAMPLE_RATE,
				1024,            /* frames per buffer */
				0,               /* number of buffers, if zero then use default minimum */
				paClipOff,  /* we won't output out of range samples so don't bother clipping them */
				recordCallback,
				&data );

	if( err != paNoError ) goto error;

	err = Pa_StartStream( stream );
	if( err != paNoError ) goto error;

    pa_len=sprintf(pa_buf,"Now recording!!\n");
    pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);

	while( Pa_StreamActive( stream ) )
	{
		Pa_Sleep(1000);
        pa_len=sprintf(pa_buf,"index = %d\n",data.frameIndex);
        pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
	}

	err = Pa_CloseStream( stream );
	if( err != paNoError ) goto error;

/* Measure maximum peak amplitude. */
	max = 0;
	average = 0;
	for( i=0; i<numSamples; i++ )
	{
		val = data.recordedSamples[i];
		if( val < 0 ) val = -val; /* ABS */
		if( val > max )
		{
			max = val;
		}
		average += val;
	}

    pa_len=sprintf(pa_buf,"sample max amplitude = %d\n",max);
    pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
	average = average / numSamples;

    pa_len=sprintf(pa_buf,"sample average = %d\n",average);
    pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);

/* Write recorded data to a file. */
//#if 0
	{
		FILE  *fid;
		fid = fopen("\\Program Files\\Live2496\\recorded.raw", "wb");
		if( fid == NULL )
		{
		    pa_len=sprintf(pa_buf,"Could not open file.\n");
            pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
		}
		else
		{
			fwrite( data.recordedSamples, data.samplesPerFrame * sizeof(SAMPLE), totalFrames, fid );
			fclose( fid );
		    pa_len=sprintf(pa_buf,"Wrote data to 'recorded.raw'\n");
            pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
		}
	}
//#endif

/* Playback recorded data.  -------------------------------------------- */
	data.frameIndex = 0;
	printf("Begin playback.\n"); fflush(stdout);
	err = Pa_OpenStream(
				&stream,
				paNoDevice,
				0,               /* NO input */
				PA_SAMPLE_TYPE,	
				NULL,
				Pa_GetDefaultOutputDeviceID(),
				data.samplesPerFrame,               /* stereo output */
				PA_SAMPLE_TYPE, 
				NULL,
				SAMPLE_RATE,
				1024,            /* frames per buffer */
				0,               /* number of buffers, if zero then use default minimum */
				paClipOff,       /* we won't output out of range samples so don't bother clipping them */
				playCallback,
				&data );
	if( err != paNoError ) goto error;

	if( stream )
	{
		err = Pa_StartStream( stream );
		if( err != paNoError ) goto error;
	    pa_len=sprintf(pa_buf,"Waiting for playback to finish.\n");
        pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
		while( Pa_StreamActive( stream ) ) Pa_Sleep(100);
		err = Pa_CloseStream( stream );
		if( err != paNoError ) goto error;
	    pa_len=sprintf(pa_buf,"Done.\n");
        pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
	}
	free( data.recordedSamples );

	Pa_Terminate();
	return 0;

error:
	Pa_Terminate();
    pa_len=sprintf(pa_buf,"An error occured while using the portaudio stream\n");
    pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout);
    pa_len=sprintf(pa_buf,"Error number: %d\n",err);
    pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout);
    pa_len=sprintf(pa_buf,"Error message: %s\n",Pa_GetErrorText(err));
    pa_nbw = fwrite(pa_buf,1,pa_len,pa_stdout); fflush(pa_stdout);
	return -1;
}

// end of portAudio stuff



