/**
 * @file
 * @brief Abstract thread function implementation
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-23 12:24:40 +0100 (Mon, 23 Feb 2004) $
 * $Revision: 635 $
 *
 * @since    Wed May 14 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: ThreadPosix.cpp 635 2004-02-23 11:24:40Z rene $
 */

#include "Thread.h"

#include <pthread.h>
#include <unistd.h>

THREAD Thread::runProc(void *userData)
{
	Thread* th = (Thread*) userData;

	th->run();

	return 0;
}

Thread::ThreadHandle::ThreadHandle()
{
	nativeHandle = (void*) new pthread_t;
}

Thread::ThreadHandle::~ThreadHandle()
{
	delete (pthread_t*) nativeHandle;
}

Thread::Lock::Lock()
{
	nativeLock = new pthread_mutex_t;
}

Thread::Lock::~Lock()
{
	delete (pthread_mutex_t*) nativeLock;
}

void Thread::start()
{
	if (!started) {
		pthread_create((pthread_t*) handle.nativeHandle, NULL, runProc, this);
		started = true;
	}
}

void Thread::join()
{
	if (started) {
		pthread_join(*((pthread_t*) handle.nativeHandle), NULL);
	}
}

void Thread::terminate()
{
	if (started) {
		pthread_cancel(*((pthread_t*) handle.nativeHandle));
	}
}

void Thread::sleep(unsigned milliseconds)
{
	usleep(milliseconds*1000);
}

void Thread::Lock::lock()
{
	pthread_mutex_lock((pthread_mutex_t*) nativeLock);
}

void Thread::Lock::unlock()
{
	pthread_mutex_unlock((pthread_mutex_t*) nativeLock);
}

