/**
 * @file
 * @brief Abstract thread function implementation
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Fri Nov 28 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: ThreadSymbian.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "Thread.h"

#include <assert.h>

/** @todo documentation */
#define new new(ELeave)

THREAD Thread::runProc(void *userData)
{
	Thread* th = static_cast<Thread*> (userData);

	th->run();

	return 0;
}

void Thread::Lock::lock()
{
	((RMutex*) nativeLock)->Wait();
}

void Thread::Lock::unlock()
{
	((RMutex*) nativeLock)->Signal();
}

Thread::ThreadHandle::ThreadHandle()
{
	nativeHandle = new TThreadId;
}

Thread::ThreadHandle::~ThreadHandle()
{
	delete nativeHandle;
}

void Thread::start()
{
	if (!started) {
		_LIT(KThreadName, "Proca Thread");
		RThread th;
		if (th.Create(KThreadName, runProc, KDefaultStackSize, 
				NULL, // we share the same heap so that accessing data will work
				(TAny*) this) != KErrNone) {
			th.Close();
			assert(false);
		}
		*((TThreadId*) handle.nativeHandle) = th.Id();
		// this is not done automatically !
		th.Resume();
		th.Close();
		started = true;
	}
}

void Thread::join()
{
//	cout << "Thread::join called for handle " << *((HANDLE*) handle.nativeHandle) << "\n" << flush;
	RThread th;
	th.Open(*((TThreadId*) handle.nativeHandle));
	TExitType exitCode;

	do {
		exitCode = th.ExitType();

		Thread::sleep(100);
//		cout << "Still waiting for termination...\n" << flush;
	} while (exitCode != EExitPending);
//	cout << "Thread has now terminated\n" << flush;
	th.Close();
}

void Thread::terminate()
{
	if (started) {
		RThread th;
		th.Open(*((TThreadId*) handle.nativeHandle));
		th.Terminate(0); // th.Kill(0) would do exactly the same - why do thay have two equal methods ?
		th.Close();
	}
}

void Thread::sleep(unsigned milliseconds)
{
	RTimer timer;
	TRequestStatus timerStatus;
	timer.CreateLocal();
	timer.After(timerStatus, milliseconds*1000);
	User::WaitForRequest(timerStatus); 
	timer.Close();
}

Thread::Lock::Lock()
{
	nativeLock = new RMutex();
	if (((RMutex*) nativeLock)->CreateLocal() != KErrNone) {
		assert(false);
	}
}

Thread::Lock::~Lock()
{
	delete nativeLock;
}