/**
 * @file
 * @brief Abstract thread function implementation
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Tue May 13 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: ThreadWindows.cpp 612 2004-02-12 20:08:39Z harald $
 */

#include "Thread.h"

THREAD Thread::runProc(void *userData)
{
	Thread* th = (Thread*) userData;

	th->run();

	return 0;
}

void Thread::start()
{
	if (!started) {
		*((HANDLE*) handle.nativeHandle) = CreateThread(0, 0, runProc, this, 0, NULL);
		started = true;
	}
}

void Thread::join()
{
//	cout << "Thread::join called for handle " << *((HANDLE*) handle.nativeHandle) << "\n" << flush;
	DWORD exitCode;

	do {
		GetExitCodeThread(*((HANDLE*) handle.nativeHandle), &exitCode);
		Thread::sleep(100);
//		cout << "Still waiting for termination...\n" << flush;
	} while (exitCode == STILL_ACTIVE);
//	cout << "Thread has now terminated\n" << flush;
}

void Thread::terminate()
{
}

void Thread::sleep(unsigned milliseconds)
{
	Sleep(milliseconds);
}

Thread::Lock::Lock()
{
	/// @todo leaking 24 bytes
	nativeLock = new CRITICAL_SECTION;
	InitializeCriticalSection((LPCRITICAL_SECTION) nativeLock);
}

Thread::Lock::~Lock()
{
	DeleteCriticalSection((LPCRITICAL_SECTION) nativeLock);
	delete (LPCRITICAL_SECTION) nativeLock;
}

void Thread::Lock::lock()
{
	EnterCriticalSection((LPCRITICAL_SECTION) nativeLock);
}

void Thread::Lock::unlock()
{
	LeaveCriticalSection((LPCRITICAL_SECTION) nativeLock);
}

Thread::ThreadHandle::ThreadHandle()
{
	/// @todo leaking 4 bytes
	nativeHandle = (void*) new HANDLE;
}

Thread::ThreadHandle::~ThreadHandle()
{
	delete (HANDLE*) nativeHandle;
}