/**
 * @file
 * @brief Debug log declaration
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-12 21:08:39 +0100 (Don, 12 Feb 2004) $
 * $Revision: 612 $
 *
 * @since    Sat Aug 30 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: debug.h 612 2004-02-12 20:08:39Z harald $
 */

#ifndef _DEBUG_H
#define _DEBUG_H

#if defined(_DEBUG) || defined(_DEBUG_CONSOLE)
# ifdef _DEBUG_CONSOLE
#  define open_log(path) _open_log(path, true)
# else
#  define open_log(path) _open_log(path, false)
# endif // _DEBUG_CONSOLE
# define close_log() _close_log()
# define dll_init_log(x) _dll_init_log(x)
# define plog _log
#else
/**
 * @name Logging macros
 *
 * Macros encapsulating the logging functions for easier use
 */
//@{
# define open_log(path)
# define close_log()
# define dll_init_log(x)
# define plog _logfoo
//@}
#endif // _DEBUG

/**
 * Initialize the logfile
 *
 * @param path Filename of the logfile or <var>NULL</var> if no
 *             logfile should be written.
 * @param console <var>true</var> if the log should be printed to the
 *                console, <var>false</var> otherwise.
 *
 * @remarks Not every platform provides a console, writing a logfile is
 *          the only possibility of logging information there.
 */
void _open_log(const char* path, bool console);

/**
 * Close the logfile
 */
void _close_log();

/**
 * Wrapper function to convert printf style parameters
 * into va_list style parameters.
 *
 * @param format printf style format string
 * @param ... Additional parameters
 */
void _log(const char *format, ...);

/**
 * dummy function if logging is disabled
 *
 * @see _log
 * @todo gcc supports 'args...' macros, afaik ms doesn't - check, would be a nicer hack
 */
void _logfoo(const char *format, ...);

/**
 * Loads the main logging module into dynamically loaded libraries
 * to ensure that logging is performed synchronously into a single
 * logfile.
 *
 * @param dll Handle to the dynamically loaded library
 */
void _dll_init_log(void *dll);

#endif // _DEBUG_H

