/**
 * @file
 * @brief %Feature helper functions
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-03-12 11:55:21 +0100 (Fre, 12 Mär 2004) $
 * $Revision: 691 $
 *
 * @since    Thu Aug 28 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: featurevector.cpp 691 2004-03-12 10:55:21Z rene $
 */

#include "featurevector.h"

char* strsplit(char *str, const char sep)
{
	static char *p = NULL;
	char *s;

	assert((str != NULL) || (p != NULL));

	if (str != NULL) {
		p = str;
	}

	for (s = p; (*p != sep) && (*p != '\0'); p++);
	if (*p == sep) {
		*p = '\0';
		p++;
	}
	else
		s = NULL;

	return s;
}


string serializeFeatureVector(const featurevector& samples)
{
	string ret;

	for (featurevector::const_iterator iter = samples.begin(); iter != samples.end(); iter++) {
		if (*iter != NULL) {
			char *serialize = strdup((*iter)->serialize().c_str());
			char *ptr, *outstr = serialize;
			for (ptr = outstr; *ptr; ptr++) {
				switch (*ptr) {
					case ';':
					case '\\':
					case '[':
					case ']':
						{
							char esc = *ptr;
							*ptr = '\0';

							ret += outstr;
							ret += '\\';
							ret += esc;

							outstr = ptr + 1;
						}
				}
			}

			ret += outstr;
			ret += ';';

			free(serialize);
		} else {
			ret += "[missing];";
		}
	}

	return ret;
}


size_t unserializeFeatureVector(const featurevector& features, featurevector& samples, char* data)
{
	char* tok = NULL;

	for (featurevector::const_iterator iter = features.begin(); iter != features.end(); iter++) {
		if (iter == features.begin()) {
			tok = strsplit(data, ';');
		} else {
			tok = strsplit(NULL, ';');
		}

		// for a correctly formatted file, tok should never be NULL
		// but you never know
		/// @todo catch "[invalid]" entries in the file and de-escape ";", "\", "[" and "]"
		if (tok == NULL || string(tok) == "[missing]") {
			samples.push_back(NULL);
		} else {
		    // important: clone _before_ unserializing -
		    // unserializing modifies the object and we definitely don't want
		    // to modify the given features !
                        Feature* newf = (*iter)->clone();
			newf->unserialize(tok);
			samples.push_back(newf);
		}
	}
	if (tok != NULL)
		return tok + strlen(tok) - data;
	else
		return 0;
}
