/**
 * @file
 * @brief %Feature helper function declarations
 *
 * &copy; 2003-2004 by Rene Mayrhofer, Harald Radi
 *
 * $Date: 2004-02-23 12:24:40 +0100 (Mon, 23 Feb 2004) $
 * $Revision: 635 $
 *
 * @since    Thu Aug 28 2003
 * @author   Rene Mayrhofer <rene@mayrhofer.eu.org>
 * @author   Harald Radi    <harald.radi@nme.at>
 *
 * <hr><strong>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * </strong><hr>
 *
 * $Id: featurevector.h 635 2004-02-23 11:24:40Z rene $
 */

#include "../Features/Feature.h"

/**
 * A very short helper for replacing the brain-damaged strtok (which can't handle empty tokens)
 *
 * @param str String to be tokenized, <var>NULL</var> on subsequent calls
 * @param sep Separator
 * @return A valid token on success, <var>NULL</var> if the string can't be tokenized
 *
 * @remarks This function alters the string passed as parameter str.
 */
char* strsplit(char *str, const char sep);

/**
 * These are just two helper functions for serializing and deserializing a whole feature vector
 * just returns, for a given feature vector, the serialized, correctly escaped form with ';' as delimiter
 *
 * @relatesalso LoggingFeatureContainer
 * @relatesalso ReplayFeatureContainer
 */
string serializeFeatureVector(const featurevector& samples);

/** parameters: features must contain all _features_ that will be unserialized from data, while samples must be
 * empty as this method will fill it with newly created Feature objects of the correct type (by cloning them from
 * the objects in features)
 * beware that data will be changed by this method !
 * returns the number of characters processed from dat
 *
 * @relatesalso LoggingFeatureContainer
 * @relatesalso ReplayFeatureContainer
 */
size_t unserializeFeatureVector(const featurevector& features, featurevector& samples, char* data);

